@@ -11,6 +11,7 @@ from Orange.widgets.settings import Setting
 from Orange.data import Table
 from Orange.data.sql.table import SqlTable
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Msg, OWWidget, Input, Output
 from Orange.util import Reprable
 
@@ -22,7 +23,7 @@ class OWDataSampler(OWWidget):
     icon = "icons/DataSampler.svg"
     priority = 100
     category = "Data"
-    keywords = ["random"]
+    keywords = ["shuju", 'caiyang', 'chouyang', 'yangben', 'shujucaiyang', 'shujuchouyang']
 
     _MAX_SAMPLE_SIZE = 2 ** 31 - 1
 
@@ -54,18 +55,33 @@ class OWDataSampler(OWWidget):
     number_of_folds = Setting(10)
     selectedFold = Setting(1)
 
+    # Older versions of the widget had swapped outputs for cross validation
+    # Migrations set this to True for compability with older workflows
+    compatibility_mode = Setting(False, schema_only=True)
+
+    settings_version = 2
+
+    class Information(OWWidget.Information):
+        compatibility_mode = Msg(
+            "Compatibility mode\n"
+            "New versions of widget have swapped outputs for cross validation"
+        )
+
     class Warning(OWWidget.Warning):
         could_not_stratify = Msg("Stratification failed\n{}")
         bigger_sample = Msg('Sample is bigger than input')
 
     class Error(OWWidget.Error):
-        too_many_folds = Msg("Number of folds exceeds data size")
+        too_many_folds = Msg("Number of subsets exceeds data size")
         sample_larger_than_data = Msg("Sample can't be larger than data")
         not_enough_to_stratify = Msg("Data is too small to stratify")
         no_data = Msg("Dataset is empty")
 
     def __init__(self):
         super().__init__()
+        if self.compatibility_mode:
+            self.Information.compatibility_mode()
+
         self.data = None
         self.indices = None
         self.sampled_instances = self.remaining_instances = None
@@ -109,7 +125,7 @@ class OWDataSampler(OWWidget):
             labelAlignment=Qt.AlignLeft,
             fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow)
         ibox = gui.indentedBox(sampling, addSpace=True, orientation=form)
-        form.addRow("折叠次数(Number of folds):",
+        form.addRow("子集数量:",
                     gui.spin(
                         ibox, self, "number_of_folds", 2, 100,
                         addToLayout=False,
@@ -117,7 +133,8 @@ class OWDataSampler(OWWidget):
         self.selected_fold_spin = gui.spin(
             ibox, self, "selectedFold", 1, self.number_of_folds,
             addToLayout=False, callback=self.fold_changed)
-        form.addRow("选定的折叠(Selected fold):", self.selected_fold_spin)
+        form.addRow("未使用的子集:" if not self.compatibility_mode
+                    else "选中的子集:", self.selected_fold_spin)
 
         gui.appendRadioButton(sampling, "Bootstrap")
 
@@ -179,14 +196,14 @@ class OWDataSampler(OWWidget):
             self.cb_seed.setVisible(not sql)
             self.cb_stratify.setVisible(not sql)
             self.cb_sql_dl.setVisible(sql)
-            self.info.set_input_summary(str(len(dataset)))
+            self.info.set_input_summary(dataset.approx_len(),
+                                        format_summary_details(dataset))
 
             if not sql:
                 self._update_sample_max_size()
                 self.updateindices()
         else:
             self.info.set_input_summary(self.info.NoInput)
-            self.info.set_output_summary(self.info.NoInput)
             self.indices = None
             self.clear_messages()
         self.commit()
@@ -223,13 +240,20 @@ class OWDataSampler(OWWidget):
                     self.FixedProportion, self.FixedSize, self.Bootstrap):
                 remaining, sample = self.indices
             elif self.sampling_type == self.CrossValidation:
-                remaining, sample = self.indices[self.selectedFold - 1]
-            self.info.set_output_summary(str(len(sample)))
+                if self.compatibility_mode:
+                    remaining, sample = self.indices[self.selectedFold - 1]
+                else:
+                    sample, remaining = self.indices[self.selectedFold - 1]
 
             sample = self.data[sample]
             other = self.data[remaining]
             self.sampled_instances = len(sample)
             self.remaining_instances = len(other)
+
+        summary = sample.approx_len() if sample else self.info.NoOutput
+        details = format_summary_details(sample) if sample else ""
+        self.info.set_output_summary(summary, details)
+
         self.Outputs.data_sample.send(sample)
         self.Outputs.remaining_data.send(other)
 
@@ -310,9 +334,11 @@ class OWDataSampler(OWWidget):
                 if self.replacement:
                     tpe += ", with replacement"
         elif self.sampling_type == self.CrossValidation:
-            tpe = "Fold {} of {}-fold cross-validation".format(
-                self.selectedFold, self.number_of_folds)
-        else:
+            tpe = f"{self.number_of_folds}-fold cross-validation " \
+                  f"without subset #{self.selectedFold}"
+        elif self.sampling_type == self.Bootstrap:
+            tpe = "Bootstrap"
+        else:  # pragma: no cover
             tpe = "Undefined"  # should not come here at all
         if self.stratify:
             tpe += ", stratified (if possible)"
@@ -327,6 +353,12 @@ class OWDataSampler(OWWidget):
             ]
         self.report_items(items)
 
+    @classmethod
+    def migrate_settings(cls, settings, version):
+        if not version or version < 2 \
+                and settings["sampling_type"] == cls.CrossValidation:
+            settings["compatibility_mode"] = True
+
 
 class SampleFoldIndices(Reprable):
     def __init__(self, folds=10, stratified=False, random_state=None):
@@ -349,12 +381,12 @@ class SampleFoldIndices(Reprable):
     def __call__(self, table):
         if self.stratified and table.domain.has_discrete_class:
             splitter = skl.StratifiedKFold(
-                self.folds, random_state=self.random_state)
+                self.folds, shuffle=True, random_state=self.random_state)
             splitter.get_n_splits(table.X, table.Y)
             ind = splitter.split(table.X, table.Y)
         else:
             splitter = skl.KFold(
-                self.folds, random_state=self.random_state)
+                self.folds, shuffle=True, random_state=self.random_state)
             splitter.get_n_splits(table)
             ind = splitter.split(table)
         return tuple(ind)
