@@ -26,16 +26,19 @@ from AnyQt.QtWidgets import (
     QSizePolicy, QAbstractItemView, QComboBox, QFormLayout, QLineEdit,
     QHBoxLayout, QVBoxLayout, QStackedWidget, QStyledItemDelegate,
     QPushButton, QMenu, QListView, QFrame, QLabel)
-from AnyQt.QtGui import QKeySequence
+from AnyQt.QtGui import QKeySequence, QColor
 from AnyQt.QtCore import Qt, pyqtSignal as Signal, pyqtProperty as Property
+from orangewidget.utils.combobox import ComboBoxSearch
 
 import Orange
+from Orange.data.util import get_unique_names
 from Orange.widgets import gui
 from Orange.widgets.settings import ContextSetting, DomainContextHandler
 from Orange.widgets.utils import itemmodels, vartype
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets import report
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import OWWidget, Msg, Input, Output
 
 FeatureDescriptor = \
@@ -53,6 +56,7 @@ DiscreteDescriptor = \
 
 StringDescriptor = namedtuple("StringDescriptor", ["name", "expression"])
 
+#warningIcon = gui.createAttributePixmap('!', QColor((202, 0, 32)))
 
 def make_variable(descriptor, compute_value):
     if isinstance(descriptor, ContinuousDescriptor):
@@ -68,7 +72,6 @@ def make_variable(descriptor, compute_value):
         return Orange.data.DiscreteVariable(
             descriptor.name,
             values=descriptor.values,
-            ordered=descriptor.ordered,
             compute_value=compute_value)
     elif isinstance(descriptor, StringDescriptor):
         return Orange.data.StringVariable(
@@ -126,7 +129,7 @@ class FeatureEditor(QFrame):
 
         self.attrs_model = itemmodels.VariableListModel(
             ["选择特征"], parent=self)
-        self.attributescb = gui.OrangeComboBox(
+        self.attributescb = ComboBoxSearch(
             minimumContentsLength=16,
             sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
             sizePolicy=QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
@@ -142,7 +145,7 @@ class FeatureEditor(QFrame):
             [''],
             [self.FUNCTIONS[func].__doc__ for func in sorted_funcs])
 
-        self.functionscb = gui.OrangeComboBox(
+        self.functionscb = ComboBoxSearch(
             minimumContentsLength=16,
             sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
             sizePolicy=QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
@@ -233,7 +236,7 @@ class ContinuousFeatureEditor(FeatureEditor):
     def editorData(self):
         return ContinuousDescriptor(
             name=self.nameedit.text(),
-            number_of_decimals=3,
+            number_of_decimals=None,
             expression=self.expressionedit.text()
         )
 
@@ -266,7 +269,7 @@ class DiscreteFeatureEditor(FeatureEditor):
         self.valuesedit.textChanged.connect(self._invalidate)
 
         layout = self.layout()
-        label = QLabel(self.tr("Values (optional)"))
+        label = QLabel(self.tr("取值(可选)"))
         label.setToolTip(tooltip)
         layout.addRow(label, self.valuesedit)
 
@@ -362,7 +365,8 @@ class OWFeatureConstructor(OWWidget):
     name = "特征构造器(Feature Constructor)"
     description = "用输入数据集中的现有特征构造新特征。"
     icon = "icons/FeatureConstructor.svg"
-    keywords = []
+    keywords = ['tezheng', 'gouzao', 'tezhenggouzao']
+    category = "Data"
 
     class Inputs:
         data = Input("数据(Data)", Orange.data.Table, replaces=['Data'])
@@ -387,6 +391,10 @@ class OWFeatureConstructor(OWWidget):
         more_values_needed = Msg("Categorical feature {} needs more values.")
         invalid_expressions = Msg("Invalid expressions: {}.")
 
+    class Warning(OWWidget.Warning):
+        renamed_var = Msg("Recently added variable has been renamed, "
+                           "to avoid duplicates.\n")
+
     def __init__(self):
         super().__init__()
         self.data = None
@@ -424,16 +432,8 @@ class OWFeatureConstructor(OWWidget):
             candidates = (fmt.format(i) for i in count(1))
             return next(c for c in candidates if c not in reserved)
 
-        def reserved_names():
-            varnames = []
-            if self.data is not None:
-                varnames = [var.name for var in
-                            self.data.domain.variables + self.data.domain.metas]
-            varnames += [desc.name for desc in self.featuremodel]
-            return set(varnames)
-
         def generate_newname(fmt):
-            return unique_name(fmt, reserved_names())
+            return unique_name(fmt, self.reserved_names())
 
         menu = QMenu(self.addbutton)
         cont = menu.addAction("数值数据")
@@ -482,7 +482,7 @@ class OWFeatureConstructor(OWWidget):
         self.featuremodel = DescriptorModel(parent=self)
 
         self.featureview = QListView(
-            minimumWidth=200,
+            minimumWidth=200, minimumHeight=50,
             sizePolicy=QSizePolicy(QSizePolicy.Minimum,
                                    QSizePolicy.MinimumExpanding)
         )
@@ -493,6 +493,9 @@ class OWFeatureConstructor(OWWidget):
             self._on_selectedVariableChanged
         )
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         layout.addWidget(self.featureview)
 
         box.layout().addLayout(layout, 1)
@@ -525,8 +528,18 @@ class OWFeatureConstructor(OWWidget):
 
     def _on_modified(self):
         if self.currentIndex >= 0:
+            self.Warning.clear()
             editor = self.editorstack.currentWidget()
-            self.featuremodel[self.currentIndex] = editor.editorData()
+            proposed = editor.editorData().name
+            unique = get_unique_names(self.reserved_names(self.currentIndex),
+                                      proposed)
+
+            feature = editor.editorData()
+            if editor.editorData().name != unique:
+                self.Warning.renamed_var()
+                feature = feature.__class__(unique, *feature[1:])
+
+            self.featuremodel[self.currentIndex] = feature
             self.descriptors = list(self.featuremodel)
 
     def setDescriptors(self, descriptors):
@@ -536,6 +549,15 @@ class OWFeatureConstructor(OWWidget):
         self.descriptors = descriptors
         self.featuremodel[:] = list(self.descriptors)
 
+    def reserved_names(self, idx_=None):
+        varnames = []
+        if self.data is not None:
+            varnames = [var.name for var in
+                        self.data.domain.variables + self.data.domain.metas]
+        varnames += [desc.name for idx, desc in enumerate(self.featuremodel)
+                     if idx != idx_]
+        return set(varnames)
+
     @Inputs.data
     @check_sql_input
     def setData(self, data=None):
@@ -544,12 +566,14 @@ class OWFeatureConstructor(OWWidget):
 
         self.data = data
 
+        self.info.set_input_summary(self.info.NoInput)
         if self.data is not None:
             descriptors = list(self.descriptors)
             currindex = self.currentIndex
             self.descriptors = []
             self.currentIndex = -1
             self.openContext(data)
+            self.info.set_input_summary(len(data), format_summary_details(data))
 
             if descriptors != self.descriptors or \
                     self.currentIndex != currindex:
@@ -570,6 +594,7 @@ class OWFeatureConstructor(OWWidget):
         if self.data is not None:
             self.apply()
         else:
+            self.info.set_output_summary(self.info.NoOutput)
             self.Outputs.data.send(None)
 
     def addFeature(self, descriptor):
@@ -658,18 +683,25 @@ class OWFeatureConstructor(OWWidget):
         )
 
         try:
+            for variable in new_variables:
+                variable.compute_value.mask_exceptions = False
             data = self.data.transform(new_domain)
         # user's expression can contain arbitrary errors
         # pylint: disable=broad-except
         except Exception as err:
             report_error(err)
             return
+        finally:
+            for variable in new_variables:
+                variable.compute_value.mask_exceptions = True
+
         disc_attrs_not_ok = self.check_attrs_values(
             [var for var in attrs if var.is_discrete], data)
         if disc_attrs_not_ok:
             self.Error.more_values_needed(disc_attrs_not_ok)
             return
 
+        self.info.set_output_summary(len(data), format_summary_details(data))
         self.Outputs.data.send(data)
 
     def send_report(self):
@@ -1063,15 +1095,23 @@ class FeatureFunc:
         self.func = make_lambda(ast.parse(expression, mode="eval"),
                                 [name for name, _ in args], self.extra_env)
         self.cast = cast
+        self.mask_exceptions = True
 
     def __call__(self, instance, *_):
         if isinstance(instance, Orange.data.Table):
             return [self(inst) for inst in instance]
         else:
-            args = [str(instance[var])
-                    if instance.domain[var].is_string else instance[var]
-                    for _, var in self.args]
-            y = self.func(*args)
+            try:
+                args = [str(instance[var])
+                        if instance.domain[var].is_string else instance[var]
+                        for _, var in self.args]
+                y = self.func(*args)
+            # user's expression can contain arbitrary errors
+            # this also covers missing attributes
+            except:  # pylint: disable=bare-except
+                if not self.mask_exceptions:
+                    raise
+                return np.nan
             if self.cast:
                 y = self.cast(y)
             return y
