@@ -21,7 +21,8 @@ from Orange.widgets.utils.itemmodels import PyListModel
 from Orange.widgets.utils.filedialogs import RecentPathsWComboMixin, \
     open_filename_dialog
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.widget import Output
+from Orange.widgets.utils.state_summary import format_summary_details
+from Orange.widgets.widget import Output, Msg
 
 # Backward compatibility: class RecentPath used to be defined in this module,
 # and it is used in saved (pickled) settings. It must be imported into the
@@ -80,7 +81,7 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
     icon = "icons/File.svg"
     priority = 10
     category = "Data"
-    keywords = ["file", "load", "read", "open"]
+    keywords = ["file", "load", "read", "open", 'wenjian', 'daikai', 'zairu', 'daoru']
 
     class Outputs:
         data = Output("数据(Data)", Table,
@@ -112,7 +113,6 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
     ])
     recent_urls = Setting([])
     source = Setting(LOCAL_FILE)
-    xls_sheet = ContextSetting("")
     sheet_names = Setting({})
     url = Setting("")
 
@@ -121,17 +121,19 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
     domain_editor = SettingProvider(DomainEditor)
 
     class Warning(widget.OWWidget.Warning):
-        file_too_big = widget.Msg("The file is too large to load automatically."
-                                  " Press Reload to load.")
-        load_warning = widget.Msg("Read warning:\n{}")
-        performance_warning = widget.Msg(
+        file_too_big = Msg("The file is too large to load automatically."
+                           " Press Reload to load.")
+        load_warning = Msg("Read warning:\n{}")
+        performance_warning = Msg(
             "Categorical variables with >100 values may decrease performance.")
+        renamed_vars = Msg("Some variables have been renamed "
+                           "to avoid duplicates.\n{}")
 
     class Error(widget.OWWidget.Error):
-        file_not_found = widget.Msg("File not found.")
-        missing_reader = widget.Msg("Missing reader.")
-        sheet_error = widget.Msg("Error listing available sheets.")
-        unknown = widget.Msg("Read error:\n{}")
+        file_not_found = Msg("File not found.")
+        missing_reader = Msg("Missing reader.")
+        sheet_error = Msg("Error listing available sheets.")
+        unknown = Msg("Read error:\n{}")
 
     class NoFileSelected:
         pass
@@ -185,9 +187,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         layout.addWidget(reload_button, 0, 3)
 
         self.sheet_box = gui.hBox(None, addToLayout=False, margin=0)
-        self.sheet_combo = gui.comboBox(None, self, "xls_sheet",
-                                        callback=self.select_sheet,
-                                        sendSelectedValue=True,)
+        self.sheet_combo = QComboBox()
+        self.sheet_combo.activated[str].connect(self.select_sheet)
         self.sheet_combo.setSizePolicy(
             Policy.MinimumExpanding, Policy.Fixed)
         self.sheet_label = QLabel()
@@ -247,6 +248,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         self.editor_model.dataChanged.connect(
             lambda: self.apply_button.setEnabled(True))
 
+        self.info.set_output_summary(self.info.NoOutput)
+
         self.set_file_list()
         # Must not call open_file from within __init__. open_file
         # explicitly re-enters the event loop (by a progress bar)
@@ -331,7 +334,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
             self.data = None
             self.sheet_box.hide()
             self.Outputs.data.send(None)
-            self.infolabel.setText("无数据。")
+            self.infolabel.setText("无数据.")
+            self.info.set_output_summary(self.info.NoOutput)
 
     def _try_load(self):
         # pylint: disable=broad-except
@@ -346,6 +350,7 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
         if self.reader is self.NoFileSelected:
             self.Outputs.data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
             return None
 
         try:
@@ -478,10 +483,13 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
     def apply_domain_edit(self):
         self.Warning.performance_warning.clear()
+        self.Warning.renamed_vars.clear()
         if self.data is None:
             table = None
         else:
-            domain, cols = self.domain_editor.get_domain(self.data.domain, self.data)
+            domain, cols, renamed = \
+                self.domain_editor.get_domain(self.data.domain, self.data,
+                                              deduplicate=True)
             if not (domain.variables or domain.metas):
                 table = None
             elif domain is self.data.domain:
@@ -493,7 +501,12 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
                 table.ids = np.array(self.data.ids)
                 table.attributes = getattr(self.data, 'attributes', {})
                 self._inspect_discrete_variables(domain)
+            if renamed:
+                self.Warning.renamed_vars(f"Renamed: {', '.join(renamed)}")
 
+        summary = len(table) if table else self.info.NoOutput
+        details = format_summary_details(table) if table else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.data.send(table)
         self.apply_button.setEnabled(False)
 
