@@ -21,7 +21,8 @@ from AnyQt.QtCore import pyqtSignal as Signal
 
 import pyqtgraph as pg
 
-import Orange.data
+from Orange.data import ContinuousVariable, DiscreteVariable, Domain, Table
+from Orange.data.util import get_unique_names_duplicates
 
 from Orange.widgets import gui
 from Orange.widgets.settings import Setting
@@ -29,6 +30,7 @@ from Orange.widgets.utils import itemmodels, colorpalettes
 
 from Orange.util import scale, namegen
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import OWWidget, Msg, Input, Output
 
 
@@ -727,26 +729,27 @@ def _icon(name, icon_path="icons/paintdata",
 
 class OWPaintData(OWWidget):
     TOOLS = [
-        ("毛刷(Brush)", "建立多个实例", AirBrushTool, _icon("brush.svg")),
-        ("单个放置(Put)", "放置单个实例", PutInstanceTool, _icon("put.svg")),
-        ("选择(Select)", " 选择并移动实例", SelectTool,
+        ("毛刷", "建立多个实例", AirBrushTool, _icon("brush.svg")),
+        ("单个放置", "放置单个实例", PutInstanceTool, _icon("put.svg")),
+        ("选择", " 选择并移动实例", SelectTool,
          _icon("select-transparent_42px.png")),
-        ("抖动(Jitter)", " 抖动实例", JitterTool, _icon("jitter.svg")),
-        ("磁铁(Magnet)", " 吸引多个实例", MagnetTool, _icon("magnet.svg")),
-        ("清除(Clear)", " 清除绘图", ClearTool, _icon("../../../icons/Dlg_clear.png"))
+        ("抖动", " 抖动实例", JitterTool, _icon("jitter.svg")),
+        ("磁吸", " 吸引多个实例", MagnetTool, _icon("magnet.svg")),
+        ("清除", " 清除绘图", ClearTool, _icon("../../../icons/Dlg_clear.png"))
     ]
 
     name = "绘制数据(Paint Data)"
     description = "通过在平面上绘制数据点来创建数据。"
     icon = "icons/PaintData.svg"
     priority = 60
-    keywords = ["create", "draw"]
+    keywords = ["create", "draw", 'huizhi', 'hua']
+    category = "Data"
 
     class Inputs:
-        data = Input("数据(Data)", Orange.data.Table, replaces=['Data'])
+        data = Input("数据(Data)", Table, replaces=['Data'])
 
     class Outputs:
-        data = Output("数据(Data)", Orange.data.Table, replaces=['Data'])
+        data = Output("数据(Data)", Table, replaces=['Data'])
 
     autocommit = Setting(True)
     table_name = Setting("Painted data")
@@ -766,8 +769,10 @@ class OWPaintData(OWWidget):
 
     class Warning(OWWidget.Warning):
         no_input_variables = Msg("Input data has no variables")
-        continuous_target = Msg("Continuous target value can not be used.")
+        continuous_target = Msg("Numeric target value can not be used.")
         sparse_not_supported = Msg("Sparse data is ignored.")
+        renamed_vars = Msg("Some variables have been renamed "
+                           "to avoid duplicates.\n{}")
 
     class Information(OWWidget.Information):
         use_first_two = \
@@ -799,7 +804,9 @@ class OWPaintData(OWWidget):
         self.class_model.rowsInserted.connect(self._class_count_changed)
         self.class_model.rowsRemoved.connect(self._class_count_changed)
 
-        if not self.data:
+        # if self.data: raises Deprecation warning in older workflows, where
+        # data could be a np.array. This would raise an error in the future.
+        if self.data is None or len(self.data) == 0:
             self.data = []
             self.__buffer = np.zeros((0, 3))
         elif isinstance(self.data, np.ndarray):
@@ -898,7 +905,7 @@ class OWPaintData(OWWidget):
         redo.setShortcut(QKeySequence.Redo)
 
         self.addActions([undo, redo])
-        self.undo_stack.indexChanged.connect(lambda _: self.invalidate())
+        self.undo_stack.indexChanged.connect(self.invalidate)
 
         gui.separator(tBox)
         indBox = gui.indentedBox(tBox, sep=8)
@@ -963,6 +970,9 @@ class OWPaintData(OWWidget):
 
         self.mainArea.layout().addWidget(self.plotview)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         # enable brush tool
         self.toolActions.actions()[0].setChecked(True)
         self.set_current_tool(self.TOOLS[0][2])
@@ -1001,11 +1011,13 @@ class OWPaintData(OWWidget):
                     data = None
                 elif len(data.domain.attributes) > 2:
                     self.Information.use_first_two()
+                self.info.set_input_summary(len(data), format_summary_details(data))
             self.input_data = data
             self.btResetToInput.setDisabled(data is None)
             return bool(data)
 
         if not _check_and_set_data(data):
+            self.info.set_input_summary(self.info.NoInput)
             return
 
         X = np.array([scale(vals) for vals in data.X[:, :2].T]).T
@@ -1059,6 +1071,8 @@ class OWPaintData(OWWidget):
         else:  # set_dimensions already calls _replot, no need to call it again
             self._replot()
 
+        self.commit()
+
     def add_new_class_label(self, undoable=True):
 
         newlabel = next(label for label in namegen('C', 1)
@@ -1259,35 +1273,49 @@ class OWPaintData(OWWidget):
         self.commit()
 
     def commit(self):
+        self.Warning.renamed_vars.clear()
+
         if not self.data:
             self.Outputs.data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
             return
         data = np.array(self.data)
         if self.hasAttr2:
             X, Y = data[:, :2], data[:, 2]
-            attrs = (Orange.data.ContinuousVariable(self.attr1),
-                     Orange.data.ContinuousVariable(self.attr2))
+            proposed = [self.attr1.strip(), self.attr2.strip()]
         else:
             X, Y = data[:, np.newaxis, 0], data[:, 2]
-            attrs = (Orange.data.ContinuousVariable(self.attr1),)
+            proposed = [self.attr1.strip()]
+
         if len(np.unique(Y)) >= 2:
-            domain = Orange.data.Domain(
-                attrs,
-                Orange.data.DiscreteVariable(
-                    "Class", values=list(self.class_model))
+            proposed.append("Class")
+            unique_names, renamed = get_unique_names_duplicates(proposed, True)
+            domain = Domain(
+                (map(ContinuousVariable, unique_names[:-1])),
+                DiscreteVariable(
+                    unique_names[-1], values=tuple(self.class_model))
             )
-            data = Orange.data.Table.from_numpy(domain, X, Y)
+            data = Table.from_numpy(domain, X, Y)
         else:
-            domain = Orange.data.Domain(attrs)
-            data = Orange.data.Table.from_numpy(domain, X)
+            unique_names, renamed = get_unique_names_duplicates(proposed, True)
+            domain = Domain(map(ContinuousVariable, unique_names))
+            data = Table.from_numpy(domain, X)
+
+        if renamed:
+            self.Warning.renamed_vars(", ".join(renamed))
+            self.plot.getAxis("bottom").setLabel(unique_names[0])
+            self.plot.getAxis("left").setLabel(unique_names[1])
+
         data.name = self.table_name
         self.Outputs.data.send(data)
+        self.info.set_output_summary(len(data), format_summary_details(data))
 
     def sizeHint(self):
         sh = super().sizeHint()
         return sh.expandedTo(QSize(570, 690))
 
     def onDeleteWidget(self):
+        self.undo_stack.indexChanged.disconnect(self.invalidate)
         self.plot.clear()
 
     def send_report(self):
