@@ -1,27 +1,24 @@
 import enum
 from collections import OrderedDict
-from itertools import chain
+from datetime import datetime, timezone, timedelta
 
 import numpy as np
 
 from AnyQt.QtWidgets import (
     QWidget, QTableWidget, QHeaderView, QComboBox, QLineEdit, QToolButton,
     QMessageBox, QMenu, QListView, QGridLayout, QPushButton, QSizePolicy,
-    QLabel)
-from AnyQt.QtGui import (
-    QDoubleValidator, QRegExpValidator, QStandardItemModel, QStandardItem,
-    QFontMetrics, QPalette
-)
-from AnyQt.QtCore import Qt, QPoint, QRegExp, QPersistentModelIndex, QLocale
+    QLabel, QHBoxLayout, QDateTimeEdit)
+from AnyQt.QtGui import (QDoubleValidator, QStandardItemModel, QStandardItem,
+                         QFontMetrics, QPalette)
+from AnyQt.QtCore import Qt, QPoint, QPersistentModelIndex, QLocale, \
+    QDateTime, QDate, QTime
 
+from orangewidget.utils.combobox import ComboBoxSearch
+from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.data import (
-    Variable, ContinuousVariable, DiscreteVariable, StringVariable,
-    TimeVariable,
-    Table
-)
+    ContinuousVariable, DiscreteVariable, StringVariable, TimeVariable, Table)
 import Orange.data.filter as data_filter
 from Orange.data.filter import FilterContinuous, FilterString
-from Orange.data.domain import filter_visible
 from Orange.data.sql.table import SqlTable
 from Orange.preprocess import Remove
 from Orange.widgets import widget, gui
@@ -32,12 +29,15 @@ from Orange.widgets.utils import vartype
 from Orange.widgets import report
 from Orange.widgets.widget import Msg
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME,
                                                  ANNOTATED_DATA_SIGNAL_Chinese_NAME)
+from Orange.widgets.utils.state_summary import format_summary_details
 
 
 class SelectRowsContextHandler(DomainContextHandler):
     """Context handler that filters conditions"""
 
+    # pylint: disable=arguments-differ
     def is_valid_item(self, setting, condition, attrs, metas):
         """Return True if condition applies to a variable in given domain."""
         varname, *_ = condition
@@ -50,24 +50,57 @@ class SelectRowsContextHandler(DomainContextHandler):
         encoded = []
         CONTINUOUS = vartype(ContinuousVariable("x"))
         for attr, op, values in value:
-            vtype = context.attributes.get(attr)
-            if vtype == CONTINUOUS and values and isinstance(values[0], str):
-                values = [QLocale().toDouble(v)[0] for v in values]
-            encoded.append((attr, vtype, op, values))
+            if isinstance(attr, str):
+                if OWSelectRows.AllTypes.get(attr) == CONTINUOUS:
+                    values = [QLocale().toDouble(v)[0] for v in values]
+                # None will match the value returned by all_vars.get
+                encoded.append((attr, None, op, values))
+            else:
+                # check for exact match, pylint: disable=unidiomatic-typecheck
+                if type(attr) is ContinuousVariable \
+                        and values and isinstance(values[0], str):
+                    values = [QLocale().toDouble(v)[0] for v in values]
+                elif isinstance(attr, DiscreteVariable):
+                    values = [attr.values[i - 1] if i else "" for i in values]
+                encoded.append((
+                    attr.name,
+                    context.attributes.get(attr.name)
+                    or context.metas.get(attr.name),
+                    op,
+                    values
+                ))
         return encoded
 
-    def decode_setting(self, setting, value, domain=None):
+    # pylint: disable=arguments-differ
+    def decode_setting(self, setting, value, domain=None, *_args):
         value = super().decode_setting(setting, value, domain)
         if setting.name == 'conditions':
+            CONTINUOUS = vartype(ContinuousVariable("x"))
             # Use this after 2022/2/2:
-            # for i, (attr, _, op, values) in enumerate(value):
-            for i, condition in enumerate(value):
-                attr = condition[0]
-                op, values = condition[-2:]
-
-                var = attr in domain and domain[attr]
-                if var and var.is_continuous and not isinstance(var, TimeVariable):
+            # for i, (attr, tpe, op, values) in enumerate(value):
+            #     if tpe is not None:
+            for i, (attr, *tpe, op, values) in enumerate(value):
+                if tpe != [None] \
+                        or not tpe and attr not in OWSelectRows.AllTypes:
+                    attr = domain[attr]
+                # check for exact match, pylint: disable=unidiomatic-typecheck
+                if type(attr) is ContinuousVariable \
+                        or OWSelectRows.AllTypes.get(attr) == CONTINUOUS:
                     values = [QLocale().toString(float(i), 'f') for i in values]
+                elif isinstance(attr, DiscreteVariable):
+                    # After 2022/2/2, use just the expression in else clause
+                    if values and isinstance(values[0], int):
+                        # Backwards compatibility. Reset setting if we detect
+                        # that the number of values decreased. Still broken if
+                        # they're reordered or we don't detect the decrease.
+                        #
+                        # indices start with 1, thus >, not >=
+                        if max(values) > len(attr.values):
+                            values = (0, )
+                    else:
+                        values = tuple(attr.to_val(val) + 1 if val else 0
+                                       for val in values if val in attr.values) \
+                                 or (0, )
                 value[i] = (attr, op, values)
         return value
 
@@ -76,17 +109,37 @@ class SelectRowsContextHandler(DomainContextHandler):
             return self.PERFECT_MATCH
 
         conditions = context.values["conditions"]
-        all_vars = attrs
+        all_vars = attrs.copy()
         all_vars.update(metas)
-        # Use this after 2022/2/2:
-        # if all(all_vars.get(name) == tpe for name, tpe, *_ in conditions):
-        if all(all_vars.get(name) == tpe if len(rest) == 2 else name in all_vars
-               for name, tpe, *rest in conditions):
-            return 0.5
+        matched = [all_vars.get(name) == tpe  # also matches "all (...)" strings
+                   # After 2022/2/2 remove this line:
+                   if len(rest) == 2 else name in all_vars
+                   for name, tpe, *rest in conditions]
+        if any(matched):
+            return 0.5 * sum(matched) / len(matched)
         return self.NO_MATCH
 
+    def filter_value(self, setting, data, domain, attrs, metas):
+        if setting.name != "conditions":
+            super().filter_value(setting, data, domain, attrs, metas)
+            return
+
+        all_vars = attrs.copy()
+        all_vars.update(metas)
+        conditions = data["conditions"]
+        # Use this after 2022/2/2: if any(all_vars.get(name) == tpe:
+        # conditions[:] = [(name, tpe, *rest) for name, tpe, *rest in conditions
+        #                  if all_vars.get(name) == tpe]
+        conditions[:] = [
+            (name, tpe, *rest) for name, tpe, *rest in conditions
+            # all_vars.get(name) == tpe also matches "all (...)" which are
+            # encoded with type `None`
+            if (all_vars.get(name) == tpe if len(rest) == 2
+                else name in all_vars)]
+
 
 class FilterDiscreteType(enum.Enum):
+    # pylint: disable=invalid-name
     Equal = "Equal"
     NotEqual = "NotEqual"
     In = "In"
@@ -107,7 +160,7 @@ class OWSelectRows(widget.OWWidget):
     icon = "icons/SelectRows.svg"
     priority = 100
     category = "Data"
-    keywords = ["filter"]
+    keywords = ["filter", 'xuanze', 'hang', 'xuanzehang']
 
     class Inputs:
         data = Input("数据(Data)", Table, replaces=['Data'])
@@ -138,13 +191,13 @@ class OWSelectRows(widget.OWWidget):
             (FilterContinuous.GreaterEqual, "至少(is at least)"),
             (FilterContinuous.Between, "介于(is between)"),
             (FilterContinuous.Outside, "超出(is outside)"),
-            (FilterContinuous.IsDefined, "已定义(is defined)"),
+            (FilterContinuous.IsDefined, "is defined"),
         ],
         DiscreteVariable: [
             (FilterDiscreteType.Equal, "是(is)"),
             (FilterDiscreteType.NotEqual, "不是(is not)"),
             (FilterDiscreteType.In, "是...中的一个(is one of)"),
-            (FilterDiscreteType.IsDefined, "已定义(is defined)")
+            (FilterDiscreteType.IsDefined, "is defined")
         ],
         StringVariable: [
             (FilterString.Equal, "等于(equals)"),
@@ -158,7 +211,7 @@ class OWSelectRows(widget.OWWidget):
             (FilterString.Contains, "包含(contains)"),
             (FilterString.StartsWith, "以...开始(begins with)"),
             (FilterString.EndsWith, "以...结尾(ends with)"),
-            (FilterString.IsDefined, "已定义(is defined)"),
+            (FilterString.IsDefined, "is defined"),
         ]
     }
 
@@ -190,6 +243,9 @@ class OWSelectRows(widget.OWWidget):
         self.last_output_conditions = None
         self.data = None
         self.data_desc = self.match_desc = self.nonmatch_desc = None
+        self.variable_model = DomainModel(
+            [list(self.AllTypes), DomainModel.Separator,
+             DomainModel.CLASSES, DomainModel.ATTRIBUTES, DomainModel.METAS])
 
         box = gui.vBox(self.controlArea, '条件', stretch=100)
         self.cond_list = QTableWidget(
@@ -206,7 +262,6 @@ class OWSelectRows(widget.OWWidget):
 
         box2 = gui.hBox(box)
         gui.rubber(box2)
-        # TODO: button style
         self.add_button = gui.button(
             box2, self, "添加条件", callback=self.add_row)
         self.add_all_button = gui.button(
@@ -215,17 +270,10 @@ class OWSelectRows(widget.OWWidget):
             box2, self, "删除全部", callback=self.remove_all)
         gui.rubber(box2)
 
-        boxes = gui.widgetBox(self.controlArea, orientation=QGridLayout())
+        boxes = gui.widgetBox(self.controlArea, orientation=QHBoxLayout())
         layout = boxes.layout()
-        layout.setColumnStretch(0, 1)
-        layout.setColumnStretch(1, 1)
 
-        box_data = gui.vBox(boxes, '数据', addToLayout=False)
-        self.data_in_variables = gui.widgetLabel(box_data, " ")
-        self.data_out_rows = gui.widgetLabel(box_data, " ")
-        layout.addWidget(box_data, 0, 0)
-
-        box_setting = gui.vBox(boxes, '清除', addToLayout=False)
+        box_setting = gui.vBox(boxes, addToLayout=False, box=True)
         self.cb_pa = gui.checkBox(
             box_setting, self, "purge_attributes", "删除未使用的特征",
             callback=self.conditions_changed)
@@ -233,14 +281,18 @@ class OWSelectRows(widget.OWWidget):
         self.cb_pc = gui.checkBox(
             box_setting, self, "purge_classes", "删除未使用的分类",
             callback=self.conditions_changed)
-        layout.addWidget(box_setting, 0, 1)
+        layout.addWidget(box_setting, 1)
 
         self.report_button.setFixedWidth(120)
         gui.rubber(self.buttonsArea.layout())
-        layout.addWidget(self.buttonsArea, 1, 0)
+        layout.addWidget(self.buttonsArea)
 
         acbox = gui.auto_send(None, self, "auto_commit")
-        layout.addWidget(acbox, 1, 1)
+        layout.addWidget(acbox, 1)
+        layout.setAlignment(acbox, Qt.AlignBottom)
+
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
 
         self.set_data(None)
         self.resize(600, 400)
@@ -250,21 +302,13 @@ class OWSelectRows(widget.OWWidget):
         row = model.rowCount()
         model.insertRow(row)
 
-        attr_combo = gui.OrangeComboBox(
+        attr_combo = ComboBoxSearch(
             minimumContentsLength=12,
             sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon)
+        attr_combo.setModel(self.variable_model)
         attr_combo.row = row
-        for var in self._visible_variables(self.data.domain):
-            if isinstance(var, Variable):
-                attr_combo.addItem(*gui.attributeItem(var))
-            else:
-                attr_combo.addItem(var)
-        if isinstance(attr, str):
-            attr_combo.setCurrentText(attr)
-        else:
-            attr_combo.setCurrentIndex(
-                attr or
-                len(self.AllTypes) - (attr_combo.count() == len(self.AllTypes)))
+        attr_combo.setCurrentIndex(self.variable_model.indexOf(attr) if attr
+                                   else len(self.AllTypes) + 1)
         self.cond_list.setCellWidget(row, 0, attr_combo)
 
         index = QPersistentModelIndex(model.index(row, 3))
@@ -282,15 +326,6 @@ class OWSelectRows(widget.OWWidget):
 
         self.cond_list.resizeRowToContents(row)
 
-    @classmethod
-    def _visible_variables(cls, domain):
-        """Generate variables in order they should be presented in in combos."""
-        return chain(
-            cls.AllTypes,
-            filter_visible(chain(domain.class_vars,
-                                 domain.metas,
-                                 domain.attributes)))
-
     def add_all(self):
         if self.cond_list.rowCount():
             Mb = QMessageBox
@@ -299,9 +334,9 @@ class OWSelectRows(widget.OWWidget):
                     "这将用所有变量的过滤器替换现有的过滤器。", Mb.Ok | Mb.Cancel) != Mb.Ok:
                 return
             self.remove_all()
-        domain = self.data.domain
-        for i in range(len(domain.variables) + len(domain.metas)):
-            self.add_row(i)
+        for attr in self.variable_model[len(self.AllTypes) + 1:]:
+            self.add_row(attr)
+        self.conditions_changed()
 
     def remove_one(self, rownum):
         self.remove_one_row(rownum)
@@ -317,12 +352,20 @@ class OWSelectRows(widget.OWWidget):
             self.remove_all_button.setDisabled(True)
 
     def remove_all_rows(self):
+        # Disconnect signals to avoid stray emits when changing variable_model
+        for row in range(self.cond_list.rowCount()):
+            for col in (0, 1):
+                widg = self.cond_list.cellWidget(row, col)
+                if widg:
+                    widg.currentIndexChanged.disconnect()
         self.cond_list.clear()
         self.cond_list.setRowCount(0)
         self.remove_all_button.setDisabled(True)
 
     def set_new_operators(self, attr_combo, adding_all,
                           selected_index=None, selected_values=None):
+        old_combo = self.cond_list.cellWidget(attr_combo.row, 1)
+        prev_text = old_combo.currentText() if old_combo else ""
         oper_combo = QComboBox()
         oper_combo.row = attr_combo.row
         oper_combo.attr_combo = attr_combo
@@ -332,7 +375,11 @@ class OWSelectRows(widget.OWWidget):
         else:
             var = self.data.domain[attr_name]
             oper_combo.addItems(self.operator_names[type(var)])
-        oper_combo.setCurrentIndex(selected_index or 0)
+        if selected_index is None:
+            selected_index = oper_combo.findText(prev_text)
+            if selected_index == -1:
+                selected_index = 0
+        oper_combo.setCurrentIndex(selected_index)
         self.cond_list.setCellWidget(oper_combo.row, 1, oper_combo)
         self.set_new_values(oper_combo, adding_all, selected_values)
         oper_combo.currentIndexChanged.connect(
@@ -340,8 +387,18 @@ class OWSelectRows(widget.OWWidget):
 
     @staticmethod
     def _get_lineedit_contents(box):
-        return [child.text() for child in getattr(box, "controls", [box])
-                if isinstance(child, QLineEdit)]
+        contents = []
+        for child in getattr(box, "controls", [box]):
+            if isinstance(child, QLineEdit):
+                contents.append(child.text())
+            elif isinstance(child, DateTimeWidget):
+                if child.format == (0, 1):
+                    contents.append(child.time())
+                elif child.format == (1, 0):
+                    contents.append(child.date())
+                elif child.format == (1, 1):
+                    contents.append(child.dateTime())
+        return contents
 
     @staticmethod
     def _get_value_contents(box):
@@ -362,6 +419,13 @@ class OWSelectRows(widget.OWWidget):
                             names.append(item.text())
                     child.desc_text = ', '.join(names)
                     child.set_text()
+            elif isinstance(child, DateTimeWidget):
+                if child.format == (0, 1):
+                    cont.append(child.time())
+                elif child.format == (1, 0):
+                    cont.append(child.date())
+                elif child.format == (1, 1):
+                    cont.append(child.dateTime())
             elif isinstance(child, QLabel) or child is None:
                 pass
             else:
@@ -397,11 +461,6 @@ class OWSelectRows(widget.OWWidget):
             le.setValidator(OWSelectRows.QDoubleValidatorEmpty())
             return le
 
-        def add_datetime(contents):
-            le = add_textual(contents)
-            le.setValidator(QRegExpValidator(QRegExp(TimeVariable.REGEX)))
-            return le
-
         box = self.cond_list.cellWidget(oper_combo.row, 2)
         lc = ["", ""]
         oper = oper_combo.currentIndex()
@@ -411,10 +470,11 @@ class OWSelectRows(widget.OWWidget):
             var = None
         else:
             var = self.data.domain[attr_name]
+            var_idx = self.data.domain.index(attr_name)
             vtype = vartype(var)
             if selected_values is not None:
                 lc = list(selected_values) + ["", ""]
-                lc = [str(x) for x in lc[:2]]
+                lc = [str(x) if vtype != 4 else x for x in lc[:2]]
         if box and vtype == box.var_type:
             lc = self._get_lineedit_contents(box) + lc
 
@@ -425,13 +485,13 @@ class OWSelectRows(widget.OWWidget):
         elif var is not None and var.is_discrete:
             if oper_combo.currentText().endswith(" one of"):
                 if selected_values:
-                    lc = [x for x in list(selected_values)]
+                    lc = list(selected_values)
                 button = DropDownToolButton(self, var, lc)
                 button.var_type = vtype
                 self.cond_list.setCellWidget(oper_combo.row, 2, button)
             else:
-                combo = QComboBox()
-                combo.addItems([""] + var.values)
+                combo = ComboBoxSearch()
+                combo.addItems(("", ) + var.values)
                 if lc[0]:
                     combo.setCurrentIndex(int(lc[0]))
                 else:
@@ -443,17 +503,48 @@ class OWSelectRows(widget.OWWidget):
             box = gui.hBox(self, addToLayout=False)
             box.var_type = vtype
             self.cond_list.setCellWidget(oper_combo.row, 2, box)
-            if vtype in (2, 4):  # continuous, time:
-                validator = add_datetime if isinstance(var, TimeVariable) else add_numeric
-                box.controls = [validator(lc[0])]
+            if vtype == 2:  # continuous:
+                box.controls = [add_numeric(lc[0])]
                 if oper > 5:
                     gui.widgetLabel(box, " and ")
-                    box.controls.append(validator(lc[1]))
+                    box.controls.append(add_numeric(lc[1]))
             elif vtype == 3:  # string:
                 box.controls = [add_textual(lc[0])]
                 if oper in [6, 7]:
                     gui.widgetLabel(box, " and ")
                     box.controls.append(add_textual(lc[1]))
+            elif vtype == 4:  # time:
+                def invalidate_datetime():
+                    if w_:
+                        if w.dateTime() > w_.dateTime():
+                            w_.setDateTime(w.dateTime())
+                        if w.format == (1, 1):
+                            w.calendarWidget.timeedit.setTime(w.time())
+                            w_.calendarWidget.timeedit.setTime(w_.time())
+                    elif w.format == (1, 1):
+                        w.calendarWidget.timeedit.setTime(w.time())
+
+                def datetime_changed():
+                    self.conditions_changed()
+                    invalidate_datetime()
+
+                datetime_format = (var.have_date, var.have_time)
+                column = self.data.get_column_view(var_idx)[0]
+                w = DateTimeWidget(self, column, datetime_format)
+                w.set_datetime(lc[0])
+                box.controls = [w]
+                box.layout().addWidget(w)
+                w.dateTimeChanged.connect(datetime_changed)
+                if oper > 5:
+                    gui.widgetLabel(box, " and ")
+                    w_ = DateTimeWidget(self, column, datetime_format)
+                    w_.set_datetime(lc[1])
+                    box.layout().addWidget(w_)
+                    box.controls.append(w_)
+                    invalidate_datetime()
+                    w_.dateTimeChanged.connect(datetime_changed)
+                else:
+                    w_ = None
             else:
                 box.controls = []
         if not adding_all:
@@ -471,38 +562,37 @@ class OWSelectRows(widget.OWWidget):
             data is None or
             len(data.domain.variables) + len(data.domain.metas) > 100)
         if not data:
+            self.info.set_input_summary(self.info.NoInput)
             self.data_desc = None
+            self.variable_model.set_domain(None)
             self.commit()
             return
         self.data_desc = report.describe_data_brief(data)
-        self.conditions = []
-        try:
-            self.openContext(data)
-        except Exception:
-            pass
+        self.variable_model.set_domain(data.domain)
 
-        variables = list(self._visible_variables(self.data.domain))
-        varnames = [v.name if isinstance(v, Variable) else v for v in variables]
-        if self.conditions:
-            for attr, cond_type, cond_value in self.conditions:
-                if attr in varnames:
-                    self.add_row(varnames.index(attr), cond_type, cond_value)
-                elif attr in self.AllTypes:
-                    self.add_row(attr, cond_type, cond_value)
-        else:
+        self.conditions = []
+        self.openContext(data)
+        for attr, cond_type, cond_value in self.conditions:
+            if attr in self.variable_model:
+                self.add_row(attr, cond_type, cond_value)
+        if not self.cond_list.model().rowCount():
             self.add_row()
 
-        self.update_info(data, self.data_in_variables, "输入: ")
+        self.info.set_input_summary(data.approx_len(),
+                                    format_summary_details(data))
         self.unconditional_commit()
 
     def conditions_changed(self):
         try:
-            self.conditions = []
+            cells_by_rows = (
+                [self.cond_list.cellWidget(row, col) for col in range(3)]
+                for row in range(self.cond_list.rowCount())
+            )
             self.conditions = [
-                (self.cond_list.cellWidget(row, 0).currentText(),
-                 self.cond_list.cellWidget(row, 1).currentIndex(),
-                 self._get_value_contents(self.cond_list.cellWidget(row, 2)))
-                for row in range(self.cond_list.rowCount())]
+                (var_cell.currentData(gui.TableVariable) or var_cell.currentText(),
+                 oper_cell.currentIndex(),
+                 self._get_value_contents(val_cell))
+                for var_cell, oper_cell, val_cell in cells_by_rows]
             if self.update_on_change and (
                     self.last_output_conditions is None or
                     self.last_output_conditions != self.conditions):
@@ -512,12 +602,14 @@ class OWSelectRows(widget.OWWidget):
             # controls are being constructed
             pass
 
-    def _values_to_floats(self, attr, values):
-        if not len(values):
+    @staticmethod
+    def _values_to_floats(attr, values):
+        if len(values) == 0:
             return values
         if not all(values):
             return None
         if isinstance(attr, TimeVariable):
+            values = (value.toString(format=Qt.ISODate) for value in values)
             parse = lambda x: (attr.parse(x), True)
         else:
             parse = QLocale().toDouble
@@ -553,7 +645,7 @@ class OWSelectRows(widget.OWWidget):
                     operators = self.Operators[type(attr)]
                 opertype, _ = operators[oper_idx]
                 if attr_type == 0:
-                    filter = data_filter.IsDefined()
+                    filt = data_filter.IsDefined()
                 elif attr_type in (2, 4):  # continuous, time
                     try:
                         floats = self._values_to_floats(attr, values)
@@ -562,10 +654,10 @@ class OWSelectRows(widget.OWWidget):
                         return
                     if floats is None:
                         continue
-                    filter = data_filter.FilterContinuous(
+                    filt = data_filter.FilterContinuous(
                         attr_index, opertype, *floats)
                 elif attr_type == 3:  # string
-                    filter = data_filter.FilterString(
+                    filt = data_filter.FilterString(
                         attr_index, opertype, *[str(v) for v in values])
                 else:
                     if opertype == FilterDiscreteType.IsDefined:
@@ -583,14 +675,14 @@ class OWSelectRows(widget.OWWidget):
                             f_values = set(values)
                         else:
                             raise ValueError("invalid operand")
-                    filter = data_filter.FilterDiscrete(attr_index, f_values)
-                conditions.append(filter)
+                    filt = data_filter.FilterDiscrete(attr_index, f_values)
+                conditions.append(filt)
 
             if conditions:
-                self.filters = data_filter.Values(conditions)
-                matching_output = self.filters(self.data)
-                self.filters.negate = True
-                non_matching_output = self.filters(self.data)
+                filters = data_filter.Values(conditions)
+                matching_output = filters(self.data)
+                filters.negate = True
+                non_matching_output = filters(self.data)
 
                 row_sel = np.in1d(self.data.ids, matching_output.ids)
                 annotated_output = create_annotated_table(self.data, row_sel)
@@ -614,11 +706,11 @@ class OWSelectRows(widget.OWWidget):
                 non_matching_output = remover(non_matching_output)
                 annotated_output = remover(annotated_output)
 
-        if matching_output is not None and not len(matching_output):
+        if not matching_output:
             matching_output = None
-        if non_matching_output is not None and not len(non_matching_output):
+        if not non_matching_output:
             non_matching_output = None
-        if annotated_output is not None and not len(annotated_output):
+        if not annotated_output:
             annotated_output = None
 
         self.Outputs.matching_data.send(matching_output)
@@ -628,20 +720,10 @@ class OWSelectRows(widget.OWWidget):
         self.match_desc = report.describe_data_brief(matching_output)
         self.nonmatch_desc = report.describe_data_brief(non_matching_output)
 
-        self.update_info(matching_output, self.data_out_rows, "输出: ")
-
-    def update_info(self, data, lab1, label):
-        def sp(s, capitalize=True):
-            return s and s or ("No" if capitalize else "no"), #"s" * (s != 1)
-
-        if data is None:
-            lab1.setText("")
-        else:
-            lab1.setText(label + "~%s 行, %s 个变量" %
-                         (sp(data.approx_len()) +
-                          sp(len(data.domain.variables) +
-                             len(data.domain.metas)))
-                        )
+        summary = matching_output.approx_len() if matching_output else \
+            self.info.NoOutput
+        details = format_summary_details(matching_output) if matching_output else ""
+        self.info.set_output_summary(summary, details)
 
     def send_report(self):
         if not self.data:
@@ -660,19 +742,18 @@ class OWSelectRows(widget.OWWidget):
             pdesc = ndesc
 
         conditions = []
-        domain = self.data.domain
-        for attr_name, oper, values in self.conditions:
-            if attr_name in self.AllTypes:
-                attr = attr_name
+        for attr, oper, values in self.conditions:
+            if isinstance(attr, str):
+                attr_name = attr
+                var_type = self.AllTypes[attr]
                 names = self.operator_names[attr_name]
-                var_type = self.AllTypes[attr_name]
             else:
-                attr = domain[attr_name]
+                attr_name = attr.name
                 var_type = vartype(attr)
                 names = self.operator_names[type(attr)]
             name = names[oper]
             if oper == len(names) - 1:
-                conditions.append("{} {}".format(attr, name))
+                conditions.append("{} {}".format(attr_name, name))
             elif var_type == 1:  # discrete
                 if name == "is one of":
                     valnames = [attr.values[v - 1] for v in values]
@@ -689,6 +770,9 @@ class OWSelectRows(widget.OWWidget):
             elif var_type == 3:  # string variable
                 conditions.append(
                     f"{attr} {name} {' and '.join(map(repr, values))}")
+            elif var_type == 4:  # time
+                values = (value.toString(format=Qt.ISODate) for value in values)
+                conditions.append(f"{attr} {name} {' and '.join(values)}")
             elif all(x for x in values):  # numeric variable
                 conditions.append(f"{attr} {name} {' and '.join(values)}")
         items = OrderedDict()
@@ -725,7 +809,7 @@ class OWSelectRows(widget.OWWidget):
 
 
 class CheckBoxPopup(QWidget):
-    def __init__(self, var, lc, widget_parent=None, widget=None):
+    def __init__(self, var, lc, widget_parent=None, widg=None):
         QWidget.__init__(self)
 
         self.list_view = QListView()
@@ -749,7 +833,7 @@ class CheckBoxPopup(QWidget):
         self.adjustSize()
         self.setWindowFlags(Qt.Popup)
 
-        self.widget = widget
+        self.widget = widg
         self.widget.desc_text = ', '.join(text)
         self.widget.set_text()
 
@@ -776,9 +860,79 @@ class DropDownToolButton(QToolButton):
         self.setText(metrics.elidedText(self.desc_text, Qt.ElideRight,
                                         self.width() - 15))
 
-    def resizeEvent(self, QResizeEvent):
+    def resizeEvent(self, _):
         self.set_text()
 
 
+class DateTimeWidget(QDateTimeEdit):
+    def __init__(self, parent, column, datetime_format):
+        QDateTimeEdit.__init__(self, parent)
+
+        self.format = datetime_format
+        self.have_date, self.have_time = datetime_format[0], datetime_format[1]
+        self.set_format(column)
+        self.setSizePolicy(
+            QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
+
+    def set_format(self, column):
+        str_format = Qt.ISODate
+        if self.have_date and self.have_time:
+            self.setDisplayFormat("yyyy-MM-dd hh:mm:ss")
+            c_format = "%Y-%m-%d %H:%M:%S"
+            min_datetime, max_datetime = self.find_range(column, c_format)
+            self.min_datetime = QDateTime.fromString(min_datetime, str_format)
+            self.max_datetime = QDateTime.fromString(max_datetime, str_format)
+            self.setCalendarPopup(True)
+            self.calendarWidget = gui.CalendarWidgetWithTime(
+                self, time=self.min_datetime.time())
+            self.calendarWidget.timeedit.timeChanged.connect(
+                self.set_datetime)
+            self.setCalendarWidget(self.calendarWidget)
+            self.setDateTimeRange(self.min_datetime, self.max_datetime)
+
+        elif self.have_date and not self.have_time:
+            self.setDisplayFormat("yyyy-MM-dd")
+            self.setCalendarPopup(True)
+            min_datetime, max_datetime = self.find_range(column, "%Y-%m-%d")
+            self.min_datetime = QDate.fromString(min_datetime, str_format)
+            self.max_datetime = QDate.fromString(max_datetime, str_format)
+            self.setDateRange(self.min_datetime, self.max_datetime)
+
+        elif not self.have_date and self.have_time:
+            self.setDisplayFormat("hh:mm:ss")
+            min_datetime, max_datetime = self.find_range(column, "%H:%M:%S")
+            self.min_datetime = QTime.fromString(min_datetime, str_format)
+            self.max_datetime = QTime.fromString(max_datetime, str_format)
+            self.setTimeRange(self.min_datetime, self.max_datetime)
+
+    def set_datetime(self, date_time):
+        if not date_time:
+            date_time = self.min_datetime
+        if self.have_date and self.have_time:
+            if isinstance(date_time, QTime):
+                self.setDateTime(
+                    QDateTime(self.date(), self.calendarWidget.timeedit.time()))
+            else:
+                self.setDateTime(date_time)
+        elif self.have_date and not self.have_time:
+            self.setDate(date_time)
+        elif not self.have_date and self.have_time:
+            self.setTime(date_time)
+
+    @staticmethod
+    def find_range(column, convert_format):
+        def convert_timestamp(timestamp):
+            if timestamp >= 0:
+                return datetime.fromtimestamp(timestamp, tz=timezone.utc)
+            return datetime(1970, 1, 1, tzinfo=timezone.utc) + \
+                       timedelta(seconds=int(timestamp))
+
+        min_datetime = convert_timestamp(
+            np.nanmin(column)).strftime(convert_format)
+        max_datetime = convert_timestamp(
+            np.nanmax(column)).strftime(convert_format)
+        return min_datetime, max_datetime
+
+
 if __name__ == "__main__":  # pragma: no cover
-    WidgetPreview(OWSelectRows).run(Table("zoo"))
+    WidgetPreview(OWSelectRows).run(Table("heart_disease"))
