@@ -1,12 +1,9 @@
 import sys
 import threading
-import io
-import csv
 import itertools
 import concurrent.futures
 
 from collections import OrderedDict, namedtuple
-from typing import List, Tuple, Iterable
 
 from math import isnan
 
@@ -19,9 +16,9 @@ from AnyQt.QtWidgets import (
 )
 from AnyQt.QtGui import QColor, QClipboard
 from AnyQt.QtCore import (
-    Qt, QSize, QEvent, QByteArray, QMimeData, QObject, QMetaObject,
+    Qt, QSize, QEvent, QObject, QMetaObject,
     QAbstractProxyModel, QIdentityProxyModel, QModelIndex,
-    QItemSelectionModel, QItemSelection, QItemSelectionRange
+    QItemSelectionModel, QItemSelection, QItemSelectionRange,
 )
 from AnyQt.QtCore import pyqtSlot as Slot
 
@@ -32,13 +29,19 @@ from Orange.data.sql.table import SqlTable
 from Orange.statistics import basic_stats
 
 from Orange.widgets import gui
-from Orange.widgets.settings import Setting, DomainContextHandler
+from Orange.widgets.settings import Setting
+from Orange.widgets.utils.itemselectionmodel import (
+    BlockSelectionModel, ranges, selection_blocks
+)
+from Orange.widgets.utils.tableview import TableView, \
+    table_selection_to_mime_data
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.widget import OWWidget, Input, Output
 from Orange.widgets.utils import datacaching
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_Chinese_NAME)
 from Orange.widgets.utils.itemmodels import TableModel
+from Orange.widgets.utils.state_summary import format_summary_details
 
 
 class RichTableModel(TableModel):
@@ -159,215 +162,21 @@ class TableSliceProxy(QIdentityProxyModel):
         return stop - start
 
 
-class BlockSelectionModel(QItemSelectionModel):
-    """
-    Item selection model ensuring the selection maintains a simple block
-    like structure.
-
-    e.g.
-
-        [a b] c [d e]
-        [f g] h [i j]
-
-    is allowed but this is not
-
-        [a] b  c  d e
-        [f  g] h [i j]
-
-    I.e. select the Cartesian product of row and column indices.
-
-    """
-    def __init__(self, model, parent=None, selectBlocks=True, **kwargs):
-        super().__init__(model, parent, **kwargs)
-        self.__selectBlocks = selectBlocks
-
-    def select(self, selection, flags):
-        """Reimplemented."""
-        if isinstance(selection, QModelIndex):
-            selection = QItemSelection(selection, selection)
-
-        if not self.__selectBlocks:
-            super().select(selection, flags)
-            return
-
-        model = self.model()
-
-        def to_ranges(spans):
-            return list(range(*r) for r in spans)
-
-        if flags & QItemSelectionModel.Current:  # no current selection support
-            flags &= ~QItemSelectionModel.Current
-        if flags & QItemSelectionModel.Toggle:  # no toggle support either
-            flags &= ~QItemSelectionModel.Toggle
-            flags |= QItemSelectionModel.Select
-
-        if flags == QItemSelectionModel.ClearAndSelect:
-            # extend selection ranges in `selection` to span all row/columns
-            sel_rows = selection_rows(selection)
-            sel_cols = selection_columns(selection)
-            selection = QItemSelection()
-            for row_range, col_range in \
-                    itertools.product(to_ranges(sel_rows), to_ranges(sel_cols)):
-                selection.select(
-                    model.index(row_range.start, col_range.start),
-                    model.index(row_range.stop - 1, col_range.stop - 1)
-                )
-        elif flags & (QItemSelectionModel.Select |
-                      QItemSelectionModel.Deselect):
-            # extend all selection ranges in `selection` with the full current
-            # row/col spans
-            rows, cols = selection_blocks(self.selection())
-            sel_rows = selection_rows(selection)
-            sel_cols = selection_columns(selection)
-            ext_selection = QItemSelection()
-            for row_range, col_range in \
-                    itertools.product(to_ranges(rows), to_ranges(sel_cols)):
-                ext_selection.select(
-                    model.index(row_range.start, col_range.start),
-                    model.index(row_range.stop - 1, col_range.stop - 1)
-                )
-            for row_range, col_range in \
-                    itertools.product(to_ranges(sel_rows), to_ranges(cols)):
-                ext_selection.select(
-                    model.index(row_range.start, col_range.start),
-                    model.index(row_range.stop - 1, col_range.stop - 1)
-                )
-            selection.merge(ext_selection, QItemSelectionModel.Select)
-        super().select(selection, flags)
-
-    def selectBlocks(self):
-        """Is the block selection in effect."""
-        return self.__selectBlocks
-
-    def setSelectBlocks(self, state):
-        """Set the block selection state.
-
-        If set to False, the selection model behaves as the base
-        QItemSelectionModel
-
-        """
-        self.__selectBlocks = state
-
-
-def selection_rows(selection):
-    # type: (QItemSelection) -> List[Tuple[int, int]]
-    """
-    Return a list of ranges for all referenced rows contained in selection
-
-    Parameters
-    ----------
-    selection : QItemSelection
-
-    Returns
-    -------
-    rows : List[Tuple[int, int]]
-    """
-    spans = set(range(s.top(), s.bottom() + 1) for s in selection)
-    indices = sorted(set(itertools.chain(*spans)))
-    return list(ranges(indices))
-
-
-def selection_columns(selection):
-    # type: (QItemSelection) -> List[Tuple[int, int]]
-    """
-    Return a list of ranges for all referenced columns contained in selection
-
-    Parameters
-    ----------
-    selection : QItemSelection
-
-    Returns
-    -------
-    rows : List[Tuple[int, int]]
-    """
-    spans = {range(s.left(), s.right() + 1) for s in selection}
-    indices = sorted(set(itertools.chain(*spans)))
-    return list(ranges(indices))
-
-
-def selection_blocks(selection):
-    # type: (QItemSelection) -> Tuple[List[Tuple[int, int]], List[Tuple[int, int]]]
-    if selection.count() > 0:
-        rowranges = {range(span.top(), span.bottom() + 1)
-                     for span in selection}
-        colranges = {range(span.left(), span.right() + 1)
-                     for span in selection}
-    else:
-        return [], []
-
-    rows = sorted(set(itertools.chain(*rowranges)))
-    cols = sorted(set(itertools.chain(*colranges)))
-    return list(ranges(rows)), list(ranges(cols))
-
-
-def ranges(indices):
-    # type: (Iterable[int]) -> Iterable[Tuple[int, int]]
-    """
-    Group consecutive indices into `(start, stop)` tuple 'ranges'.
-
-    >>> list(ranges([1, 2, 3, 5, 3, 4]))
-    >>> [(1, 4), (5, 6), (3, 5)]
-
-    """
-    g = itertools.groupby(enumerate(indices),
-                          key=lambda t: t[1] - t[0])
-    for _, range_ind in g:
-        range_ind = list(range_ind)
-        _, start = range_ind[0]
-        _, end = range_ind[-1]
-        yield start, end + 1
-
-
-def table_selection_to_mime_data(table):
-    """Copy the current selection in a QTableView to the clipboard.
-    """
-    lines = table_selection_to_list(table)
-
-    as_csv = lines_to_csv_string(lines, dialect="excel").encode("utf-8")
-    as_tsv = lines_to_csv_string(lines, dialect="excel-tab").encode("utf-8")
-
-    mime = QMimeData()
-    mime.setData("text/csv", QByteArray(as_csv))
-    mime.setData("text/tab-separated-values", QByteArray(as_tsv))
-    mime.setData("text/plain", QByteArray(as_tsv))
-    return mime
-
-
-def lines_to_csv_string(lines, dialect="excel"):
-    stream = io.StringIO()
-    writer = csv.writer(stream, dialect=dialect)
-    writer.writerows(lines)
-    return stream.getvalue()
-
-
-def table_selection_to_list(table):
-    model = table.model()
-    indexes = table.selectedIndexes()
-
-    rows = sorted(set(index.row() for index in indexes))
-    columns = sorted(set(index.column() for index in indexes))
-
-    lines = []
-    for row in rows:
-        line = []
-        for col in columns:
-            val = model.index(row, col).data(Qt.DisplayRole)
-            # TODO: use style item delegate displayText?
-            line.append(str(val))
-        lines.append(line)
-
-    return lines
-
-
 TableSlot = namedtuple("TableSlot", ["input_id", "table", "summary", "view"])
 
 
+class DataTableView(gui.HScrollStepMixin, TableView):
+    dataset: Table
+    input_slot: TableSlot
+
+
 class OWDataTable(OWWidget):
     name = "数据表(Data Table)"
     description = "在电子表格中查看数据集。"
     icon = "icons/Table.svg"
     priority = 50
-    keywords = []
+    keywords = ['shujubiao', 'biao']
+    category = "Data"
 
     buttons_area_orientation = Qt.Vertical
 
@@ -385,8 +194,6 @@ class OWDataTable(OWWidget):
     auto_commit = Setting(True)
 
     color_by_class = Setting(True)
-    settingsHandler = DomainContextHandler(
-        match_values=DomainContextHandler.MATCH_VALUES_ALL)
     selected_rows = Setting([], schema_only=True)
     selected_cols = Setting([], schema_only=True)
 
@@ -402,16 +209,10 @@ class OWDataTable(OWWidget):
 
         self.dist_color = QColor(*self.dist_color_RGB)
 
-        info_box = gui.vBox(self.controlArea, "信息")
-        self.info_ex = gui.widgetLabel(info_box, '没有输入数据。', )
-        self.info_ex.setWordWrap(True)
-        self.info_attr = gui.widgetLabel(info_box, ' ')
-        self.info_attr.setWordWrap(True)
-        self.info_class = gui.widgetLabel(info_box, ' ')
-        self.info_class.setWordWrap(True)
-        self.info_meta = gui.widgetLabel(info_box, ' ')
-        self.info_meta.setWordWrap(True)
-        info_box.setMinimumWidth(200)
+        info_box = gui.vBox(self.controlArea, "Info")
+        self.info_text = gui.widgetLabel(info_box)
+        self._set_input_summary(None)
+        self._set_output_summary(None)
         gui.separator(self.controlArea)
 
         box = gui.vBox(self.controlArea, "变量")
@@ -453,7 +254,6 @@ class OWDataTable(OWWidget):
     @Inputs.data
     def set_dataset(self, data, tid=None):
         """Set the input dataset."""
-        self.closeContext()
         if data is not None:
             datasetname = getattr(data, "name", "Data")
             if tid in self._inputs:
@@ -466,9 +266,8 @@ class OWDataTable(OWWidget):
                 assert self.tabs.indexOf(view) != -1
                 self.tabs.setTabText(self.tabs.indexOf(view), datasetname)
             else:
-                view = QTableView()
+                view = DataTableView()
                 view.setSortingEnabled(True)
-                view.setHorizontalScrollMode(QTableView.ScrollPerPixel)
 
                 if self.select_rows:
                     view.setSelectionBehavior(QTableView.SelectRows)
@@ -494,12 +293,12 @@ class OWDataTable(OWWidget):
 
             self._setup_table_view(view, data)
             slot = TableSlot(tid, data, table_summary(data), view)
-            view._input_slot = slot  # pylint: disable=protected-access
+            view.input_slot = slot
             self._inputs[tid] = slot
 
             self.tabs.setCurrentIndex(self.tabs.indexOf(view))
 
-            self.set_info(slot.summary)
+            self._set_input_summary(slot)
 
             if isinstance(slot.summary.len, concurrent.futures.Future):
                 def update(_):
@@ -517,11 +316,11 @@ class OWDataTable(OWWidget):
 
             current = self.tabs.currentWidget()
             if current is not None:
-                # pylint: disable=protected-access
-                self.set_info(current._input_slot.summary)
+                self._set_input_summary(current.input_slot)
+        else:
+            self._set_input_summary(None)
 
         self.tabs.tabBar().setVisible(self.tabs.count() > 1)
-        self.openContext(data)
 
         if data and self.__pending_selected_rows is not None:
             self.selected_rows = self.__pending_selected_rows
@@ -567,6 +366,7 @@ class OWDataTable(OWWidget):
         header = view.horizontalHeader()
         header.setSectionsClickable(is_sortable(data))
         header.setSortIndicatorShown(is_sortable(data))
+        header.sortIndicatorChanged.connect(self.update_selection)
 
         view.setModel(datamodel)
 
@@ -601,7 +401,7 @@ class OWDataTable(OWWidget):
         selmodel = BlockSelectionModel(
             view.model(), parent=view, selectBlocks=not self.select_rows)
         view.setSelectionModel(selmodel)
-        view.selectionModel().selectionChanged.connect(self.update_selection)
+        view.selectionFinished.connect(self.update_selection)
 
     #noinspection PyBroadException
     def set_corner_text(self, table, text):
@@ -666,9 +466,89 @@ class OWDataTable(OWWidget):
             except Exception:
                 pass
 
+    def _set_input_summary(self, slot):
+        def format_summary(summary):
+            if isinstance(summary, ApproxSummary):
+                length = summary.len.result() if summary.len.done() else \
+                    summary.approx_len
+            elif isinstance(summary, Summary):
+                length = summary.len
+            return length
+
+        summary, details = self.info.NoInput, ""
+        if slot:
+            summary = format_summary(slot.summary)
+            details = format_summary_details(slot.table)
+        self.info.set_input_summary(summary, details)
+
+        self.info_text.setText("\n".join(self._info_box_text(slot)))
+
+    @staticmethod
+    def _info_box_text(slot):
+        def format_part(part):
+            if isinstance(part, DenseArray):
+                if not part.nans:
+                    return ""
+                perc = 100 * part.nans / (part.nans + part.non_nans)
+                return f" ({perc:.1f} % missing data)"
+
+            if isinstance(part, SparseArray):
+                tag = "sparse"
+            elif isinstance(part, SparseBoolArray):
+                tag = "tags"
+            else:  # isinstance(part, NotAvailable)
+                return ""
+            dens = 100 * part.non_nans / (part.nans + part.non_nans)
+            return f" ({tag}, density {dens:.2f} %)"
+
+        def desc(n, part):
+            if n == 0:
+                return f"No {part}s"
+            elif n == 1:
+                return f"1 {part}"
+            else:
+                return f"{n} {part}s"
+
+        if slot is None:
+            return ["No data."]
+        summary = slot.summary
+        text = []
+        if isinstance(summary, ApproxSummary):
+            if summary.len.done():
+                text.append(f"{summary.len.result()} instances")
+            else:
+                text.append(f"~{summary.approx_len} instances")
+        elif isinstance(summary, Summary):
+            text.append(f"{summary.len} instances")
+            if sum(p.nans for p in [summary.X, summary.Y, summary.M]) == 0:
+                text[-1] += " (no missing data)"
+
+        text.append(desc(len(summary.domain.attributes), "feature")
+                    + format_part(summary.X))
+
+        if not summary.domain.class_vars:
+            text.append("No target variable.")
+        else:
+            if len(summary.domain.class_vars) > 1:
+                c_text = desc(len(summary.domain.class_vars), "outcome")
+            elif summary.domain.has_continuous_class:
+                c_text = "Numeric outcome"
+            else:
+                c_text = "Target with " \
+                    + desc(len(summary.domain.class_var.values), "value")
+            text.append(c_text + format_part(summary.Y))
+
+        text.append(desc(len(summary.domain.metas), "meta attribute")
+                    + format_part(summary.M))
+        return text
+
+    def _set_output_summary(self, output):
+        summary = len(output) if output else self.info.NoOutput
+        details = format_summary_details(output) if output else ""
+        self.info.set_output_summary(summary, details)
+
     def _on_select_all(self, _):
-        # pylint: disable=protected-access
-        data_info = self.tabs.currentWidget()._input_slot.summary
+        data_info = self.tabs.currentWidget().input_slot.summary
         if len(self.selected_rows) == data_info.len \
                 and len(self.selected_cols) == len(data_info.domain):
             self.tabs.currentWidget().clearSelection()
@@ -676,13 +556,13 @@ class OWDataTable(OWWidget):
             self.tabs.currentWidget().selectAll()
 
     def _on_current_tab_changed(self, index):
-        """Update the info box on current tab change"""
+        """Update the status bar on current tab change"""
         view = self.tabs.widget(index)
         if view is not None and view.model() is not None:
-            # pylint: disable=protected-access
-            self.set_info(view._input_slot.summary)
+            self._set_input_summary(view.input_slot)
+            self.update_selection()
         else:
-            self.set_info(None)
+            self._set_input_summary(None)
 
     def _update_variable_labels(self, view):
         "Update the variable labels visibility for `view`"
@@ -752,27 +632,11 @@ class OWDataTable(OWWidget):
         if table is not None:
             table.horizontalHeader().setSortIndicator(-1, Qt.AscendingOrder)
 
-    def set_info(self, summary):
-        if summary is None:
-            self.info_ex.setText("No data on input.")
-            self.info_attr.setText("")
-            self.info_class.setText("")
-            self.info_meta.setText("")
-        else:
-            info_len, info_attr, info_class, info_meta = \
-                format_summary(summary)
-
-            self.info_ex.setText(info_len)
-            self.info_attr.setText(info_attr)
-            self.info_class.setText(info_class)
-            self.info_meta.setText(info_meta)
-
     @Slot()
     def _update_info(self):
         current = self.tabs.currentWidget()
         if current is not None and current.model() is not None:
-            # pylint: disable=protected-access
-            self.set_info(current._input_slot.summary)
+            self._set_input_summary(current.input_slot)
 
     def update_selection(self, *_):
         self.commit()
@@ -823,7 +687,6 @@ class OWDataTable(OWWidget):
         rows = numpy.array(rows, dtype=numpy.intp)
         # map the rows through the applied sorting (if any)
         rows = model.mapToSourceRows(rows)
-        rows.sort()
         rows = rows.tolist()
         return rows, cols
 
@@ -847,6 +710,7 @@ class OWDataTable(OWWidget):
             # Selections of individual instances are not implemented
             # for SqlTables
             if isinstance(table, SqlTable):
+                self._set_output_summary(selected_data)
                 self.Outputs.selected_data.send(selected_data)
                 self.Outputs.annotated_data.send(None)
                 return
@@ -854,21 +718,6 @@ class OWDataTable(OWWidget):
             rowsel, colsel = self.get_selection(view)
             self.selected_rows, self.selected_cols = rowsel, colsel
 
-            def select(data, rows, domain):
-                """
-                Select the data subset with specified rows and domain subsets.
-
-                If either rows or domain is None they mean select all.
-                """
-                if rows is not None and domain is not None:
-                    return data.from_table(domain, data, rows)
-                elif rows is not None:
-                    return data.from_table(data.domain, rows)
-                elif domain is not None:
-                    return data.from_table(domain, data)
-                else:
-                    return data
-
             domain = table.domain
 
             if len(colsel) < len(domain) + len(domain.metas):
@@ -891,14 +740,13 @@ class OWDataTable(OWWidget):
                 metas = select_vars(TableModel.Meta)
                 domain = Orange.data.Domain(attrs, class_vars, metas)
 
-            # Avoid a copy if all/none rows are selected.
+            # Avoid a copy if none rows are selected.
             if not rowsel:
                 selected_data = None
-            elif len(rowsel) == len(table):
-                selected_data = select(table, None, domain)
             else:
-                selected_data = select(table, rowsel, domain)
+                selected_data = table.from_table(domain, table, rowsel)
 
+        self._set_output_summary(selected_data)
         self.Outputs.selected_data.send(selected_data)
         self.Outputs.annotated_data.send(create_annotated_table(table, rowsel))
 
@@ -991,69 +839,6 @@ def table_summary(table):
         return Summary(n_instances, domain, X_part, Y_part, M_part)
 
 
-def format_summary(summary):
-    text = []
-    if isinstance(summary, ApproxSummary):
-        if summary.len.done():
-            text += ["{} 个实例".format(summary.len.result())]
-        else:
-            text += ["~{} 个实例".format(summary.approx_len)]
-
-    elif isinstance(summary, Summary):
-        text += ["{} 个实例".format(summary.len)]
-
-        if sum(p.nans for p in [summary.X, summary.Y, summary.M]) == 0:
-            text[-1] += " (无缺失值)"
-
-    def format_part(part):
-        if isinstance(part, NotAvailable):
-            return ""
-        elif part.nans + part.non_nans == 0:
-            return ""
-
-        if isinstance(part, DenseArray):
-            total = part.nans + part.non_nans
-            miss = ("%.1f%%" % (100 * part.nans / total) if part.nans > 0
-                    else "无")
-            return " (%s缺失值)" % miss
-        elif isinstance(part, (SparseArray, SparseBoolArray)):
-            text = " ({}, 密度 {:.2f}%)"
-            tag = "稀疏" if isinstance(part, SparseArray) else "标签"
-            total = part.nans + part.non_nans
-            return text.format(tag, 100 * part.non_nans / total)
-        else:
-            # MISSING, N/A
-            return ""
-
-    def sp(n):
-        if n == 0:
-            return "0"
-        else:
-            return str(n)
-        
-
-    text += [("%s 个特征" % sp(len(summary.domain.attributes)))
-             + format_part(summary.X)]
-
-    if not summary.domain.class_vars:
-        text += ["没有目标变量。"]
-    else:
-        if len(summary.domain.class_vars) > 1:
-            c_text = "%s结果" % sp(len(summary.domain.class_vars))
-        elif summary.domain.has_continuous_class:
-            c_text = "连续目标变量"
-        else:
-            c_text = "具有 %s 个值的离散类" % sp(
-                len(summary.domain.class_var.values))
-        c_text += format_part(summary.Y)
-        text += [c_text]
-
-    text += [("%s 个元属性" % sp(len(summary.domain.metas)))
-             + format_part(summary.M)]
-
-    return text
-
-
 def is_sortable(table):
     if isinstance(table, SqlTable):
         return False
@@ -1063,21 +848,6 @@ def is_sortable(table):
         return False
 
 
-def test_model():
-    app = QApplication([])
-    view = QTableView(
-        sortingEnabled=True
-    )
-    data = Orange.data.Table("lenses")
-    model = TableModel(data)
-
-    view.setModel(model)
-
-    view.show()
-    view.raise_()
-    return app.exec()
-
-
 if __name__ == "__main__":  # pragma: no cover
     WidgetPreview(OWDataTable).run(
         [(Table("iris"), "iris"),
