@@ -1,16 +1,22 @@
 # Test methods with long descriptive names can omit docstrings
 # pylint: disable=all
 import pickle
+import unittest
 from itertools import product
 from unittest import TestCase
+from unittest.mock import Mock, patch
 
 import numpy as np
 from numpy.testing import assert_array_equal
+import pandas as pd
 
 from AnyQt.QtCore import QItemSelectionModel, Qt, QItemSelection
-from AnyQt.QtWidgets import QAction, QComboBox, QLineEdit, QStyleOptionViewItem
+from AnyQt.QtWidgets import QAction, QComboBox, QLineEdit, \
+    QStyleOptionViewItem, QDialog, QMenu
 from AnyQt.QtTest import QTest, QSignalSpy
 
+from orangewidget.widget import StateInfo
+
 from Orange.widgets.utils import colorpalettes
 from orangewidget.tests.utils import simulate
 from orangewidget.utils.itemmodels import PyListModel
@@ -24,38 +30,45 @@ from Orange.widgets.data.oweditdomain import (
     OWEditDomain,
     ContinuousVariableEditor, DiscreteVariableEditor, VariableEditor,
     TimeVariableEditor, Categorical, Real, Time, String,
-    Rename, Annotate, CategoriesMapping, ChangeOrdered, report_transform,
+    Rename, Annotate, Unlink, CategoriesMapping, report_transform,
     apply_transform, apply_transform_var, apply_reinterpret, MultiplicityRole,
     AsString, AsCategorical, AsContinuous, AsTime,
     table_column_data, ReinterpretVariableEditor, CategoricalVector,
     VariableEditDelegate, TransformRole,
     RealVector, TimeVector, StringVector, make_dict_mapper, DictMissingConst,
-    LookupMappingTransform, as_float_or_nan, column_str_repr
-)
+    LookupMappingTransform, as_float_or_nan, column_str_repr, time_parse,
+    GroupItemsDialog)
 from Orange.widgets.data.owcolor import OWColor, ColorRole
 from Orange.widgets.tests.base import WidgetTest, GuiTest
+from Orange.widgets.tests.utils import contextMenu
 from Orange.tests import test_filename, assert_array_nanequal
+from Orange.widgets.utils.state_summary import format_summary_details
 
 MArray = np.ma.MaskedArray
 
 
 class TestReport(TestCase):
     def test_rename(self):
-        var = Real("X", (-1, ""), ())
+        var = Real("X", (-1, ""), (), False)
         tr = Rename("Y")
         val = report_transform(var, [tr])
         self.assertIn("X", val)
         self.assertIn("Y", val)
 
     def test_annotate(self):
-        var = Real("X", (-1, ""), (("a", "1"), ("b", "z")))
+        var = Real("X", (-1, ""), (("a", "1"), ("b", "z")), False)
         tr = Annotate((("a", "2"), ("j", "z")))
         r = report_transform(var, [tr])
         self.assertIn("a", r)
         self.assertIn("b", r)
 
+    def test_unlinke(self):
+        var = Real("X", (-1, ""), (("a", "1"), ("b", "z")), True)
+        r = report_transform(var, [Unlink()])
+        self.assertIn("unlinked", r)
+
     def test_categories_mapping(self):
-        var = Categorical("C", ("a", "b", "c"), ())
+        var = Categorical("C", ("a", "b", "c"), (), False)
         tr = CategoriesMapping(
             (("a", "aa"),
              ("b", None),
@@ -69,7 +82,7 @@ class TestReport(TestCase):
         self.assertIn("<s>", r)
 
     def test_categorical_merge_mapping(self):
-        var = Categorical("C", ("a", "b1", "b2"), ())
+        var = Categorical("C", ("a", "b1", "b2"), (), False)
         tr = CategoriesMapping(
             (("a", "a"),
              ("b1", "b"),
@@ -79,14 +92,8 @@ class TestReport(TestCase):
         r = report_transform(var, [tr])
         self.assertIn('b', r)
 
-    def test_change_ordered(self):
-        var = Categorical("C", ("a", "b"), ())
-        tr = ChangeOrdered(True)
-        r = report_transform(var, [tr])
-        self.assertIn("ordered", r)
-
     def test_reinterpret(self):
-        var = String("T", ())
+        var = String("T", (), False)
         for tr in (AsContinuous(), AsCategorical(), AsTime()):
             t = report_transform(var, [tr])
             self.assertIn("→ (", t)
@@ -110,6 +117,54 @@ class TestOWEditDomain(WidgetTest):
         self.send_signal(self.widget.Inputs.data, None)
         self.assertEqual(self.widget.data, None)
 
+    def test_widget_state(self):
+        """Check if widget clears its state when the input is disconnected"""
+        editor = self.widget.findChild(ContinuousVariableEditor)
+        self.send_signal(self.widget.Inputs.data, self.iris)
+        self.assertEqual(editor.name_edit.text(), "sepal length")
+        self.send_signal(self.widget.Inputs.data, None)
+        self.assertEqual(editor.name_edit.text(), "")
+        self.assertEqual(
+            self.widget.variables_model.index(0).data(Qt.EditRole), None)
+
+        editor = self.widget.findChild(DiscreteVariableEditor)
+        self.send_signal(self.widget.Inputs.data, self.iris)
+        index = self.widget.domain_view.model().index(4)
+        self.widget.variables_view.setCurrentIndex(index)
+        self.assertEqual(editor.name_edit.text(), "iris")
+        self.assertEqual(editor.labels_model.get_dict(), {})
+        self.assertNotEqual(
+            self.widget.variables_model.index(0).data(Qt.EditRole), None)
+        model = editor.values_edit.selectionModel().model()
+        self.assertEqual(model.index(0).data(Qt.EditRole), "Iris-setosa")
+        self.send_signal(self.widget.Inputs.data, None)
+        self.assertEqual(editor.name_edit.text(), "")
+        self.assertEqual(editor.labels_model.get_dict(), {})
+        self.assertEqual(model.index(0).data(Qt.EditRole), None)
+        self.assertEqual(
+            self.widget.variables_model.index(0).data(Qt.EditRole), None)
+
+        editor = self.widget.findChild(TimeVariableEditor)
+        table = Table(test_filename("datasets/cyber-security-breaches.tab"))
+        self.send_signal(self.widget.Inputs.data, table)
+        index = self.widget.domain_view.model().index(4)
+        self.widget.variables_view.setCurrentIndex(index)
+        self.assertEqual(editor.name_edit.text(), "Date_Posted_or_Updated")
+        self.send_signal(self.widget.Inputs.data, None)
+        self.assertEqual(editor.name_edit.text(), "")
+        self.assertEqual(
+            self.widget.variables_model.index(0).data(Qt.EditRole), None)
+
+        editor = self.widget.findChild(VariableEditor)
+        self.send_signal(self.widget.Inputs.data, table)
+        index = self.widget.domain_view.model().index(8)
+        self.widget.variables_view.setCurrentIndex(index)
+        self.assertEqual(editor.var.name, "Business_Associate_Involved")
+        self.send_signal(self.widget.Inputs.data, None)
+        self.assertEqual(editor.var, None)
+        self.assertEqual(
+            self.widget.variables_model.index(0).data(Qt.EditRole), None)
+
     def test_output_data(self):
         """Check data on the output after apply"""
         self.send_signal(self.widget.Inputs.data, self.iris)
@@ -118,17 +173,22 @@ class TestOWEditDomain(WidgetTest):
         np.testing.assert_array_equal(output.Y, self.iris.Y)
         self.assertEqual(output.domain, self.iris.domain)
 
+        self.widget.output_table_name = "Iris 2"
+        self.widget.commit()
+        output = self.get_output(self.widget.Outputs.data)
+        self.assertEqual(output.name, "Iris 2")
+
     def test_input_from_owcolor(self):
         """Check widget's data sent from OWColor widget"""
         owcolor = self.create_widget(OWColor)
-        self.send_signal("Data", self.iris, widget=owcolor)
+        self.send_signal("数据(Data)", self.iris, widget=owcolor)
         disc_model = owcolor.disc_model
         disc_model.setData(disc_model.index(0, 1), (1, 2, 3), ColorRole)
         cont_model = owcolor.cont_model
         palette = list(colorpalettes.ContinuousPalettes.values())[-1]
         cont_model.setData(cont_model.index(1, 1), palette, ColorRole)
-        owcolor_output = self.get_output("Data", owcolor)
-        self.send_signal("Data", owcolor_output)
+        owcolor_output = self.get_output("数据(Data)", owcolor)
+        self.send_signal("数据(Data)", owcolor_output)
         self.assertEqual(self.widget.data, owcolor_output)
         np.testing.assert_equal(self.widget.data.domain.class_var.colors[0],
                                 (1, 2, 3))
@@ -191,6 +251,34 @@ class TestOWEditDomain(WidgetTest):
         output = self.get_output(self.widget.Outputs.data)
         self.assertIsInstance(output, Table)
 
+    def test_unlink(self):
+        var0, var1, var2 = [ContinuousVariable("x", compute_value=Mock()),
+                            ContinuousVariable("y", compute_value=Mock()),
+                            ContinuousVariable("z")]
+        domain = Domain([var0, var1, var2], None)
+        table = Table.from_numpy(domain, np.zeros((5, 3)), np.zeros((5, 0)))
+        self.send_signal(self.widget.Inputs.data, table)
+
+        index = self.widget.domain_view.model().index
+        for i in range(3):
+            self.widget.domain_view.setCurrentIndex(index(i))
+            editor = self.widget.findChild(ContinuousVariableEditor)
+            self.assertIs(editor.unlink_var_cb.isEnabled(), i < 2)
+            editor._set_unlink(i == 1)
+
+        self.widget.commit()
+        out = self.get_output(self.widget.Outputs.data)
+        out0, out1, out2 = out.domain.variables
+        self.assertIs(out0, domain[0])
+        self.assertIsNot(out1, domain[1])
+        self.assertIs(out2, domain[2])
+
+        self.assertIsNotNone(out0.compute_value)
+        self.assertIsNone(out1.compute_value)
+        self.assertIsNone(out2.compute_value)
+
+
+
     def test_time_variable_preservation(self):
         """Test if time variables preserve format specific attributes"""
         table = Table(test_filename("datasets/cyber-security-breaches.tab"))
@@ -207,26 +295,12 @@ class TestOWEditDomain(WidgetTest):
         output = self.get_output(self.widget.Outputs.data)
         self.assertEqual(str(table[0, 4]), str(output[0, 4]))
 
-    def test_change_ordered(self):
-        """Test categorical ordered flag change"""
-        table = Table.from_domain(Domain(
-            [DiscreteVariable("A", values=["a", "b"], ordered=True)]))
-        self.send_signal(self.widget.Inputs.data, table)
-        output = self.get_output(self.widget.Outputs.data)
-        self.assertTrue(output.domain[0].ordered)
-
-        editor = self.widget.findChild(DiscreteVariableEditor)
-        assert isinstance(editor, DiscreteVariableEditor)
-        editor.ordered_cb.setChecked(False)
-        self.widget.commit()
-        output = self.get_output(self.widget.Outputs.data)
-        self.assertFalse(output.domain[0].ordered)
-
     def test_restore(self):
         iris = self.iris
         viris = (
             "Categorical",
-            ("iris", ("Iris-setosa", "Iris-versicolor", "Iris-virginica"), ())
+            ("iris", ("Iris-setosa", "Iris-versicolor", "Iris-virginica"), (),
+             False)
         )
         w = self.widget
 
@@ -244,13 +318,52 @@ class TestOWEditDomain(WidgetTest):
         tr = model.data(model.index(4), TransformRole)
         self.assertEqual(tr, [AsString(), Rename("Z")])
 
+    def test_summary(self):
+        """Check if status bar is updated when data is received"""
+        data = Table("iris")
+        input_sum = self.widget.info.set_input_summary = Mock()
+        output_sum = self.widget.info.set_output_summary = Mock()
+
+        self.send_signal(self.widget.Inputs.data, data)
+        input_sum.assert_called_with(len(data), format_summary_details(data))
+        output = self.get_output(self.widget.Outputs.data)
+        output_sum.assert_called_with(len(output),
+                                      format_summary_details(output))
+
+        def enter_text(widget, text):
+            # type: (QLineEdit, str) -> None
+            widget.selectAll()
+            QTest.keyClick(widget, Qt.Key_Delete)
+            QTest.keyClicks(widget, text)
+            QTest.keyClick(widget, Qt.Key_Return)
+
+        editor = self.widget.findChild(ContinuousVariableEditor)
+        enter_text(editor.name_edit, "sepal height")
+        self.widget.commit()
+        output = self.get_output(self.widget.Outputs.data)
+        output_sum.assert_called_with(len(output),
+                                      format_summary_details(output))
+        output_sum.reset_mock()
+        enter_text(editor.name_edit, "sepal width")
+        self.widget.commit()
+        output_sum.assert_called_once()
+        self.assertIsInstance(output_sum.call_args[0][0], StateInfo.Empty)
+
+        input_sum.reset_mock()
+        output_sum.reset_mock()
+        self.send_signal(self.widget.Inputs.data, None)
+        input_sum.assert_called_once()
+        self.assertIsInstance(input_sum.call_args[0][0], StateInfo.Empty)
+        output_sum.assert_called_once()
+        self.assertIsInstance(output_sum.call_args[0][0], StateInfo.Empty)
+
 
 class TestEditors(GuiTest):
     def test_variable_editor(self):
         w = VariableEditor()
         self.assertEqual(w.get_data(), (None, []))
 
-        v = String("S", (("A", "1"), ("B", "b")))
+        v = String("S", (("A", "1"), ("B", "b")), False)
         w.set_data(v, [])
 
         self.assertEqual(w.name_edit.text(), v.name)
@@ -275,7 +388,7 @@ class TestEditors(GuiTest):
         w = ContinuousVariableEditor()
         self.assertEqual(w.get_data(), (None, []))
 
-        v = Real("X", (-1, ""), (("A", "1"), ("B", "b")))
+        v = Real("X", (-1, ""), (("A", "1"), ("B", "b")), False)
         w.set_data(v, [])
 
         self.assertEqual(w.name_edit.text(), v.name)
@@ -290,14 +403,14 @@ class TestEditors(GuiTest):
         w = DiscreteVariableEditor()
         self.assertEqual(w.get_data(), (None, []))
 
-        v = Categorical("C", ("a", "b", "c"), (("A", "1"), ("B", "b")))
-        w.set_data(v)
+        v = Categorical("C", ("a", "b", "c"), (("A", "1"), ("B", "b")), False)
+        values = [0, 0, 0, 1, 1, 2]
+        w.set_data_categorical(v, values)
 
         self.assertEqual(w.name_edit.text(), v.name)
-        self.assertFalse(w.ordered_cb.isChecked())
         self.assertEqual(w.labels_model.get_dict(), dict(v.annotations))
         self.assertEqual(w.get_data(), (v, []))
-        w.set_data(None)
+        w.set_data_categorical(None, None)
         self.assertEqual(w.name_edit.text(), "")
         self.assertEqual(w.labels_model.get_dict(), {})
         self.assertEqual(w.get_data(), (None, []))
@@ -307,17 +420,12 @@ class TestEditors(GuiTest):
             ("b", None),
             (None, "b")
         ]
-        w.set_data(v, [CategoriesMapping(mapping)])
+        w.set_data_categorical(v, values, [CategoriesMapping(mapping)])
         w.grab()  # run delegate paint method
         self.assertEqual(w.get_data(), (v, [CategoriesMapping(mapping)]))
 
-        w.set_data(v, [CategoriesMapping(mapping), ChangeOrdered(True)])
-        self.assertTrue(w.ordered_cb.isChecked())
-        self.assertEqual(
-            w.get_data()[1], [CategoriesMapping(mapping), ChangeOrdered(True)]
-        )
         # test selection/deselection in the view
-        w.set_data(v)
+        w.set_data_categorical(v, values)
         view = w.values_edit
         model = view.model()
         assert model.rowCount()
@@ -332,7 +440,7 @@ class TestEditors(GuiTest):
             ("b", "b"),
             ("c", "b")
         ]
-        w.set_data(v, [CategoriesMapping(mapping)])
+        w.set_data_categorical(v, values, [CategoriesMapping(mapping)])
         self.assertEqual(w.get_data()[1], [CategoriesMapping(mapping)])
         self.assertEqual(model.data(model.index(0, 0), MultiplicityRole), 1)
         self.assertEqual(model.data(model.index(1, 0), MultiplicityRole), 2)
@@ -347,8 +455,9 @@ class TestEditors(GuiTest):
     def test_discrete_editor_add_remove_action(self):
         w = DiscreteVariableEditor()
         v = Categorical("C", ("a", "b", "c"),
-                        (("A", "1"), ("B", "b")))
-        w.set_data(v)
+                        (("A", "1"), ("B", "b")), False)
+        values = [0, 0, 0, 1, 1, 2]
+        w.set_data_categorical(v, values)
         action_add = w.add_new_item
         action_remove = w.remove_item
         view = w.values_edit
@@ -382,13 +491,27 @@ class TestEditors(GuiTest):
         self.assertGreaterEqual(len(changedspy), 1, "Did not change data")
         w.grab()
 
+    # mocking exec_ make dialog never show - dialog blocks running until closed
+    @patch(
+        "Orange.widgets.data.oweditdomain.GroupItemsDialog.exec_",
+        Mock(side_effect=lambda: QDialog.Accepted)
+    )
     def test_discrete_editor_merge_action(self):
+        """
+        This function check whether results of dialog have effect on
+        merging the attributes. The dialog itself is tested separately.
+        """
         w = DiscreteVariableEditor()
         v = Categorical("C", ("a", "b", "c"),
-                        (("A", "1"), ("B", "b")))
-        w.set_data(v)
-        action = w.merge_items
-        self.assertFalse(action.isEnabled())
+                        (("A", "1"), ("B", "b")), False)
+
+        w.set_data_categorical(
+            v, [0, 0, 0, 1, 1, 2],
+            [CategoriesMapping([
+                ("a", "AA"), ("b", "BB"), ("c", "CC"),
+            ])]
+        )
+
         view = w.values_edit
         model = view.model()
         selmodel = view.selectionModel()  # type: QItemSelectionModel
@@ -396,26 +519,66 @@ class TestEditors(GuiTest):
             QItemSelection(model.index(0, 0), model.index(1, 0)),
             QItemSelectionModel.ClearAndSelect
         )
-        self.assertTrue(action.isEnabled())
-        # trigger the action, then find the active popup, and simulate entry
         spy = QSignalSpy(w.variable_changed)
         w.merge_items.trigger()
-        cb = w.findChild(QComboBox)
-        cb.setCurrentText("BA")
-        cb.activated[str].emit("BA")
-        cb.close()
-        self.assertEqual(model.index(0, 0).data(Qt.EditRole), "BA")
-        self.assertEqual(model.index(1, 0).data(Qt.EditRole), "BA")
+
+        self.assertEqual(model.index(0, 0).data(Qt.EditRole), "other")
+        self.assertEqual(model.index(1, 0).data(Qt.EditRole), "other")
+        self.assertEqual(model.index(2, 0).data(Qt.EditRole), "CC")
 
         self.assertSequenceEqual(
             list(spy), [[]], 'variable_changed should emit exactly once'
         )
 
+    def test_discrete_editor_rename_selected_items_action(self):
+        w = DiscreteVariableEditor()
+        v = Categorical("C", ("a", "b", "c"),
+                        (("A", "1"), ("B", "b")), False)
+        w.set_data_categorical(v, [])
+        action = w.rename_selected_items
+        view = w.values_edit
+        model = view.model()
+        selmodel = view.selectionModel()  # type: QItemSelectionModel
+        selmodel.select(
+            QItemSelection(model.index(0, 0), model.index(1, 0)),
+            QItemSelectionModel.ClearAndSelect
+        )
+        # trigger the action, then find the active popup, and simulate entry
+        spy = QSignalSpy(w.variable_changed)
+        with patch.object(QComboBox, "setVisible", return_value=None) as m:
+            action.trigger()
+            m.assert_called()
+        cb = view.findChild(QComboBox)
+        cb.setCurrentText("BA")
+        view.commitData(cb)
+        self.assertEqual(model.index(0, 0).data(Qt.EditRole), "BA")
+        self.assertEqual(model.index(1, 0).data(Qt.EditRole), "BA")
+        self.assertSequenceEqual(
+            list(spy), [[]], 'variable_changed should emit exactly once'
+        )
+
+    def test_discrete_editor_context_menu(self):
+        w = DiscreteVariableEditor()
+        v = Categorical("C", ("a", "b", "c"),
+                        (("A", "1"), ("B", "b")), False)
+        w.set_data_categorical(v, [])
+        view = w.values_edit
+        model = view.model()
+
+        pos = view.visualRect(model.index(0, 0)).center()
+        with patch.object(QMenu, "setVisible", return_value=None) as m:
+            contextMenu(view.viewport(), pos)
+            m.assert_called()
+
+        menu = view.findChild(QMenu)
+        self.assertIsNotNone(menu)
+        menu.close()
+
     def test_time_editor(self):
         w = TimeVariableEditor()
         self.assertEqual(w.get_data(), (None, []))
 
-        v = Time("T", (("A", "1"), ("B", "b")))
+        v = Time("T", (("A", "1"), ("B", "b")), False)
         w.set_data(v,)
 
         self.assertEqual(w.name_edit.text(), v.name)
@@ -428,19 +591,19 @@ class TestEditors(GuiTest):
 
     DataVectors = [
         CategoricalVector(
-            Categorical("A", ("a", "aa"), ()), lambda:
+            Categorical("A", ("a", "aa"), (), False), lambda:
                 MArray([0, 1, 2], mask=[False, False, True])
         ),
         RealVector(
-            Real("B", (6, "f"), ()), lambda:
+            Real("B", (6, "f"), (), False), lambda:
                 MArray([0.1, 0.2, 0.3], mask=[True, False, True])
         ),
         TimeVector(
-            Time("T", ()), lambda:
+            Time("T", (), False), lambda:
                 MArray([0, 100, 200], dtype="M8[us]", mask=[True, False, True])
         ),
         StringVector(
-            String("S", ()), lambda:
+            String("S", (), False), lambda:
                 MArray(["0", "1", "2"], dtype=object, mask=[True, False, True])
         ),
     ]
@@ -483,6 +646,44 @@ class TestEditors(GuiTest):
             w.set_data(vec, [Rename("Z")])
             simulate.combobox_run_through_all(tc, callback=cb)
 
+    def test_unlink(self):
+        w = ContinuousVariableEditor()
+        cbox = w.unlink_var_cb
+        self.assertEqual(w.get_data(), (None, []))
+
+        v = Real("X", (-1, ""), (("A", "1"), ("B", "b")), False)
+        w.set_data(v, [])
+        self.assertFalse(cbox.isEnabled())
+
+        v = Real("X", (-1, ""), (("A", "1"), ("B", "b")), True)
+        w.set_data(v, [Unlink()])
+        self.assertTrue(cbox.isEnabled())
+        self.assertTrue(cbox.isChecked())
+
+        v = Real("X", (-1, ""), (("A", "1"), ("B", "b")), True)
+        w.set_data(v, [])
+        self.assertTrue(cbox.isEnabled())
+        self.assertFalse(cbox.isChecked())
+
+        cbox.setChecked(True)
+        self.assertEqual(w.get_data()[1], [Unlink()])
+
+        w.set_data(v, [Unlink()])
+        self.assertTrue(cbox.isChecked())
+
+        cbox.setChecked(False)
+        self.assertEqual(w.get_data()[1], [])
+
+        cbox.setChecked(True)
+        w.clear()
+        self.assertFalse(cbox.isChecked())
+        self.assertEqual(w.get_data()[1], [])
+
+        w._set_unlink(True)
+        self.assertTrue(cbox.isChecked())
+        w._set_unlink(False)
+        self.assertFalse(cbox.isChecked())
+
 
 class TestDelegates(GuiTest):
     def test_delegate(self):
@@ -496,7 +697,7 @@ class TestDelegates(GuiTest):
             delegate.initStyleOption(opt, model.index(0))
             return opt
 
-        set_item({Qt.EditRole: Categorical("a", (), ())})
+        set_item({Qt.EditRole: Categorical("a", (), (), False)})
         delegate = VariableEditDelegate()
         opt = get_style_option()
         self.assertEqual(opt.text, "a")
@@ -552,11 +753,6 @@ class TestTransforms(TestCase):
             DD.compute_value, Lookup(D, np.array([2, 3, 1, 0]))
         )
 
-    def test_ordered_change(self):
-        D = DiscreteVariable("D", values=("a", "b"), ordered=True)
-        Do = apply_transform_var(D, [ChangeOrdered(False)])
-        self.assertFalse(Do.ordered)
-
     def test_discrete_add_drop(self):
         D = DiscreteVariable("D", values=("2", "3", "1", "0"))
         mapping = (
@@ -600,8 +796,8 @@ class TestReinterpretTransforms(TestCase):
     def setUpClass(cls) -> None:
         super().setUpClass()
         domain = Domain([
-            DiscreteVariable("A", values=["a", "b", "c"]),
-            DiscreteVariable("B", values=["0", "1", "2"]),
+            DiscreteVariable("A", values=("a", "b", "c")),
+            DiscreteVariable("B", values=("0", "1", "2")),
             ContinuousVariable("C"),
             TimeVariable("D", have_time=True),
         ],
@@ -665,13 +861,13 @@ class TestReinterpretTransforms(TestCase):
                 [1, 0, 0, 0],
             ], dtype=float)
         )
-        self.assertEqual(tdomain["A"].values, ["a", "b", "c"])
-        self.assertEqual(tdomain["B"].values, ["0", "1", "2"])
-        self.assertEqual(tdomain["C"].values, ["0.0", "0.2", "0.25", "1.25"])
+        self.assertEqual(tdomain["A"].values, ("a", "b", "c"))
+        self.assertEqual(tdomain["B"].values, ("0", "1", "2"))
+        self.assertEqual(tdomain["C"].values, ("0.0", "0.2", "0.25", "1.25"))
         self.assertEqual(
             tdomain["D"].values,
-            ["1970-01-01 00:00:00", "1970-01-01 00:03:00",
-             "1970-01-01 00:06:00", "1970-01-01 00:12:00"]
+            ("1970-01-01 00:00:00", "1970-01-01 00:03:00",
+             "1970-01-01 00:06:00", "1970-01-01 00:12:00")
         )
 
     def test_as_continuous(self):
@@ -720,8 +916,8 @@ class TestReinterpretTransforms(TestCase):
         domain = table.domain
         tvars = []
         for v in domain.metas:
-            for tr in [AsContinuous(), AsCategorical(), AsTime(), AsString()]:
-                tr = apply_reinterpret(v, tr, table_column_data(table, v))
+            for i, tr in enumerate([AsContinuous(), AsCategorical(), AsTime(), AsString()]):
+                tr = apply_reinterpret(v, tr, table_column_data(table, v)).renamed(f'{v.name}_{i}')
                 tvars.append(tr)
         tdomain = Domain([], metas=tvars)
         ttable = table.transform(tdomain)
@@ -793,7 +989,7 @@ class TestUtils(TestCase):
         self.assertEqual(d[1], 1)
         self.assertEqual(d[-1], "<->")
         # must be sufficiently different from defaultdict to warrant existence
-        self.assertEqual(d, {1: 1, 2: 2})
+        self.assertEqual(d, DictMissingConst("<->", {1: 1, 2: 2}))
 
     def test_as_float_or_nan(self):
         a = np.array(["a", "1.1", ".2", "NaN"], object)
@@ -822,6 +1018,19 @@ class TestUtils(TestCase):
         d = column_str_repr(v, np.array([0., np.nan, 1.0]))
         assert_array_equal(d, ["00:00:00", "?", "00:00:01"])
 
+    def test_time_parse(self):
+        """parsing additional datetimes by pandas"""
+        date = ["1/22/20", "1/23/20", "1/24/20"]
+        # we use privet method, check if still exists
+        assert hasattr(pd.DatetimeIndex, '_is_dates_only')
+
+        tval, values = time_parse(date)
+
+        self.assertTrue(tval.have_date)
+        self.assertFalse(tval.have_time)
+        self.assertListEqual(list(values),
+                             [1579651200.0, 1579737600.0, 1579824000.0])
+
 
 class TestLookupMappingTransform(TestCase):
     def setUp(self) -> None:
@@ -843,3 +1052,151 @@ class TestLookupMappingTransform(TestCase):
         assert_array_equal(r, [np.nan, 0, 1, np.nan])
         r_ = lookup_.transform(c)
         assert_array_equal(r_, [np.nan, 0, 1, np.nan])
+
+    def test_equality(self):
+        v1 = DiscreteVariable("v1", values=tuple("abc"))
+        v2 = DiscreteVariable("v1", values=tuple("abc"))
+        v3 = DiscreteVariable("v3", values=tuple("abc"))
+
+        map1 = DictMissingConst(np.nan, {"a": 2, "b": 0, "c": 1})
+        map2 = DictMissingConst(np.nan, {"a": 2, "b": 0, "c": 1})
+        map3 = DictMissingConst(np.nan, {"a": 2, "b": 0, "c": 1})
+
+        t1 = LookupMappingTransform(v1, map1, float)
+        t1a = LookupMappingTransform(v2, map2, float)
+        t2 = LookupMappingTransform(v3, map3, float)
+        self.assertEqual(t1, t1)
+        self.assertEqual(t1, t1a)
+        self.assertNotEqual(t1, t2)
+
+        self.assertEqual(hash(t1), hash(t1a))
+        self.assertNotEqual(hash(t1), hash(t2))
+
+        map1a = DictMissingConst(np.nan, {"a": 2, "b": 1, "c": 0})
+        t1 = LookupMappingTransform(v1, map1, float)
+        t1a = LookupMappingTransform(v1, map1a, float)
+        self.assertNotEqual(t1, t1a)
+        self.assertNotEqual(hash(t1), hash(t1a))
+
+        map1a = DictMissingConst(2, {"a": 2, "b": 0, "c": 1})
+        t1 = LookupMappingTransform(v1, map1, float)
+        t1a = LookupMappingTransform(v1, map1a, float)
+        self.assertNotEqual(t1, t1a)
+        self.assertNotEqual(hash(t1), hash(t1a))
+
+        t1 = LookupMappingTransform(v1, map1, float)
+        t1a = LookupMappingTransform(v1, map1, int)
+        self.assertNotEqual(t1, t1a)
+        self.assertNotEqual(hash(t1), hash(t1a))
+
+
+class TestGroupLessFrequentItemsDialog(GuiTest):
+    def setUp(self) -> None:
+        self.v = Categorical("C", ("a", "b", "c"),
+                        (("A", "1"), ("B", "b")), False)
+        self.data = [0, 0, 0, 1, 1, 2]
+
+    def test_dialog_open(self):
+        dialog = GroupItemsDialog(self.v, self.data, ["a", "b"], {})
+        self.assertTrue(dialog.selected_radio.isChecked())
+        self.assertFalse(dialog.frequent_abs_radio.isChecked())
+        self.assertFalse(dialog.frequent_rel_radio.isChecked())
+        self.assertFalse(dialog.n_values_radio.isChecked())
+
+        dialog = GroupItemsDialog(self.v, self.data, [], {})
+        self.assertFalse(dialog.selected_radio.isChecked())
+        self.assertTrue(dialog.frequent_abs_radio.isChecked())
+        self.assertFalse(dialog.frequent_rel_radio.isChecked())
+        self.assertFalse(dialog.n_values_radio.isChecked())
+
+    def test_group_selected(self):
+        dialog = GroupItemsDialog(self.v, self.data, ["a", "b"], {})
+        dialog.selected_radio.setChecked(True)
+        dialog.new_name_line_edit.setText("BA")
+
+        self.assertListEqual(dialog.get_merge_attributes(), ["a", "b"])
+        self.assertEqual(dialog.get_merged_value_name(), "BA")
+
+    def test_group_less_frequent_abs(self):
+        dialog = GroupItemsDialog(self.v, self.data, ["a", "b"], {})
+        dialog.frequent_abs_radio.setChecked(True)
+        dialog.frequent_abs_spin.setValue(3)
+        dialog.new_name_line_edit.setText("BA")
+
+        self.assertListEqual(dialog.get_merge_attributes(), ["b", "c"])
+        self.assertEqual(dialog.get_merged_value_name(), "BA")
+
+        dialog.frequent_abs_spin.setValue(2)
+        self.assertListEqual(dialog.get_merge_attributes(), ["c"])
+
+        dialog.frequent_abs_spin.setValue(1)
+        self.assertListEqual(dialog.get_merge_attributes(), [])
+
+    def test_group_less_frequent_rel(self):
+        dialog = GroupItemsDialog(self.v, self.data, ["a", "b"], {})
+        dialog.frequent_rel_radio.setChecked(True)
+        dialog.frequent_rel_spin.setValue(50)
+        dialog.new_name_line_edit.setText("BA")
+
+        self.assertListEqual(dialog.get_merge_attributes(), ["b", "c"])
+        self.assertEqual(dialog.get_merged_value_name(), "BA")
+
+        dialog.frequent_rel_spin.setValue(20)
+        self.assertListEqual(dialog.get_merge_attributes(), ["c"])
+
+        dialog.frequent_rel_spin.setValue(15)
+        self.assertListEqual(dialog.get_merge_attributes(), [])
+
+    def test_group_keep_n(self):
+        dialog = GroupItemsDialog(self.v, self.data, ["a", "b"], {})
+        dialog.n_values_radio.setChecked(True)
+        dialog.n_values_spin.setValue(1)
+        dialog.new_name_line_edit.setText("BA")
+
+        self.assertListEqual(dialog.get_merge_attributes(), ["b", "c"])
+        self.assertEqual(dialog.get_merged_value_name(), "BA")
+
+        dialog.n_values_spin.setValue(2)
+        self.assertListEqual(dialog.get_merge_attributes(), ["c"])
+
+        dialog.n_values_spin.setValue(3)
+        self.assertListEqual(dialog.get_merge_attributes(), [])
+
+    def test_group_less_frequent_missing(self):
+        """
+        Widget gives MaskedArray to GroupItemsDialog which can have missing
+        values.
+        gh-4599
+        """
+        def _test_correctness():
+            dialog.frequent_abs_radio.setChecked(True)
+            dialog.frequent_abs_spin.setValue(3)
+            self.assertListEqual(dialog.get_merge_attributes(), ["b", "c"])
+
+            dialog.frequent_rel_radio.setChecked(True)
+            dialog.frequent_rel_spin.setValue(50)
+            self.assertListEqual(dialog.get_merge_attributes(), ["b", "c"])
+
+            dialog.n_values_radio.setChecked(True)
+            dialog.n_values_spin.setValue(1)
+            self.assertListEqual(dialog.get_merge_attributes(), ["b", "c"])
+
+        # masked array
+        data_masked = np.ma.array(
+            [0, 0, np.nan, 0, 1, 1, 2], mask=[0, 0, 1, 0, 0, 0, 0]
+        )
+        dialog = GroupItemsDialog(self.v, data_masked, [], {})
+        _test_correctness()
+
+        data_array = np.array([0, 0, np.nan, 0, 1, 1, 2])
+        dialog = GroupItemsDialog(self.v, data_array, [], {})
+        _test_correctness()
+
+        data_list = [0, 0, None, 0, 1, 1, 2]
+        dialog = GroupItemsDialog(self.v, data_list, [], {})
+        _test_correctness()
+
+
+if __name__ == '__main__':
+    unittest.main()
+
