@@ -2,6 +2,7 @@
 # pylint: disable=missing-docstring
 from os import path, remove, getcwd
 from os.path import dirname
+import unittest
 from unittest.mock import Mock, patch
 import pickle
 import tempfile
@@ -24,6 +25,7 @@ from Orange.data.io import TabReader
 from Orange.tests import named_file
 from Orange.widgets.data.owfile import OWFile
 from Orange.widgets.utils.filedialogs import dialog_formats, format_filter, RecentPath
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.tests.base import WidgetTest
 from Orange.widgets.utils.domaineditor import ComboDelegate, VarTypeDelegate, VarTableModel
 
@@ -128,7 +130,7 @@ class TestOWFile(WidgetTest):
 
         self.open_dataset("iris")
         idx = self.widget.domain_editor.model().createIndex(4, 1)
-        self.widget.domain_editor.model().setData(idx, "text", Qt.EditRole)
+        self.widget.domain_editor.model().setData(idx, '文本数据', Qt.EditRole)
         self.widget.apply_button.click()
         data = self.get_output(self.widget.Outputs.data)
         self.assertIsInstance(data.domain["iris"], StringVariable)
@@ -141,6 +143,22 @@ class TestOWFile(WidgetTest):
         data = self.get_output(self.widget.Outputs.data)
         self.assertIsInstance(data.domain["iris"], StringVariable)
 
+    def test_rename_duplicates(self):
+        self.open_dataset("iris")
+
+        idx = self.widget.domain_editor.model().createIndex(3, 0)
+        self.assertFalse(self.widget.Warning.renamed_vars.is_shown())
+        self.widget.domain_editor.model().setData(idx, "iris", Qt.EditRole)
+        self.widget.apply_button.click()
+        data = self.get_output(self.widget.Outputs.data)
+        self.assertIn("iris (1)", data.domain)
+        self.assertIn("iris (2)", data.domain)
+        self.assertTrue(self.widget.Warning.renamed_vars.is_shown())
+
+        self.widget.domain_editor.model().setData(idx, "different iris", Qt.EditRole)
+        self.widget.apply_button.click()
+        self.assertFalse(self.widget.Warning.renamed_vars.is_shown())
+
     def test_variable_name_change(self):
         """
         Test whether the name of the variable is changed correctly by
@@ -155,11 +173,17 @@ class TestOWFile(WidgetTest):
         data = self.get_output(self.widget.Outputs.data)
         self.assertIn("a", data.domain)
 
+        idx = self.widget.domain_editor.model().createIndex(3, 0)
+        self.widget.domain_editor.model().setData(idx, "d", Qt.EditRole)
+        self.widget.apply_button.click()
+        data = self.get_output(self.widget.Outputs.data)
+        self.assertIn("d", data.domain)
+
         # rename and change to text
         idx = self.widget.domain_editor.model().createIndex(4, 0)
         self.widget.domain_editor.model().setData(idx, "b", Qt.EditRole)
         idx = self.widget.domain_editor.model().createIndex(4, 1)
-        self.widget.domain_editor.model().setData(idx, "text", Qt.EditRole)
+        self.widget.domain_editor.model().setData(idx, "文本数据", Qt.EditRole)
         self.widget.apply_button.click()
         data = self.get_output(self.widget.Outputs.data)
         self.assertIn("b", data.domain)
@@ -170,7 +194,7 @@ class TestOWFile(WidgetTest):
         self.widget.domain_editor.model().setData(idx, "c", Qt.EditRole)
         idx = self.widget.domain_editor.model().createIndex(4, 1)
         self.widget.domain_editor.model().setData(
-            idx, "categorical", Qt.EditRole)
+            idx, '分类数据', Qt.EditRole)
         self.widget.apply_button.click()
         data = self.get_output(self.widget.Outputs.data)
         self.assertIn("c", data.domain)
@@ -181,7 +205,7 @@ class TestOWFile(WidgetTest):
         idx = self.widget.domain_editor.model().createIndex(0, 0)
         self.widget.domain_editor.model().setData(idx, "c", Qt.EditRole)
         idx = self.widget.domain_editor.model().createIndex(0, 1)
-        self.widget.domain_editor.model().setData(idx, "numeric", Qt.EditRole)
+        self.widget.domain_editor.model().setData(idx, '数值数据', Qt.EditRole)
         self.widget.apply_button.click()
         data = self.get_output(self.widget.Outputs.data)
         self.assertIn("c", data.domain)
@@ -217,7 +241,7 @@ class TestOWFile(WidgetTest):
         self.assertEqual(file_name, path.basename(self.widget.last_path()))
         self.assertTrue(self.widget.Error.file_not_found.is_shown())
         self.assertIsNone(self.get_output(self.widget.Outputs.data))
-        self.assertEqual(self.widget.infolabel.text(), "No data.")
+        self.assertEqual(self.widget.infolabel.text(), "无数据.")
 
         # Open a sample dataset
         self.open_dataset("iris")
@@ -250,6 +274,13 @@ class TestOWFile(WidgetTest):
         self.widget.domain_editor.model().setData(idx, "", Qt.EditRole)
         self.assertEqual(self.widget.domain_editor.model().data(idx, Qt.DisplayRole), temp)
 
+    def test_invalid_role_mode(self):
+        self.open_dataset("iris")
+        model = self.widget.domain_editor.model()
+        idx = model.createIndex(1, 0)
+        self.assertFalse(model.setData(idx, Qt.StatusTipRole, ""))
+        self.assertIsNone(model.data(idx, Qt.StatusTipRole))
+
     def test_context_match_includes_variable_values(self):
         file1 = """\
 var
@@ -400,7 +431,7 @@ a
         self.assertTrue(len(data.domain), 5)
         for i in range(5):
             idx = self.widget.domain_editor.model().createIndex(i, 2)
-            self.widget.domain_editor.model().setData(idx, "skip", Qt.EditRole)
+            self.widget.domain_editor.model().setData(idx, "忽略", Qt.EditRole)
         self.widget.apply_button.click()
         data = self.get_output(self.widget.Outputs.data)
         self.assertIsNone(data)
@@ -438,11 +469,11 @@ a
             for i, a in enumerate(data1.domain.attributes):
                 self.assertEqual(str(a), model.data(model.createIndex(i, 0), Qt.DisplayRole))
             # make conversions
-            model.setData(model.createIndex(0, 1), "categorical", Qt.EditRole)
-            model.setData(model.createIndex(1, 1), "text", Qt.EditRole)
-            model.setData(model.createIndex(2, 1), "numeric", Qt.EditRole)
-            model.setData(model.createIndex(3, 1), "numeric", Qt.EditRole)
-            model.setData(model.createIndex(6, 1), "numeric", Qt.EditRole)
+            model.setData(model.createIndex(0, 1), "分类数据", Qt.EditRole)
+            model.setData(model.createIndex(1, 1), "文本数据", Qt.EditRole)
+            model.setData(model.createIndex(2, 1), "数值数据", Qt.EditRole)
+            model.setData(model.createIndex(3, 1), "数值数据", Qt.EditRole)
+            model.setData(model.createIndex(6, 1), "数值数据", Qt.EditRole)
             self.widget.apply_button.click()
             data2 = self.get_output(self.widget.Outputs.data)
             # round continuous values should be converted to integers (3.0 -> 3, "3")
@@ -460,7 +491,7 @@ a
             self.open_dataset(filename)
 
             model = self.widget.domain_editor.model()
-            model.setData(model.createIndex(0, 1), "text", Qt.EditRole)
+            model.setData(model.createIndex(0, 1), '文本数据', Qt.EditRole)
             self.widget.apply_button.click()
 
             data = self.get_output(self.widget.Outputs.data)
@@ -478,8 +509,8 @@ a
             self.open_dataset(filename)
 
             model = self.widget.domain_editor.model()
-            model.setData(model.createIndex(0, 1), "text", Qt.EditRole)
-            model.setData(model.createIndex(1, 1), "numeric", Qt.EditRole)
+            model.setData(model.createIndex(0, 1), '文本数据', Qt.EditRole)
+            model.setData(model.createIndex(1, 1), '数值数据', Qt.EditRole)
             self.widget.apply_button.click()
 
             data = self.get_output(self.widget.Outputs.data)
@@ -514,6 +545,14 @@ a
         self.assertIn("origin", attrs)
         self.assertIn("origin1", attrs["origin"])
 
+    def test_summary(self):
+        """Check if the status bar is updated when data is received"""
+        output_sum = self.widget.info.set_output_summary = Mock()
+        self.open_dataset("iris")
+        output = self.get_output(self.widget.Outputs.data)
+        output_sum.assert_called_with(len(output),
+                                      format_summary_details(output))
+
     @patch("Orange.widgets.widget.OWWidget.workflowEnv",
            Mock(return_value={"basedir": getcwd()}))
     def test_open_moved_workflow(self):
@@ -565,3 +604,6 @@ a
             self.assertEqual(w.recent_paths[0].relpath, base_name)
         finally:
             remove(file_name)
+
+if __name__ == "__main__":
+    unittest.main()
\ No newline at end of file
