@@ -1,6 +1,8 @@
+# pylint: disable=unsubscriptable-object
+import time
 import warnings
 import unittest
-from unittest.mock import patch
+from unittest.mock import patch, Mock
 
 import numpy as np
 from sklearn.exceptions import ConvergenceWarning
@@ -8,6 +10,9 @@ from sklearn.exceptions import ConvergenceWarning
 from AnyQt.QtCore import Qt, QItemSelection
 from AnyQt.QtWidgets import QCheckBox
 
+from orangewidget.widget import StateInfo
+from orangewidget.settings import Context, IncompatibleContext
+
 from Orange.data import Table, Domain, ContinuousVariable, DiscreteVariable
 from Orange.modelling import RandomForestLearner, SGDLearner
 from Orange.preprocess.score import Scorer
@@ -17,6 +22,15 @@ from Orange.projection import PCA
 from Orange.widgets.data.owrank import OWRank, ProblemType, CLS_SCORES, REG_SCORES
 from Orange.widgets.tests.base import WidgetTest, datasets
 from Orange.widgets.widget import AttributeList
+from Orange.widgets.utils.state_summary import format_summary_details
+
+
+class SlowScorer(Scorer):
+    name = "Slow scorer"
+
+    def score_data(self, data, feature=None):
+        time.sleep(1)
+        return np.ones((1, len(data.domain.attributes)))
 
 
 class TestOWRank(WidgetTest):
@@ -41,6 +55,7 @@ class TestOWRank(WidgetTest):
         """Check widget's data after disconnecting data on the input"""
         self.send_signal(self.widget.Inputs.data, self.iris)
         self.assertEqual(self.widget.data, self.iris)
+        self.wait_until_finished()
         self.send_signal(self.widget.Inputs.data, None)
         self.assertEqual(self.widget.data, None)
 
@@ -48,6 +63,7 @@ class TestOWRank(WidgetTest):
         """Check widget's scorer with scorer on the input"""
         self.assertEqual(self.widget.scorers, {})
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 1)
+        self.wait_until_finished()
         value = self.widget.scorers[1]
         self.assertEqual(self.log_reg, value.scorer)
         self.assertIsInstance(value.scorer, Scorer)
@@ -63,12 +79,13 @@ class TestOWRank(WidgetTest):
             with self.subTest(fitter=fitter),\
                     warnings.catch_warnings():
                 warnings.filterwarnings("ignore", ".*", ConvergenceWarning)
-                self.send_signal("Scorer", fitter, 1)
+                self.send_signal("评分器(Scorer)", fitter, 1)
 
                 for data in (self.housing,
                              heart_disease):
                     with self.subTest(data=data.name):
-                        self.send_signal('Data', data)
+                        self.send_signal('数据(Data)', data)
+                        self.wait_until_finished()
                         scores = [model.data(model.index(row, model.columnCount() - 1))
                                   for row in range(model.rowCount())]
                         self.assertEqual(len(scores), len(data.domain.attributes))
@@ -78,7 +95,7 @@ class TestOWRank(WidgetTest):
                             model.columnCount() - 1, Qt.Horizontal).lower()
                         self.assertIn(name, last_column)
 
-                self.send_signal("Scorer", None, 1)
+                self.send_signal("评分器(Scorer)", None, 1)
                 self.assertEqual(self.widget.scorers, {})
 
     def test_input_scorer_disconnect(self):
@@ -91,6 +108,7 @@ class TestOWRank(WidgetTest):
     def test_output_data(self):
         """Check data on the output after apply"""
         self.send_signal(self.widget.Inputs.data, self.iris)
+        self.wait_until_finished()
         output = self.get_output(self.widget.Outputs.reduced_data)
         self.assertIsInstance(output, Table)
         self.assertEqual(len(output.X), len(self.iris))
@@ -101,6 +119,7 @@ class TestOWRank(WidgetTest):
     def test_output_scores(self):
         """Check scores on the output after apply"""
         self.send_signal(self.widget.Inputs.data, self.iris)
+        self.wait_until_finished()
         output = self.get_output(self.widget.Outputs.scores)
         self.assertIsInstance(output, Table)
         self.assertEqual(output.X.shape, (len(self.iris.domain.attributes), 2))
@@ -111,12 +130,14 @@ class TestOWRank(WidgetTest):
         """Check scores on the output after apply with scorer on the input"""
         self.send_signal(self.widget.Inputs.data, self.iris)
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 1)
+        self.wait_until_finished()
         output = self.get_output(self.widget.Outputs.scores)
         self.assertIsInstance(output, Table)
         self.assertEqual(output.X.shape, (len(self.iris.domain.attributes), 5))
 
     def test_output_features(self):
         self.send_signal(self.widget.Inputs.data, self.iris)
+        self.wait_until_finished()
         output = self.get_output(self.widget.Outputs.features)
         self.assertIsInstance(output, AttributeList)
         self.send_signal(self.widget.Inputs.data, None)
@@ -158,8 +179,10 @@ class TestOWRank(WidgetTest):
         """Check scores on the output with inadequate scorer"""
         self.send_signal(self.widget.Inputs.data, self.housing)
         self.send_signal(self.widget.Inputs.scorer, self.pca, 1)
+        self.wait_until_finished()
         with patch("Orange.widgets.data.owrank.log.error") as log:
             self.send_signal(self.widget.Inputs.scorer, self.log_reg, 2)
+            self.wait_until_finished()
             log.assert_called()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.housing.domain.attributes), 16))
@@ -168,8 +191,10 @@ class TestOWRank(WidgetTest):
         """Check scores on the output with inadequate scorer"""
         self.send_signal(self.widget.Inputs.data, self.iris)
         self.send_signal(self.widget.Inputs.scorer, self.pca, 1)
+        self.wait_until_finished()
         with patch("Orange.widgets.data.owrank.log.error") as log:
             self.send_signal(self.widget.Inputs.scorer, self.lin_reg, 2)
+            self.wait_until_finished()
             log.assert_called()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 7))
@@ -178,6 +203,7 @@ class TestOWRank(WidgetTest):
         """Check arbitrary workflow with classification data"""
         self.send_signal(self.widget.Inputs.data, self.iris)
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 5))
         self._get_checkbox('Gini').setChecked(False)
@@ -187,16 +213,20 @@ class TestOWRank(WidgetTest):
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 5))
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 2)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 8))
         self.send_signal(self.widget.Inputs.scorer, None, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 5))
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 8))
         with patch("Orange.widgets.data.owrank.log.error") as log:
             self.send_signal(self.widget.Inputs.scorer, self.lin_reg, 3)
+            self.wait_until_finished()
             log.assert_called()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.attributes), 9))
@@ -205,6 +235,7 @@ class TestOWRank(WidgetTest):
         """Check arbitrary workflow with regression data"""
         self.send_signal(self.widget.Inputs.data, self.housing)
         self.send_signal(self.widget.Inputs.scorer, self.lin_reg, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.housing.domain.attributes), 3))
 
@@ -217,10 +248,12 @@ class TestOWRank(WidgetTest):
                          (len(self.housing.domain.attributes), 3))
 
         self.send_signal(self.widget.Inputs.scorer, None, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.housing.domain.attributes), 2))
 
         self.send_signal(self.widget.Inputs.scorer, self.lin_reg, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.housing.domain.attributes), 3))
 
@@ -229,29 +262,54 @@ class TestOWRank(WidgetTest):
         data = Table.from_table(Domain(self.iris.domain.variables), self.iris)
         self.assertIsNone(data.domain.class_var)
         self.send_signal(self.widget.Inputs.data, data)
+        self.wait_until_finished()
         self.assertIsNone(self.get_output(self.widget.Outputs.scores))
 
         with patch("Orange.widgets.data.owrank.log.error") as log:
             self.send_signal(self.widget.Inputs.scorer, self.lin_reg, 1)
+            self.wait_until_finished()
             log.assert_called()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.variables), 1))
 
         self.send_signal(self.widget.Inputs.scorer, self.pca, 1)
+        self.wait_until_finished()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.variables), 7))
 
         with patch("Orange.widgets.data.owrank.log.error") as log:
             self.send_signal(self.widget.Inputs.scorer, self.lin_reg, 2)
+            self.wait_until_finished()
             log.assert_called()
         self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
                          (len(self.iris.domain.variables), 8))
 
+    def test_no_class_data_learner_class_reg(self):
+        """
+        Check workflow with learners that can be both classifier
+        or regressor and data have no class variable. This test should not
+        fail.
+        """
+        data = Table.from_table(Domain(self.iris.domain.variables), self.iris)
+        random_forest = RandomForestLearner()
+        self.assertIsNone(data.domain.class_var)
+        self.send_signal(self.widget.Inputs.data, data)
+
+        with patch("Orange.widgets.data.owrank.log.error") as log:
+            self.send_signal(self.widget.Inputs.scorer, random_forest, 1)
+            self.wait_until_finished()
+            log.assert_called()
+
+        self.assertEqual(self.get_output(self.widget.Outputs.scores).X.shape,
+                         (len(self.iris.domain.variables), 1))
+
     def test_scores_sorting(self):
         """Check clicking on header column orders scores in a different way"""
         self.send_signal(self.widget.Inputs.data, self.iris)
+        self.wait_until_finished()
         order1 = self.widget.ranksModel.mapToSourceRows(...).tolist()
         self._get_checkbox('FCBF').setChecked(True)
+        self.wait_until_finished()
         self.widget.ranksView.horizontalHeader().setSortIndicator(3, Qt.DescendingOrder)
         order2 = self.widget.ranksModel.mapToSourceRows(...).tolist()
         self.assertNotEqual(order1, order2)
@@ -261,6 +319,7 @@ class TestOWRank(WidgetTest):
         data = self.iris.copy()
         data.get_column_view('petal length')[0][:] = np.nan
         self.send_signal(self.widget.Inputs.data, data)
+        self.wait_until_finished()
 
         # Assert last row is all nan
         for order in (Qt.AscendingOrder,
@@ -271,6 +330,7 @@ class TestOWRank(WidgetTest):
 
     def test_default_sort_indicator(self):
         self.send_signal(self.widget.Inputs.data, self.iris)
+        self.wait_until_finished()
         self.assertNotEqual(
             0, self.widget.ranksView.horizontalHeader().sortIndicatorSection())
 
@@ -322,6 +382,30 @@ class TestOWRank(WidgetTest):
 
         self.assertEqual(w.sorting, (0, Qt.AscendingOrder))
 
+    def test_discard_settings_before_v3(self):
+        for version in (None, 1, 2):
+            self.assertRaises(IncompatibleContext, OWRank.migrate_context,
+                              Context(), version=version)
+
+    def test_auto_selection_manual(self):
+        w = self.widget
+
+        data = Table("heart_disease")
+        dom = data.domain
+        self.send_signal(w.Inputs.data, data)
+        self.wait_until_finished()
+
+        # Sort by number of values and set selection to attributes with most
+        # values. This must select the top 4 rows.
+        self.widget.ranksView.horizontalHeader().setSortIndicator(0, Qt.DescendingOrder)
+        w.selectionMethod = w.SelectManual
+        w.selected_attrs = [dom["chest pain"], dom["rest ECG"],
+                            dom["slope peak exc ST"], dom["thal"]]
+        w.autoSelection()
+        self.assertEqual(
+            sorted({idx.row() for idx in w.ranksView.selectedIndexes()}),
+            [0, 1, 2, 3])
+
     def test_auto_send(self):
         widget = self.widget
         model = widget.ranksModel
@@ -380,6 +464,7 @@ class TestOWRank(WidgetTest):
     def test_selected_rows(self):
         w = self.widget
         self.send_signal(w.Inputs.data, self.iris)
+        self.wait_until_finished()
 
         # select first and second row
         w.selected_rows = [1, 2]
@@ -387,6 +472,41 @@ class TestOWRank(WidgetTest):
 
         self.assertEqual(len(output), len(self.iris))
 
+    def test_summary(self):
+        """Check if the status bar is updated when data is received"""
+        data = self.iris
+        input_sum = self.widget.info.set_input_summary = Mock()
+        output_sum = self.widget.info.set_output_summary = Mock()
+
+        self.send_signal(self.widget.Inputs.data, data)
+        self.wait_until_finished()
+        input_sum.assert_called_with(len(data), format_summary_details(data))
+        output = self.get_output(self.widget.Outputs.reduced_data)
+        output_sum.assert_called_with(len(output),
+                                      format_summary_details(output))
+
+        input_sum.reset_mock()
+        output_sum.reset_mock()
+        self.send_signal(self.widget.Inputs.data, None)
+        self.wait_until_finished()
+        input_sum.assert_called_once()
+        self.assertIsInstance(input_sum.call_args[0][0], StateInfo.Empty)
+        output_sum.assert_called_once()
+        self.assertIsInstance(output_sum.call_args[0][0], StateInfo.Empty)
+
+    def test_concurrent_cancel(self):
+        """
+        Send one signal after another. It test if the first process get
+        correctly canceled when new signal comes.
+        """
+        sc = SlowScorer()
+        self.send_signal(self.widget.Inputs.scorer, sc, 1)
+        self.send_signal(self.widget.Inputs.data, self.iris)
+        self.send_signal(self.widget.Inputs.data, self.housing)
+        self.wait_until_finished()
+        output = self.get_output(self.widget.Outputs.reduced_data)
+        self.assertEqual(len(output), len(self.housing))
+
 
 if __name__ == "__main__":
     unittest.main()
