@@ -7,6 +7,8 @@ from AnyQt.QtWidgets import QListWidget, QSizePolicy
 
 import pyqtgraph as pg
 
+from orangewidget.utils.visual_settings_dlg import VisualSettingsDialog
+
 from Orange.base import Model
 from Orange.classification import ThresholdClassifier, CalibratedLearner
 from Orange.evaluation import Results
@@ -17,6 +19,9 @@ from Orange.widgets.evaluate.contexthandlers import \
 from Orange.widgets.evaluate.utils import results_for_preview
 from Orange.widgets.utils import colorpalettes
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.visualize.utils.customizableplot import \
+    CommonParameterSetter
+from Orange.widgets.visualize.utils.plotutils import AxisItem
 from Orange.widgets.widget import Input, Output, Msg
 from Orange.widgets import report
 
@@ -31,34 +36,63 @@ Metrics = [MetricDefinition(*args) for args in (
     ("F1", (Curves.f1, ), (), ""),
     ("灵敏度和特异度",
      (Curves.sensitivity, Curves.specificity),
-     ("sens", "spec"), ""),
-    #  "<p><b>灵敏度:</b> (降) 是指实际为阳性的样本中，判断为阳性的比例 "
-    #  "(真阳性&nbsp;/&nbsp;阳性 TP&nbsp;/&nbsp;P).</p>"
-    #  "<p><b>特异度:</b> (升) 实际为阴性的样本中，判断为阴性的比例" 
-    #  "(真阴性&nbsp;/&nbsp;阴性 TN&nbsp;/&nbsp;N).</p>"),
+     ("sens", "spec"),
+     "<p><b>灵敏度:</b> (降) 是指实际为阳性的样本中，判断为阳性的比例 "
+     "(真阳性&nbsp;/&nbsp;阳性 TP&nbsp;/&nbsp;P).</p>"
+     "<p><b>特异度:</b> (升) 实际为阴性的样本中，判断为阴性的比例" 
+     "(真阴性&nbsp;/&nbsp;阴性 TN&nbsp;/&nbsp;N).</p>"),
     ("精度和召回",
      (Curves.precision, Curves.recall),
-     ("prec", "recall"), ""),
-    #  "<p><b>精度</b> (升) 检索到的实例中相关实例的比例, 真阳性&nbsp;/&nbsp;(真阳性&nbsp;+&nbsp;假阳性) TP&nbsp;/&nbsp;(TP&nbsp;+&nbsp;FP).</p>"
-    #  "<p><b>召回</b> (降) 检索到的相关实例的比例, 真阳性&nbsp;/&nbsp;阳性 TP&nbsp;/&nbsp;阳性.</p>"),
+     ("prec", "recall"),
+     "<p><b>精度</b> (升) 检索到的实例中相关实例的比例, 真阳性&nbsp;/&nbsp;(真阳性&nbsp;+&nbsp;假阳性) TP&nbsp;/&nbsp;(TP&nbsp;+&nbsp;FP).</p>"
+     "<p><b>召回</b> (降) 检索到的相关实例的比例, 真阳性&nbsp;/&nbsp;阳性 TP&nbsp;/&nbsp;阳性.</p>"),
     ("阳/阴性预测值",
      (Curves.ppv, Curves.npv),
-     ("PPV", "TPV"), ""),
-    #  "<p><b>阳性预测值</b> (升) 正确预测为真的比例, 真阳性&nbsp;/&nbsp;(真阳性&nbsp;+&nbsp;假阳性), TP&nbsp;/&nbsp;(TP&nbsp;+&nbsp;FP).</p>"
-    #  "<p><b>阴性预测值</b> 正确预测为假的比例, 真阴性&nbsp;/&nbsp;(真阴性&nbsp;+&nbsp;假阴性) TN&nbsp;/&nbsp;(TN&nbsp;+&nbsp;FN).</p>"),
+     ("PPV", "TPV"),
+     "<p><b>阳性预测值</b> (升) 正确预测为真的比例, 真阳性&nbsp;/&nbsp;(真阳性&nbsp;+&nbsp;假阳性), TP&nbsp;/&nbsp;(TP&nbsp;+&nbsp;FP).</p>"
+     "<p><b>阴性预测值</b> 正确预测为假的比例, 真阴性&nbsp;/&nbsp;(真阴性&nbsp;+&nbsp;假阴性) TN&nbsp;/&nbsp;(TN&nbsp;+&nbsp;FN).</p>"),
     ("真假阳性率",
      (Curves.tpr, Curves.fpr),
-     ("TPR", "FPR"), ""),
-    #  "<p><b>真假阳性率</b> 检测到的和错检的阳性实例比例</p>"),
+     ("TPR", "FPR"),
+     "<p><b>真假阳性率</b> 检测到的和错检的阳性实例比例</p>"),
 )]
 
 
+class ParameterSetter(CommonParameterSetter):
+
+    def __init__(self, master):
+        super().__init__()
+        self.master = master
+
+    def update_setters(self):
+        self.initial_settings = {
+            self.LABELS_BOX: {
+                self.FONT_FAMILY_LABEL: self.FONT_FAMILY_SETTING,
+                self.TITLE_LABEL: self.FONT_SETTING,
+                self.AXIS_TITLE_LABEL: self.FONT_SETTING,
+                self.AXIS_TICKS_LABEL: self.FONT_SETTING,
+            },
+            self.ANNOT_BOX: {
+                self.TITLE_LABEL: {self.TITLE_LABEL: ("", "")},
+            }
+        }
+
+    @property
+    def title_item(self):
+        return self.master.titleLabel
+
+    @property
+    def axis_items(self):
+        return [value["item"] for value in self.master.axes.values()]
+
+
 class OWCalibrationPlot(widget.OWWidget):
     name = "校准图(Calibration Plot)"
     description = "基于分类器评估的校准图。"
     icon = "icons/CalibrationPlot.svg"
     priority = 1030
-    keywords = []
+    keywords = ['jiaozhun', 'jiaodui']
+    category = 'evaluate'
 
     class Inputs:
         evaluation_results = Input("评估结果(Evaluation Results)", Results, replaces=['Evaluation Results'])
@@ -67,21 +101,22 @@ class OWCalibrationPlot(widget.OWWidget):
         calibrated_model = Output("校准的模型(Calibrated Model)", Model, replaces=['Calibrated Model'])
 
     class Error(widget.OWWidget.Error):
-        non_discrete_target = Msg("Calibration plot requires a discrete target")
+        non_discrete_target = Msg("Calibration plot requires a categorical "
+                                  "target variable.")
         empty_input = widget.Msg("Empty result on input. Nothing to display.")
         nan_classes = \
-            widget.Msg("Remove test data instances with unknown classes")
+            widget.Msg("Remove test data instances with unknown classes.")
         all_target_class = widget.Msg(
-            "All data instances belong to target class")
+            "All data instances belong to target class.")
         no_target_class = widget.Msg(
-            "No data instances belong to target class")
+            "No data instances belong to target class.")
 
     class Warning(widget.OWWidget.Warning):
         omitted_folds = widget.Msg(
-            "Test folds where all data belongs to (non)-target are not shown")
+            "Test folds where all data belongs to (non)-target are not shown.")
         omitted_nan_prob_points = widget.Msg(
             "Instance for which the model couldn't compute probabilities are"
-            "skipped")
+            "skipped.")
         no_valid_data = widget.Msg("No valid data for model(s) {}")
 
     class Information(widget.OWWidget.Information):
@@ -95,6 +130,7 @@ class OWCalibrationPlot(widget.OWWidget):
     fold_curves = settings.Setting(False)
     display_rug = settings.Setting(True)
     threshold = settings.Setting(0.5)
+    visual_settings = settings.Setting({}, schema_only=True)
     auto_commit = settings.Setting(True)
 
     graph_name = "plot"
@@ -114,7 +150,7 @@ class OWCalibrationPlot(widget.OWWidget):
         self.target_cb = gui.comboBox(
             box, self, "target_index", label="目标:",
             orientation=Qt.Horizontal, callback=self.target_index_changed,
-            contentsLength=8)
+            contentsLength=8, searchable=True)
         gui.checkBox(
             box, self, "display_rug", "显示须线(Show rug)",
             callback=self._on_display_rug_changed)
@@ -152,7 +188,10 @@ class OWCalibrationPlot(widget.OWWidget):
         gui.auto_apply(self.controlArea, self, "auto_commit", commit=self.apply)
 
         self.plotview = pg.GraphicsView(background="w")
-        self.plot = pg.PlotItem(enableMenu=False)
+        axes = {"bottom": AxisItem(orientation="bottom"),
+                "left": AxisItem(orientation="left")}
+        self.plot = pg.PlotItem(enableMenu=False, axisItems=axes)
+        self.plot.parameter_setter = ParameterSetter(self.plot)
         self.plot.setMouseEnabled(False, False)
         self.plot.hideButtons()
 
@@ -172,6 +211,8 @@ class OWCalibrationPlot(widget.OWWidget):
         self.mainArea.layout().addWidget(self.plotview)
         self._set_explanation()
 
+        VisualSettingsDialog(self, self.plot.parameter_setter.initial_settings)
+
     @Inputs.evaluation_results
     def set_results(self, results):
         self.closeContext()
@@ -203,7 +244,6 @@ class OWCalibrationPlot(widget.OWWidget):
         self.classifier_names = []
         self.selected_classifiers = []
         self.target_cb.clear()
-        self.target_index = 0
         self.colors = []
 
     def target_index_changed(self):
@@ -257,6 +297,7 @@ class OWCalibrationPlot(widget.OWWidget):
 
         self.selected_classifiers = list(range(n))
         self.target_cb.addItems(results.domain.class_var.values)
+        self.target_index = 0
 
     def _rug(self, data, pen_args):
         color = pen_args["pen"].color()
@@ -430,6 +471,7 @@ class OWCalibrationPlot(widget.OWWidget):
                 text += "</tr>"
             text += "<table>"
             return text
+        return None
 
     def _update_info(self):
         self.info_label.setText(self.get_info_text(short=True))
@@ -493,6 +535,10 @@ class OWCalibrationPlot(widget.OWWidget):
         if self.score != 0:
             self.report_raw(self.get_info_text(short=False))
 
+    def set_visual_settings(self, key, value):
+        self.plot.parameter_setter.set_parameter(key, value)
+        self.visual_settings[key] = value
+
 
 def gaussian_smoother(x, y, sigma=1.0):
     x = np.asarray(x)
