@@ -1,6 +1,7 @@
 """Confusion matrix widget"""
 
 from math import isnan, isinf
+from itertools import chain
 import unicodedata
 
 from AnyQt.QtWidgets import QTableView, QHeaderView, QStyledItemDelegate, \
@@ -11,6 +12,7 @@ import numpy as np
 import sklearn.metrics as skl_metrics
 
 import Orange
+from Orange.data.util import get_unique_names
 import Orange.evaluation
 from Orange.widgets import widget, gui
 from Orange.widgets.settings import \
@@ -18,6 +20,7 @@ from Orange.widgets.settings import \
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_Chinese_NAME)
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Msg, Input, Output
 
 
@@ -85,7 +88,8 @@ class OWConfusionMatrix(widget.OWWidget):
     description = "显示根据分类器评估结果构造的混淆矩阵。"
     icon = "icons/ConfusionMatrix.svg"
     priority = 1001
-    keywords = []
+    keywords = ['hunxiaojuzhen', 'hunxiao', 'juzhen']
+    category = 'evaluate'
 
     class Inputs:
         evaluation_results = Input("评价结果(Evaluation Results)", Orange.evaluation.Results, replaces=['Evaluation Results'])
@@ -145,6 +149,8 @@ class OWConfusionMatrix(widget.OWWidget):
 
         gui.auto_apply(self.outputbox, self, "autocommit", box=False)
 
+        self.info.set_output_summary(self.info.NoOutput)
+
         self.mainArea.layout().setContentsMargins(0, 0, 0, 0)
 
         box = gui.vBox(self.mainArea, box=True)
@@ -191,13 +197,13 @@ class OWConfusionMatrix(widget.OWWidget):
 
     def _init_table(self, nclasses):
         item = self._item(0, 2)
-        item.setData("Predicted", Qt.DisplayRole)
+        item.setData("预测的", Qt.DisplayRole)
         item.setTextAlignment(Qt.AlignCenter)
         item.setFlags(Qt.NoItemFlags)
 
         self._set_item(0, 2, item)
         item = self._item(2, 0)
-        item.setData("Actual", Qt.DisplayRole)
+        item.setData("实际的", Qt.DisplayRole)
         item.setTextAlignment(Qt.AlignHCenter | Qt.AlignBottom)
         item.setFlags(Qt.NoItemFlags)
         self.tableview.setItemDelegateForColumn(0, gui.VerticalItemDelegate())
@@ -285,7 +291,7 @@ class OWConfusionMatrix(widget.OWWidget):
 
         nmodels = results.predicted.shape[0]
         self.headers = class_values + \
-                       [unicodedata.lookup("N-ARY SUMMATION")]
+                       (unicodedata.lookup("N-ARY SUMMATION"), )
 
         # NOTE: The 'learner_names' is set in 'Test Learners' widget.
         self.learners = getattr(
@@ -370,13 +376,16 @@ class OWConfusionMatrix(widget.OWWidget):
         extra = []
         class_var = self.data.domain.class_var
         metas = self.data.domain.metas
+        attrs = self.data.domain.attributes
+        names = [var.name for var in chain(metas, [class_var], attrs)]
 
         if self.append_predictions:
             extra.append(predicted.reshape(-1, 1))
+            proposed = "{}({})".format(class_var.name, learner_name)
+            name = get_unique_names(names, proposed)
             var = Orange.data.DiscreteVariable(
-                "{}({})".format(class_var.name, learner_name),
-                class_var.values
-            )
+                                               name,
+                                               class_var.values)
             metas = metas + (var,)
 
         if self.append_probabilities and \
@@ -414,6 +423,10 @@ class OWConfusionMatrix(widget.OWWidget):
             data = None
             annotated_data = None
 
+        summary = len(data) if data else self.info.NoOutput
+        details = format_summary_details(data) if data else ""
+        self.info.set_output_summary(summary, details)
+
         self.Outputs.selected_data.send(data)
         self.Outputs.annotated_data.send(annotated_data)
 
