@@ -1,6 +1,9 @@
 from collections import namedtuple
+from contextlib import contextmanager
 from functools import partial
 from operator import itemgetter
+from itertools import chain
+from typing import Set, List, Sequence, Union
 
 import numpy
 from AnyQt.QtWidgets import (
@@ -9,21 +12,26 @@ from AnyQt.QtWidgets import (
 from AnyQt.QtGui import QPainter, QStandardItem, QPen, QColor
 from AnyQt.QtCore import (
     Qt, QSize, QRect, QRectF, QPoint, QLocale,
-    QModelIndex, QAbstractTableModel, QSortFilterProxyModel, pyqtSignal)
+    QModelIndex, QAbstractTableModel, QSortFilterProxyModel, pyqtSignal, QTimer,
+    QItemSelectionModel, QItemSelection)
+
+from orangewidget.report import plural
 
 import Orange
 from Orange.evaluation import Results
 from Orange.base import Model
 from Orange.data import ContinuousVariable, DiscreteVariable, Value, Domain
 from Orange.data.table import DomainTransformationError
+from Orange.data.util import get_unique_names
 from Orange.widgets import gui, settings
 from Orange.widgets.evaluate.utils import (
     ScoreTable, usable_scorers, learner_name, scorer_caller)
-from Orange.widgets.utils.colorpalette import ColorPaletteGenerator
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.widget import OWWidget, Msg, Input, Output
 from Orange.widgets.utils.itemmodels import TableModel
 from Orange.widgets.utils.sql import check_sql_input
+from Orange.widgets.utils.state_summary import format_summary_details
+from Orange.widgets.utils.colorpalettes import LimitedDiscretePalette
 
 
 # Input slot for the Predictors channel
@@ -40,7 +48,8 @@ class OWPredictions(OWWidget):
     icon = "icons/Predictions.svg"
     priority = 200
     description = "显示输入数据集的模型预测。"
-    keywords = []
+    keywords = ['yuce', 'moxing']
+    category = 'evaluate'
 
     class Inputs:
         data = Input("数据(Data)", Orange.data.Table, replaces=['Data'])
@@ -64,6 +73,7 @@ class OWPredictions(OWWidget):
 
     #: List of selected class value indices in the `class_values` list
     selected_classes = settings.ContextSetting([])
+    selection = settings.Setting([], schema_only=True)
 
     def __init__(self):
         super().__init__()
@@ -72,11 +82,17 @@ class OWPredictions(OWWidget):
         self.predictors = {}  # type: Dict[object, PredictorSlot]
         self.class_values = []  # type: List[str]
         self._delegates = []
+        self.left_width = 10
+        self.selection_store = None
+        self.__pending_selection = self.selection
+
+        self._set_input_summary()
+        self._set_output_summary(None)
 
         gui.listBox(self.controlArea, self, "selected_classes", "class_values",
                     box="显示概率",
                     callback=self._update_prediction_delegate,
-                    selectionMode=QListWidget.MultiSelection,
+                    selectionMode=QListWidget.ExtendedSelection,
                     addSpace=False,
                     sizePolicy=(QSizePolicy.Preferred, QSizePolicy.Preferred))
         gui.rubber(self.controlArea)
@@ -87,7 +103,7 @@ class OWPredictions(OWWidget):
 
         table_opts = dict(horizontalScrollBarPolicy=Qt.ScrollBarAlwaysOn,
                           horizontalScrollMode=QTableView.ScrollPerPixel,
-                          selectionMode=QTableView.NoSelection,
+                          selectionMode=QTableView.ExtendedSelection,
                           focusPolicy=Qt.StrongFocus)
         self.dataview = TableView(
             sortingEnabled=True,
@@ -109,8 +125,11 @@ class OWPredictions(OWWidget):
             lambda index, _, size:
             self.predictionsview.verticalHeader().resizeSection(index, size))
 
+        self.dataview.setItemDelegate(DataItemDelegate(self.dataview))
+
         self.splitter = QSplitter(
             orientation=Qt.Horizontal, childrenCollapsible=False, handleWidth=2)
+        self.splitter.splitterMoved.connect(self.splitter_resized)
         self.splitter.addWidget(self.predictionsview)
         self.splitter.addWidget(self.dataview)
 
@@ -119,11 +138,19 @@ class OWPredictions(OWWidget):
         self.vsplitter.layout().addWidget(self.splitter)
         self.vsplitter.layout().addWidget(self.score_table.view)
 
+    def get_selection_store(self, proxy):
+        # Both proxies map the same, so it doesn't matter which one is used
+        # to initialize SharedSelectionStore
+        if self.selection_store is None:
+            self.selection_store = SharedSelectionStore(proxy)
+        return self.selection_store
+
     @Inputs.data
     @check_sql_input
     def set_data(self, data):
         self.Warning.empty_data(shown=data is not None and not data)
         self.data = data
+        self.selection_store = None
         if not data:
             self.dataview.setModel(None)
             self.predictionsview.setModel(None)
@@ -134,6 +161,16 @@ class OWPredictions(OWWidget):
             modelproxy = SortProxyModel()
             modelproxy.setSourceModel(model)
             self.dataview.setModel(modelproxy)
+            sel_model = SharedSelectionModel(
+                self.get_selection_store(modelproxy), modelproxy, self.dataview)
+            self.dataview.setSelectionModel(sel_model)
+            if self.__pending_selection is not None:
+                self.selection = self.__pending_selection
+                self.__pending_selection = None
+                self.selection_store.select_rows(
+                    set(self.selection), QItemSelectionModel.ClearAndSelect)
+            sel_model.selectionChanged.connect(self.commit)
+            sel_model.selectionChanged.connect(self._store_selection)
 
             self.dataview.model().list_sorted.connect(
                 partial(
@@ -142,6 +179,9 @@ class OWPredictions(OWWidget):
 
         self._invalidate_predictions()
 
+    def _store_selection(self):
+        self.selection = list(self.selection_store.rows)
+
     @property
     def class_var(self):
         return self.data and self.data.domain.class_var
@@ -184,7 +224,7 @@ class OWPredictions(OWWidget):
         self._update_predictions_model()
         self._update_prediction_delegate()
         self._set_errors()
-        self._update_info()
+        self._set_input_summary()
         self.commit()
 
     def _call_predictors(self):
@@ -290,19 +330,39 @@ class OWPredictions(OWWidget):
         else:
             self.Warning.wrong_targets.clear()
 
-    def _update_info(self):
-        n_predictors = len(self.predictors)
-        if not self.data and not n_predictors:
+    def _set_input_summary(self):
+        if not self.data and not self.predictors:
             self.info.set_input_summary(self.info.NoInput)
             return
 
-        n_valid = len(self._non_errored_predictors())
-        summary = str(len(self.data)) if self.data else "0"
-        details = f"{len(self.data)} instances" if self.data else "No data"
-        details += f"\n{n_predictors} models" if n_predictors else "No models"
-        if n_valid != n_predictors:
-            details += f" ({n_predictors - n_valid} failed)"
-        self.info.set_input_summary(summary, details)
+        summary = len(self.data) if self.data else 0
+        details = self._get_details()
+        self.info.set_input_summary(summary, details, format=Qt.RichText)
+
+    def _get_details(self):
+        details = "Data:<br>"
+        details += format_summary_details(self.data).replace('\n', '<br>') if \
+            self.data else "No data on input."
+        details += "<hr>"
+        pred_names = [v.name for v in self.predictors.values()]
+        n_predictors = len(self.predictors)
+        if n_predictors:
+            n_valid = len(self._non_errored_predictors())
+            details += plural("Model: {number} model{s}", n_predictors)
+            if n_valid != n_predictors:
+                details += f" ({n_predictors - n_valid} failed)"
+            details += "<ul>"
+            for name in pred_names:
+                details += f"<li>{name}</li>"
+            details += "</ul>"
+        else:
+            details += "Model:<br>No model on input."
+        return details
+
+    def _set_output_summary(self, output):
+        summary = len(output) if output else self.info.NoOutput
+        details = format_summary_details(output) if output else ""
+        self.info.set_output_summary(summary, details)
 
     def _invalidate_predictions(self):
         for inputid, pred in list(self.predictors.items()):
@@ -342,10 +402,19 @@ class OWPredictions(OWWidget):
         else:
             model = None
 
+        if self.selection_store is not None:
+            self.selection_store.unregister(
+                self.predictionsview.selectionModel())
+
         predmodel = PredictionsSortProxyModel()
         predmodel.setSourceModel(model)
         predmodel.setDynamicSortFilter(True)
         self.predictionsview.setModel(predmodel)
+
+        self.predictionsview.setSelectionModel(
+            SharedSelectionModel(self.get_selection_store(predmodel),
+                                 predmodel, self.predictionsview))
+
         hheader = self.predictionsview.horizontalHeader()
         hheader.setSortIndicatorShown(False)
         # SortFilterProxyModel is slow due to large abstraction overhead
@@ -360,8 +429,7 @@ class OWPredictions(OWWidget):
 
         self.predictionsview.resizeColumnsToContents()
 
-    @staticmethod
-    def _update_data_sort_order(sort_source_view, sort_dest_view):
+    def _update_data_sort_order(self, sort_source_view, sort_dest_view):
         sort_dest = sort_dest_view.model()
         sort_source = sort_source_view.model()
         sortindicatorshown = False
@@ -383,6 +451,7 @@ class OWPredictions(OWWidget):
             False)
         sort_source_view.horizontalHeader().setSortIndicatorShown(
             sortindicatorshown)
+        self.commit()
 
     def _reset_order(self):
         datamodel = self.dataview.model()
@@ -441,7 +510,7 @@ class OWPredictions(OWWidget):
                 base_values = v
 
         if len(base_color) != len(self.class_values):
-            return ColorPaletteGenerator.palette(len(self.class_values))
+            return LimitedDiscretePalette(len(self.class_values)).palette
         # reorder colors to widgets order
         colors = [None] * len(self.class_values)
         for c, v in zip(base_color, base_values):
@@ -462,7 +531,8 @@ class OWPredictions(OWWidget):
                 None if target.is_continuous else self.class_values,
                 colors,
                 shown_probs,
-                target.format_str if target.is_continuous else None
+                target.format_str if target.is_continuous else None,
+                parent=self.predictionsview
             )
             # QAbstractItemView does not take ownership of delegates, so we must
             self._delegates.append(delegate)
@@ -470,22 +540,24 @@ class OWPredictions(OWWidget):
             self.predictionsview.setColumnHidden(col, False)
 
         self.predictionsview.resizeColumnsToContents()
-        self._update_spliter()
+        self._recompute_splitter_sizes()
         if self.predictionsview.model() is not None:
             self.predictionsview.model().setProbInd(self.selected_classes)
 
-    def _update_spliter(self):
+    def _recompute_splitter_sizes(self):
         if not self.data:
             return
+        view = self.predictionsview
+        self.left_width = \
+            view.horizontalHeader().length() + view.verticalHeader().width()
+        self._update_splitter()
 
-        def width(view):
-            h_header = view.horizontalHeader()
-            v_header = view.verticalHeader()
-            return h_header.length() + v_header.width()
-
-        w = width(self.predictionsview) + 4
+    def _update_splitter(self):
         w1, w2 = self.splitter.sizes()
-        self.splitter.setSizes([w, w1 + w2 - w])
+        self.splitter.setSizes([self.left_width, w1 + w2 - self.left_width])
+
+    def splitter_resized(self):
+        self.left_width = self.splitter.sizes()[0]
 
     def commit(self):
         self._commit_predictions()
@@ -514,6 +586,7 @@ class OWPredictions(OWWidget):
 
     def _commit_predictions(self):
         if not self.data:
+            self._set_output_summary(None)
             self.Outputs.predictions.send(None)
             return
 
@@ -526,14 +599,40 @@ class OWPredictions(OWWidget):
                 self._add_regression_out_columns(slot, newmetas, newcolumns)
 
         attrs = list(self.data.domain.attributes)
-        metas = list(self.data.domain.metas) + newmetas
+        metas = list(self.data.domain.metas)
+        names = [var.name for var in chain(attrs, self.data.domain.class_vars, metas) if var]
+        uniq_newmetas = []
+        for new_ in newmetas:
+            uniq = get_unique_names(names, new_.name)
+            if uniq != new_.name:
+                new_ = new_.copy(name=uniq)
+            uniq_newmetas.append(new_)
+            names.append(uniq)
+
+        metas += uniq_newmetas
         domain = Orange.data.Domain(attrs, self.class_var, metas=metas)
         predictions = self.data.transform(domain)
         if newcolumns:
             newcolumns = numpy.hstack(
                 [numpy.atleast_2d(cols) for cols in newcolumns])
             predictions.metas[:, -newcolumns.shape[1]:] = newcolumns
+
+        index = self.dataview.model().index
+        map_to = self.dataview.model().mapToSource
+        assert self.selection_store is not None
+        rows = None
+        if self.selection_store.rows:
+            rows = [ind.row()
+                    for ind in self.dataview.selectionModel().selectedRows(0)]
+            rows.sort()
+        elif self.dataview.model().isSorted() \
+                or self.predictionsview.model().isSorted():
+            rows = list(range(len(self.data)))
+        if rows:
+            source_rows = [map_to(index(row, 0)).row() for row in rows]
+            predictions = predictions[source_rows]
         self.Outputs.predictions.send(predictions)
+        self._set_output_summary(predictions)
 
     @staticmethod
     def _add_classification_out_columns(slot, newmetas, newcolumns):
@@ -556,10 +655,12 @@ class OWPredictions(OWWidget):
     def send_report(self):
         def merge_data_with_predictions():
             data_model = self.dataview.model()
-            predictions_model = self.predictionsview.model()
+            predictions_view = self.predictionsview
+            predictions_model = predictions_view.model()
 
             # use ItemDelegate to style prediction values
-            style = lambda x: self.predictionsview.itemDelegate().displayText(x, QLocale())
+            delegates = [predictions_view.itemDelegateForColumn(i)
+                         for i in range(predictions_model.columnCount())]
 
             # iterate only over visible columns of data's QTableView
             iter_data_cols = list(filter(lambda x: not self.dataview.isColumnHidden(x),
@@ -575,13 +676,15 @@ class OWPredictions(OWWidget):
             # print data & predictions
             for i in range(data_model.rowCount()):
                 yield [data_model.headerData(i, Qt.Vertical, Qt.DisplayRole)] + \
-                      [style(predictions_model.data(predictions_model.index(i, j)))
-                       for j in range(predictions_model.columnCount())] + \
+                      [delegate.displayText(
+                          predictions_model.data(predictions_model.index(i, j)),
+                          QLocale())
+                       for j, delegate in enumerate(delegates)] + \
                       [data_model.data(data_model.index(i, j))
                        for j in iter_data_cols]
 
         if self.data:
-            text = self.infolabel.text().replace('\n', '<br>')
+            text = self._get_details().replace('\n', '<br>')
             if self.selected_classes:
                 text += '<br>Showing probabilities for: '
                 text += ', '. join([self.class_values[i]
@@ -590,6 +693,25 @@ class OWPredictions(OWWidget):
             self.report_table("Data & Predictions", merge_data_with_predictions(),
                               header_rows=1, header_columns=1)
 
+            self.report_table("Scores", self.score_table.view)
+
+    def resizeEvent(self, event):
+        super().resizeEvent(event)
+        self._update_splitter()
+
+    def showEvent(self, event):
+        super().showEvent(event)
+        QTimer.singleShot(0, self._update_splitter)
+
+
+class DataItemDelegate(QStyledItemDelegate):
+    def initStyleOption(self, option, index):
+        super().initStyleOption(option, index)
+        if self.parent().selectionModel().isSelected(index):
+            option.state |= QStyle.State_Selected \
+                            | QStyle.State_HasFocus \
+                            | QStyle.State_Active
+
 
 class PredictionsItemDelegate(QStyledItemDelegate):
     """
@@ -643,6 +765,11 @@ class PredictionsItemDelegate(QStyledItemDelegate):
 
     def initStyleOption(self, option, index):
         super().initStyleOption(option, index)
+        if self.parent().selectionModel().isSelected(index):
+            option.state |= QStyle.State_Selected \
+                            | QStyle.State_HasFocus \
+                            | QStyle.State_Active
+
         if self.class_values is None:
             option.displayAlignment = \
                 (option.displayAlignment & Qt.AlignVertical_Mask) | \
@@ -872,7 +999,222 @@ class PredictionsModel(QAbstractTableModel):
         return None
 
 
-class TableView(QTableView):
+class SharedSelectionStore:
+    """
+    An object shared between multiple selection models
+
+    The object assumes that the underlying models are proxies.
+    - Method `select` and emit refer to indices in proxy.
+    - Internally, the object stores indices into source model (as int). Method
+      `select_rows` also uses internal, source-model indices.
+
+    The class implements method `select` with the same signature as
+    QItemSelectionModel.select. Selection models that share this object
+    must call this method. After changing the selection, this method will
+    call `emit_selection_rows_changed` of all selection models, so they
+    can emit the signal selectionChanged.
+    """
+    def __init__(self, proxy):
+        # indices of selected rows in the original model, not in the proxy
+        self._rows: Set[int] = set()
+        self.proxy: SortProxyModel = proxy
+        self._selection_models: List[SharedSelectionModel] = []
+
+    @property
+    def rows(self) -> Set[int]:
+        """Indices of selected rows in the source model"""
+        return self._rows
+
+    def register(self, selection_model):
+        """
+        Add a selection mode to the list of models
+
+        Args:
+            selection_model (SharedSelectionModel): a new model
+        """
+        self._selection_models.append(selection_model)
+
+    def unregister(self, selection_model):
+        """
+        Remove a selection mode to the list of models
+
+        Args:
+            selection_model (SharedSelectionModel): a model to remove
+        """
+        if selection_model in self._selection_models:
+            self._selection_models.remove(selection_model)
+
+    def select(self, selection: Union[QModelIndex, QItemSelection], flags: int):
+        """
+        (De)Select given rows
+
+        Args:
+            selection (QModelIndex or QItemSelection):
+                rows to select; indices refer to the proxy model, not the source
+            flags (QItemSelectionModel.SelectionFlags):
+                flags that tell whether to Clear, Select, Deselect or Toggle
+        """
+        if isinstance(selection, QModelIndex):
+            if selection.model() is not None:
+                rows = {selection.model().mapToSource(selection).row()}
+            else:
+                rows = set()
+        else:
+            indices = selection.indexes()
+            if indices:
+                selection = indices[0].model().mapSelectionToSource(selection)
+                rows = {index.row() for index in selection.indexes()}
+            else:
+                rows = set()
+        self.select_rows(rows, flags)
+
+    def select_rows(self, rows: Set[int], flags):
+        """
+        (De)Select given rows
+
+        Args:
+            selection (set of int):
+                rows to select; indices refer to the source model.
+            flags (QItemSelectionModel.SelectionFlags):
+                flags that tell whether to Clear, Select, Deselect or Toggle
+        """
+        with self._emit_changed():
+            if flags & QItemSelectionModel.Clear:
+                self._rows.clear()
+            if flags & QItemSelectionModel.Select:
+                self._rows |= rows
+            if flags & QItemSelectionModel.Deselect:
+                self._rows -= rows
+            if flags & QItemSelectionModel.Toggle:
+                self._rows ^= rows
+
+    def clear_selection(self):
+        """Clear selection and emit changeSelection signal to all models"""
+        with self._emit_changed():
+            self._rows.clear()
+
+    def reset(self):
+        """Clear selection without emiting a signal,"""
+        self._rows.clear()
+
+    @contextmanager
+    def _emit_changed(self):
+        """
+        A context manager that calls `emit_selection_rows_changed after
+        changing a selection.
+        """
+        def map_from_source(rows):
+            from_src = self.proxy.mapFromSource
+            index = self.proxy.sourceModel().index
+            return {from_src(index(row, 0)).row() for row in rows}
+
+        old_rows = self._rows.copy()
+        try:
+            yield
+        finally:
+            if self.proxy.sourceModel() is not None:
+                deselected = map_from_source(old_rows - self._rows)
+                selected = map_from_source(self._rows - old_rows)
+                if selected or deselected:
+                    for model in self._selection_models:
+                        model.emit_selection_rows_changed(selected, deselected)
+
+
+class SharedSelectionModel(QItemSelectionModel):
+    """
+    A selection model that shares the selection with its peers.
+
+    It assumes that the underlying model is a proxy.
+    """
+    def __init__(self, shared_store, proxy, parent):
+        super().__init__(proxy, parent)
+        self.store: SharedSelectionStore = shared_store
+        self.store.register(self)
+
+    def select(self, selection, flags):
+        self.store.select(selection, flags)
+
+    def selection_from_rows(self, rows: Sequence[int],
+                            model=None) -> QItemSelection:
+        """
+        Return selection across all columns for given row indices (as ints)
+
+        Args:
+            rows (sequence of int): row indices (in proxy model)
+
+        Returns: QItemSelection
+        """
+        if model is None:
+            model = self.model()
+        index = model.index
+        last_col = model.columnCount() - 1
+        sel = QItemSelection()
+        for row in rows:
+            sel.select(index(row, 0), index(row, last_col))
+        return sel
+
+    def emit_selection_rows_changed(
+            self, selected: Sequence[int], deselected: Sequence[int]):
+        """
+        Given a sequence of indices of selected and deselected rows,
+        emit a selectionChanged signal. Indices refer to proxy model.
+
+        Args:
+            selected (Sequence[int]): indices of selected rows
+            deselected (Sequence[int]): indices of deselected rows
+        """
+        self.selectionChanged.emit(
+            self.selection_from_rows(selected),
+            self.selection_from_rows(deselected))
+
+    def selection(self):
+        src_sel = self.selection_from_rows(self.store.rows,
+                                           model=self.model().sourceModel())
+        return self.model().mapSelectionFromSource(src_sel)
+
+    def hasSelection(self) -> bool:
+        return bool(self.store.rows)
+
+    def isColumnSelected(self, *_) -> bool:
+        return len(self.store.rows) == self.model().rowCount()
+
+    def isRowSelected(self, row, _parent=None) -> bool:
+        return self.isSelected(self.model().index(row, 0))
+
+    rowIntersectsSelection = isRowSelected
+
+    def isSelected(self, index) -> bool:
+        return self.model().mapToSource(index).row() in self.store.rows
+
+    def selectedColumns(self, row: int):
+        if self.isColumnSelected():
+            index = self.model().index
+            return [index(row, col)
+                    for col in range(self.model().columnCount())]
+        else:
+            return []
+
+    def selectedRows(self, col: int):
+        index = self.model().sourceModel().index
+        map_from = self.model().mapFromSource
+        return [map_from(index(row, col)) for row in self.store.rows]
+
+    def selectedIndexes(self):
+        index = self.model().index
+        rows = [index.row() for index in self.selectedRows(0)]
+        return [index(row, col)
+                for col in range(self.model().columnCount())
+                for row in rows]
+
+    def clearSelection(self):
+        self.store.clear_selection()
+
+    def reset(self):
+        self.store.reset()
+        self.clearCurrentIndex()
+
+
+class TableView(gui.HScrollStepMixin, QTableView):
     MaxSizeHintSamples = 1000
 
     def sizeHintForColumn(self, column):
