@@ -3,6 +3,7 @@ from Orange.widgets import gui
 from pathlib import Path
 from PyQt5.QtCore import QProcess
 from PyQt5.QtWidgets import QScrollArea, QLabel
+import sys
 
 
 class GridWold(OWWidget):
@@ -11,6 +12,8 @@ class GridWold(OWWidget):
     icon = "icons/gridworld.png"
     manual_mode, auto_mode = 0, 1
     mode = manual_mode
+    keywords = ['gezishijie']
+    category = 'reinforcement'
 
     discount = 0.9
     noise_ratio = 0.2
@@ -26,7 +29,10 @@ class GridWold(OWWidget):
 
     dir_path = Path(__file__).resolve()
     parent_path = dir_path.parent.parent
-    command = f'{str(parent_path)}/binaries/gridworld.exe '
+    if sys.platform.startswith('win'):
+        command = f'{str(parent_path)}/binaries/gridworld.exe '
+    else:
+        command = f'{str(parent_path)}/binaries/gridworld '
     command_options = f'-m -n {noise_ratio} -g {grid_type}'
 
     want_main_area = True
@@ -92,7 +98,7 @@ class GridWold(OWWidget):
             settings_box,
             self,
             'living_reward',
-            minv=0,
+            minv=-2,
             maxv=1,
             step=0.1,
             label='生存回报:'
@@ -190,19 +196,6 @@ class GridWold(OWWidget):
         self.final_command = self.command + self.command_options
         print(self.final_command)
 
-    def construct_command(self, mode):
-        options = f'-v -d {self.discount} -r {self.living_reward} -e {self.epsilon} ' \
-                  f'-i {self.iterations} -g {self.grids[self.grid_type]} ' \
-                  f'-a {self.agents[self.agent_type]} -n {self.noise_ratio} ' \
-                  f'-k {self.episodes}'
-        if mode == self.manual_mode:
-            self.command_options = '-m ' + options
-        else:
-            self.command_options = options
-
-        self.final_command = self.command + self.command_options
-        print(self.final_command)
-
     def onstart(self):
         self.commit_button.setEnabled(False)
 
