@@ -5,8 +5,7 @@ from operator import iadd
 import numpy
 
 from AnyQt.QtWidgets import (
-    QFormLayout, QGraphicsRectItem, QGraphicsGridLayout, QApplication,
-    QSizePolicy
+    QGraphicsRectItem, QGraphicsGridLayout, QApplication, QSizePolicy
 )
 from AnyQt.QtGui import QFontMetrics, QPen, QTransform, QFont
 from AnyQt.QtCore import Qt, QRect, QRectF, QPointF
@@ -26,9 +25,13 @@ from Orange.widgets.utils.annotated_data import (create_annotated_table,
 from Orange.widgets.utils.graphicstextlist import TextListWidget
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.widget import Input, Output
-from Orange.widgets.unsupervised.owhierarchicalclustering import (
-    DendrogramWidget
+from Orange.widgets.utils.dendrogram import DendrogramWidget
+from Orange.widgets.utils.state_summary import format_summary_details
+from Orange.widgets.visualize.utils.heatmap import (
+    GradientColorMap, GradientLegendWidget,
 )
+from Orange.widgets.utils.colorgradientselection import ColorGradientSelection
+
 
 def _remove_item(item):
     item.setParentItem(None)
@@ -75,8 +78,6 @@ class DistanceMapItem(pg.ImageItem):
 
         if command & self.Select:
             area = area.normalized()
-            intersects = [rect.intersects(area)
-                          for item, rect in self.__selections]
 
             def partition(predicate, iterable):
                 t1, t2 = itertools.tee(iterable)
@@ -250,7 +251,8 @@ class OWDistanceMap(widget.OWWidget):
     description = "可视化距离矩阵"
     icon = "icons/DistanceMap.svg"
     priority = 1200
-    keywords = []
+    keywords = ['juliyingshe', 'yingshe', 'julitu']
+    category = 'unsupervised'
 
     class Inputs:
         distances = Input("距离(Distances)", Orange.misc.DistMatrix, replaces=['Distances'])
@@ -288,50 +290,45 @@ class OWDistanceMap(widget.OWWidget):
         super().__init__()
 
         self.matrix = None
+        self._matrix_range = 0.
         self._tree = None
         self._ordered_tree = None
         self._sorted_matrix = None
         self._sort_indices = None
         self._selection = None
 
+        self._set_input_summary(None)
+        self._set_output_summary(None)
+
         self.sorting_cb = gui.comboBox(
             self.controlArea, self, "sorting", box="元素排序",
             items=["无", "聚类(Clustering)", "有序叶聚类"],
             callback=self._invalidate_ordering)
 
         box = gui.vBox(self.controlArea, "颜色")
-        self.color_box = gui.palette_combo_box(self.palette_name)
-        self.color_box.currentIndexChanged.connect(self._update_color)
-        box.layout().addWidget(self.color_box)
+        self.color_map_widget = cmw = ColorGradientSelection(
+            thresholds=(self.color_low, self.color_high),
+        )
+        model = itemmodels.ContinuousPalettesModel(parent=self)
+        cmw.setModel(model)
+        idx = cmw.findData(self.palette_name, model.KeyRole)
+        if idx != -1:
+            cmw.setCurrentIndex(idx)
 
-        form = QFormLayout(
-            formAlignment=Qt.AlignLeft,
-            labelAlignment=Qt.AlignLeft,
-            fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow
-        )
-#         form.addRow(
-#             "Gamma",
-#             gui.hSlider(box, self, "color_gamma", minValue=0.0, maxValue=1.0,
-#                         step=0.05, ticks=True, intOnly=False,
-#                         createLabel=False, callback=self._update_color)
-#         )
-        form.addRow(
-            "低:",
-            gui.hSlider(box, self, "color_low", minValue=0.0, maxValue=1.0,
-                        step=0.05, ticks=True, intOnly=False,
-                        createLabel=False, callback=self._update_color)
-        )
-        form.addRow(
-            "高:",
-            gui.hSlider(box, self, "color_high", minValue=0.0, maxValue=1.0,
-                        step=0.05, ticks=True, intOnly=False,
-                        createLabel=False, callback=self._update_color)
-        )
-        box.layout().addLayout(form)
+        cmw.activated.connect(self._update_color)
+
+        def _set_thresholds(low, high):
+            self.color_low, self.color_high = low, high
+            self._update_color()
+
+        cmw.thresholdsChanged.connect(_set_thresholds)
+        box.layout().addWidget(self.color_map_widget)
 
         self.annot_combo = gui.comboBox(
             self.controlArea, self, "annotation_idx", box="注释",
-            callback=self._invalidate_annotations, contentsLength=12)
+            contentsLength=12, searchable=True,
+            callback=self._invalidate_annotations
+        )
         self.annot_combo.setModel(itemmodels.VariableListModel())
         self.annot_combo.model()[:] = ["无", "枚举"]
         self.controlArea.layout().addStretch()
@@ -345,10 +342,14 @@ class OWDistanceMap(widget.OWWidget):
         self.grid = QGraphicsGridLayout()
         self.grid_widget.setLayout(self.grid)
 
+        self.gradient_legend = GradientLegendWidget(0, 1, self._color_map())
+        self.gradient_legend.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
+        self.gradient_legend.setMaximumWidth(250)
+        self.grid.addItem(self.gradient_legend, 0, 1)
         self.viewbox = pg.ViewBox(enableMouse=False, enableMenu=False)
         self.viewbox.setAcceptedMouseButtons(Qt.NoButton)
         self.viewbox.setAcceptHoverEvents(False)
-        self.grid.addItem(self.viewbox, 1, 1)
+        self.grid.addItem(self.viewbox, 2, 1)
 
         self.left_dendrogram = DendrogramWidget(
             self.grid_widget, orientation=DendrogramWidget.Left,
@@ -366,8 +367,8 @@ class OWDistanceMap(widget.OWWidget):
         self.top_dendrogram.setAcceptedMouseButtons(Qt.NoButton)
         self.top_dendrogram.setAcceptHoverEvents(False)
 
-        self.grid.addItem(self.left_dendrogram, 1, 0)
-        self.grid.addItem(self.top_dendrogram, 0, 1)
+        self.grid.addItem(self.left_dendrogram, 2, 0)
+        self.grid.addItem(self.top_dendrogram, 1, 1)
 
         self.right_labels = TextList(
             alignment=Qt.AlignLeft | Qt.AlignVCenter,
@@ -379,11 +380,12 @@ class OWDistanceMap(widget.OWWidget):
             sizePolicy=QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         )
 
-        self.grid.addItem(self.right_labels, 1, 2)
-        self.grid.addItem(self.bottom_labels, 2, 1)
+        self.grid.addItem(self.right_labels, 2, 2)
+        self.grid.addItem(self.bottom_labels, 3, 1)
 
         self.view.setCentralItem(self.grid_widget)
 
+        self.gradient_legend.hide()
         self.left_dendrogram.hide()
         self.top_dendrogram.hide()
         self.right_labels.hide()
@@ -392,8 +394,6 @@ class OWDistanceMap(widget.OWWidget):
         self.matrix_item = None
         self.dendrogram = None
 
-        self.grid_widget.scene().installEventFilter(self)
-
         self.settingsAboutToBePacked.connect(self.pack_settings)
 
     def pack_settings(self):
@@ -407,6 +407,7 @@ class OWDistanceMap(widget.OWWidget):
         self.closeContext()
         self.clear()
         self.error()
+        self._set_input_summary(matrix)
         if matrix is not None:
             N, _ = matrix.shape
             if N < 2:
@@ -415,8 +416,10 @@ class OWDistanceMap(widget.OWWidget):
 
         self.matrix = matrix
         if matrix is not None:
+            self._matrix_range = numpy.nanmax(matrix)
             self.set_items(matrix.row_items, matrix.axis)
         else:
+            self._matrix_range = 0.
             self.set_items(None)
 
         if matrix is not None:
@@ -470,7 +473,6 @@ class OWDistanceMap(widget.OWWidget):
 
     def clear(self):
         self.matrix = None
-        self.cluster = None
         self._tree = None
         self._ordered_tree = None
         self._sorted_matrix = None
@@ -487,6 +489,15 @@ class OWDistanceMap(widget.OWWidget):
                 self.pending_selection = None
         self.unconditional_commit()
 
+    def _set_input_summary(self, matrix):
+        summary = len(matrix) if matrix is not None else self.info.NoInput
+        self.info.set_input_summary(summary)
+
+    def _set_output_summary(self, output):
+        summary = len(output) if output else self.info.NoOutput
+        details = format_summary_details(output) if output else ""
+        self.info.set_output_summary(summary, details)
+
     def _clear_plot(self):
         def remove(item):
             item.setParentItem(None)
@@ -500,6 +511,7 @@ class OWDistanceMap(widget.OWWidget):
 
         self._set_displayed_dendrogram(None)
         self._set_labels(None)
+        self.gradient_legend.hide()
 
     def _cluster_tree(self):
         if self._tree is None:
@@ -608,12 +620,23 @@ class OWDistanceMap(widget.OWWidget):
         self.right_labels.setMaximumWidth(constraint)
         self.bottom_labels.setMaximumHeight(constraint)
 
+    def _color_map(self) -> GradientColorMap:
+        palette = self.color_map_widget.currentData()
+        return GradientColorMap(
+            palette.lookup_table(),
+            thresholds=(self.color_low, max(self.color_high, self.color_low)),
+            span=(0., self._matrix_range))
+
     def _update_color(self):
-        palette = self.color_box.currentData()
+        palette = self.color_map_widget.currentData()
         self.palette_name = palette.name
         if self.matrix_item:
-            colors = palette.lookup_table(self.color_low, self.color_high)
+            cmap = self._color_map().replace(span=(0., 1.))
+            colors = cmap.apply(numpy.arange(256) / 255.)
             self.matrix_item.setLookupTable(colors)
+            self.gradient_legend.show()
+            self.gradient_legend.setRange(0, self._matrix_range)
+            self.gradient_legend.setColorMap(self._color_map())
 
     def _invalidate_selection(self):
         ranges = self.matrix_item.selections()
@@ -645,6 +668,8 @@ class OWDistanceMap(widget.OWWidget):
             subset = [self.items[i] for i in self._selection]
             featuresubset = widget.AttributeList(subset)
 
+        self._set_output_summary(datasubset)
+
         self.Outputs.selected_data.send(datasubset)
         self.Outputs.annotated_data.send(create_annotated_table(self.items, self._selection))
         self.Outputs.features.send(featuresubset)
