@@ -279,6 +279,7 @@ class OWToolbar(OrientedWidget):
                         self.buttons.update(s.buttons)
                         self.groups[buttons[i+1]] = s
                         i = j
+                        self.layout().addStretch()
                         break
                     else:
                         state_buttons.append(buttons[j])
@@ -291,7 +292,6 @@ class OWToolbar(OrientedWidget):
             else:
                 self.buttons[buttons[i][0]] = gui.tool_button(buttons[i], self)
             i = i + 1
-        self.layout().addStretch()
 
     def select_state(self, state):
         # SELECT_RECTANGLE = SELECT
@@ -599,9 +599,9 @@ class OWPlotGUI:
         '''
         self._check_box(widget, 'use_antialiasing', 'Use antialiasing', 'update_antialiasing')
 
-    def jitter_size_slider(self, widget):
+    def jitter_size_slider(self, widget, label="抖动: "):
         return self.add_control(
-            widget, gui.valueSlider, "抖动",
+            widget, gui.valueSlider, label,
             master=self._plot, value='jitter_size',
             values=getattr(self._plot, "jitter_sizes", self.JITTER_SIZES),
             callback=self._plot.update_jittering)
@@ -693,9 +693,9 @@ class OWPlotGUI:
             widget, gui.comboBox, label,
             master=self._master, value=value, items=items, model=model,
             callback=self._get_callback(cb_name, self._master),
-            orientation=Qt.Horizontal, valueType=str,
+            orientation=Qt.Horizontal,
             sendSelectedValue=True, contentsLength=12,
-            labelWidth=50)
+            labelWidth=50, searchable=True)
 
     def color_value_combo(self, widget, label="颜色: "):
         """Creates a combo box that controls point color"""
@@ -903,7 +903,7 @@ class OWPlotGUI:
 
     def theme_combo_box(self, widget):
         c = gui.comboBox(widget, self._plot, "theme_name", "Theme",
-                         callback=self._plot.update_theme, sendSelectedValue=1, valueType=str)
+                         callback=self._plot.update_theme, sendSelectedValue=1)
         c.addItem('Default')
         c.addItem('Light')
         c.addItem('Dark')
