@@ -12,7 +12,7 @@ from AnyQt.QtGui import QColor, QPainter, QPen, QStandardItem
 from AnyQt.QtWidgets import (
     QGraphicsScene, QGraphicsLineItem, QGraphicsItemGroup)
 
-from Orange.data import Table, filter, Variable, Domain
+from Orange.data import Table, filter, Variable, Domain, DiscreteVariable
 from Orange.data.sql.table import SqlTable, LARGE_TABLE, DEFAULT_SAMPLE_TIME
 from Orange.preprocess import Discretize
 from Orange.preprocess.discretize import EqualFreq
@@ -27,6 +27,7 @@ from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_Chinese_NAME)
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.visualize.utils import (
     CanvasText, CanvasRectangle, ViewWithPress, VizRankDialog)
 from Orange.widgets.visualize.utils.plotutils import wrap_legend_items
@@ -36,7 +37,7 @@ from Orange.widgets.widget import OWWidget, Msg, Input, Output
 class MosaicVizRank(VizRankDialog, OWComponent):
     """VizRank dialog for Mosaic"""
     captionTitle = "马赛克排名(Mosaic Ranking)"
-    max_attrs = Setting(3)
+    max_attrs = ContextSetting(6)
 
     pairSelected = Signal(Variable, Variable, Variable, Variable)
     _AttrRole = next(gui.OrangeUserRole)
@@ -47,10 +48,11 @@ class MosaicVizRank(VizRankDialog, OWComponent):
         OWComponent.__init__(self, master)
 
         box = gui.hBox(self)
-        self.max_attr_spin = gui.spin(
-            box, self, "max_attrs", 2, 4,
-            label="将属性数量限制为: ",
-            controlWidth=50, alignment=Qt.AlignRight,
+        self.max_attr_combo = gui.comboBox(
+            box, self, "max_attrs",
+            label="变量数目:", orientation=Qt.Horizontal,
+            items=["1", "2", "3", "4",
+                   "最多 2", "最多 3", "最多 4"],
             callback=self.max_attr_changed)
         gui.rubber(box)
         self.layout().addWidget(self.button)
@@ -86,10 +88,10 @@ class MosaicVizRank(VizRankDialog, OWComponent):
             self.rank_model.clear()
         self.compute_attr_order()
         self.last_run_max_attr = self.max_attrs
-        self.max_attr_spin.setDisabled(True)
+        self.max_attr_combo.setDisabled(True)
 
     def stopped(self):
-        self.max_attr_spin.setDisabled(False)
+        self.max_attr_combo.setDisabled(False)
 
     def max_attr_changed(self):
         """
@@ -99,12 +101,21 @@ class MosaicVizRank(VizRankDialog, OWComponent):
         results until actually restarting the search.
         """
         if self.max_attrs != self.last_run_max_attr or self.saved_state is None:
-            self.button.setText("Start")
+            self.button.setText("开始")
         else:
             self.button.setText("Continue")
         self.button.setEnabled(self.check_preconditions())
 
     def coloring_changed(self):
+        item = self.max_attr_combo.model().item(0)
+        actflags = Qt.ItemIsSelectable | Qt.ItemIsEnabled
+        if self._compute_class_dists():
+            item.setFlags(item.flags() | actflags)
+        else:
+            item.setFlags(item.flags() & ~actflags)
+            if self.max_attrs == 0:
+                self.max_attrs = 1
+
         self.stop_and_reset(self.initialize_keep_ordering)
 
     def check_preconditions(self):
@@ -144,14 +155,22 @@ class MosaicVizRank(VizRankDialog, OWComponent):
     def _compute_class_dists(self):
         return self.master.variable_color is not None
 
+    def attr_range(self):
+        n_attrs = len(self.master.discrete_data.domain.attributes)
+        mm = 1 if self._compute_class_dists() else 2
+        max_attrs = min(n_attrs, [mm, 2, 3, 4, 2, 3, 4][self.max_attrs])
+        min_attrs = [mm, 2, 3, 4, mm, mm, mm][self.max_attrs]
+        return min_attrs, max_attrs
+
     def state_count(self):
         """
         Return the number of combinations, starting with a single attribute
         if Mosaic is colored by class distributions, and two if by Pearson
         """
         n_attrs = len(self.master.discrete_data.domain.attributes)
-        min_attrs = 1 if self._compute_class_dists() else 2
-        max_attrs = min(n_attrs, self.max_attrs)
+        min_attrs, max_attrs = self.attr_range()
+        if min_attrs > max_attrs:
+            return 0
         return sum(comb(n_attrs, k, exact=True)
                    for k in range(min_attrs, max_attrs + 1))
 
@@ -165,16 +184,19 @@ class MosaicVizRank(VizRankDialog, OWComponent):
         # `score_heuristic` would be run on every call to master's `set_data`.
         master = self.master
         data = master.discrete_data
+        min_attrs, max_attrs = self.attr_range()
+        if min_attrs > max_attrs:
+            return
         if state is None:  # on the first call, compute order
             if self._compute_class_dists():
                 self.marginal = get_distribution(data, data.domain.class_var)
                 self.marginal.normalize()
-                state = [0]
+                state = list(range(min_attrs))
             else:
                 self.marginal = get_distributions(data)
                 for dist in self.marginal:
                     dist.normalize()
-                state = [0, 1]
+                state = list(range(min_attrs))
         n_attrs = len(data.domain.attributes)
         while True:
             yield state
@@ -187,7 +209,7 @@ class MosaicVizRank(VizRankDialog, OWComponent):
                     break
                 state[up] = up
             if state[-1] == len(self.attr_ordering):
-                if len(state) < min(self.max_attrs, n_attrs):
+                if len(state) < min(max_attrs, n_attrs):
                     state = list(range(len(state) + 1))
                 else:
                     break
@@ -267,11 +289,12 @@ class MosaicVizRank(VizRankDialog, OWComponent):
 
 
 class OWMosaicDisplay(OWWidget):
-    name = "马赛克显示(Mosaic Display)"
+    name = "马赛克图(Mosaic Display)"
     description = "在马赛克图中显示数据。"
     icon = "icons/MosaicDisplay.svg"
     priority = 220
-    keywords = []
+    keywords = ['masaiketu']
+    category = 'visualize'
 
     class Inputs:
         data = Input("数据(Data)", Table, default=True, replaces=['Data'])
@@ -285,11 +308,11 @@ class OWMosaicDisplay(OWWidget):
     vizrank = SettingProvider(MosaicVizRank)
     settings_version = 2
     use_boxes = Setting(True)
-    variable1 = ContextSetting(None)
-    variable2 = ContextSetting(None)
-    variable3 = ContextSetting(None)
-    variable4 = ContextSetting(None)
-    variable_color = ContextSetting(None)
+    variable1: Variable = ContextSetting(None)
+    variable2: Variable = ContextSetting(None)
+    variable3: Variable = ContextSetting(None)
+    variable4: Variable = ContextSetting(None)
+    variable_color: DiscreteVariable = ContextSetting(None)
     selection = Setting(set(), schema_only=True)
 
     BAR_WIDTH = 5
@@ -324,7 +347,7 @@ class OWMosaicDisplay(OWWidget):
 
         self.areas = []
 
-        self.canvas = QGraphicsScene()
+        self.canvas = QGraphicsScene(self)
         self.canvas_view = ViewWithPress(
             self.canvas, handler=self.clear_selection)
         self.mainArea.layout().addWidget(self.canvas_view)
@@ -332,6 +355,9 @@ class OWMosaicDisplay(OWWidget):
         self.canvas_view.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.canvas_view.setRenderHint(QPainter.Antialiasing)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         box = gui.vBox(self.controlArea, box=True)
         self.model_1 = DomainModel(
             order=DomainModel.MIXED, valid_types=DomainModel.PRIMITIVE)
@@ -342,6 +368,7 @@ class OWMosaicDisplay(OWWidget):
             gui.comboBox(
                 box, self, value="variable{}".format(i),
                 orientation=Qt.Horizontal, contentsLength=12,
+                searchable=True,
                 callback=self.attr_changed,
                 model=self.model_1 if i == 1 else self.model_234)
             for i in range(1, 5)]
@@ -351,10 +378,11 @@ class OWMosaicDisplay(OWWidget):
         box2 = gui.vBox(self.controlArea, box="内部着色")
         self.color_model = DomainModel(
             order=DomainModel.MIXED, valid_types=DomainModel.PRIMITIVE,
-            placeholder="(皮尔逊残值Pearson residuals)")
+            placeholder="(皮尔逊残差)")
         self.cb_attr_color = gui.comboBox(
             box2, self, value="variable_color",
             orientation=Qt.Horizontal, contentsLength=12, labelWidth=50,
+            searchable=True,
             callback=self.set_color_data, model=self.color_model)
         self.bar_button = gui.checkBox(
             box2, self, 'use_boxes', label='与总数比较',
@@ -377,7 +405,9 @@ class OWMosaicDisplay(OWWidget):
             return None
         elif any(attr.is_continuous for attr in data.domain.variables):
             return Discretize(
-                method=EqualFreq(n=4), remove_const=False, discretize_classes=True,
+                method=EqualFreq(n=4),
+                remove_const=False,
+                discretize_classes=True,
                 discretize_metas=True)(data)
         else:
             return data
@@ -440,8 +470,10 @@ class OWMosaicDisplay(OWWidget):
         if self.data is None:
             self.discrete_data = None
             self.init_combos(None)
+            self.info.set_input_summary(self.info.NoInput)
             return
 
+        self.info.set_input_summary(len(data), format_summary_details(data))
         self.init_combos(self.data)
         self.openContext(self.data)
 
@@ -516,6 +548,7 @@ class OWMosaicDisplay(OWWidget):
             self.Outputs.selected_data.send(None)
             self.Outputs.annotated_data.send(
                 create_annotated_table(self.data, []))
+            self.info.set_output_summary(self.info.NoOutput)
             return
         filters = []
         self.Warning.no_cont_selection_sql.clear()
@@ -537,6 +570,10 @@ class OWMosaicDisplay(OWWidget):
         sel_idx = [i for i, id in enumerate(self.data.ids) if id in idset]
         if self.discrete_data is not self.data:
             selection = self.data[sel_idx]
+
+        summary = len(selection) if selection else self.info.NoOutput
+        details = format_summary_details(selection) if selection else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(selection)
         self.Outputs.annotated_data.send(
             create_annotated_table(self.data, sel_idx))
@@ -855,19 +892,11 @@ class OWMosaicDisplay(OWWidget):
             return
         attr_list = self.get_disc_attr_list()
         class_var = data.domain.class_var
-        if class_var:
-            sql = isinstance(data, SqlTable)
-            name = not sql and data.name
-            # save class_var because it is removed in the next line
-            data = data[:, attr_list + [class_var]]
-            data.domain.class_var = class_var
-            if not sql:
-                data.name = name
-        else:
-            data = data[:, attr_list]
         # TODO: check this
         # data = Preprocessor_dropMissing(data)
-        if len(data) == 0:
+
+        unique = [v.name for v in set(attr_list + [class_var]) if v]
+        if len(data[:, unique]) == 0:
             self.Warning.no_valid_data()
             return
         else:
@@ -881,7 +910,8 @@ class OWMosaicDisplay(OWWidget):
                        self.canvas_view.height() / 2)
             return
         if self.variable_color is None:
-            apriori_dists = [get_distribution(data, attr) for attr in attr_list]
+            apriori_dists = [get_distribution(data, attr) for attr
+                             in attr_list]
         else:
             apriori_dists = []
 
@@ -992,5 +1022,5 @@ def get_conditional_distribution(data, attrs):
 
 
 if __name__ == "__main__":  # pragma: no cover
-    data = Table("zoo")
-    WidgetPreview(OWMosaicDisplay).run(data, set_subset_data=data[::10])
+    dataset = Table("zoo")
+    WidgetPreview(OWMosaicDisplay).run(dataset, set_subset_data=dataset[::10])
