@@ -119,9 +119,9 @@ class RadvizVizRank(VizRankDialog, OWComponent):
         anything so the user can still see the results until actually restarting the search.
         """
         if self.n_attrs != self.last_run_n_attrs or self.saved_state is None:
-            self.button.setText("Start")
+            self.button.setText("开始")
         else:
-            self.button.setText("Continue")
+            self.button.setText("继续")
         self.button.setEnabled(self.check_preconditions())
 
     def progressBarSet(self, value, processEvents=None):
@@ -264,6 +264,7 @@ class OWRadvizGraph(OWGraphWithAnchors):
                     label = label[:MAX_LABEL_LEN - 3] + "..."
 
             anchor.setText(label)
+            anchor.setFont(self.parameter_setter.anchor_font)
             label_len = min(MAX_LABEL_LEN, len(label))
             anchor.setColor(QColor(0, 0, 0))
 
@@ -314,6 +315,7 @@ class OWRadviz(OWAnchorProjectionWidget):
     icon = "icons/Radviz.svg"
     priority = 241
     keywords = ["viz"]
+    category = 'visualize'
 
     settings_version = 3
 
@@ -353,6 +355,10 @@ class OWRadviz(OWAnchorProjectionWidget):
     def effective_variables(self):
         return self.selected_vars
 
+    @property
+    def effective_data(self):
+        return self.data.transform(Domain(self.effective_variables))
+
     def vizrank_set_attrs(self, *attrs):
         if not attrs:
             return
