@@ -22,7 +22,8 @@ from Orange.widgets.settings import (
     Setting, ContextSetting, SettingProvider, IncompatibleContext)
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.visualize.owscatterplotgraph import OWScatterPlotBase
+from Orange.widgets.visualize.owscatterplotgraph import OWScatterPlotBase, \
+    ScatterBaseParameterSetter
 from Orange.widgets.visualize.utils import VizRankDialogAttrPair
 from Orange.widgets.visualize.utils.widget import OWDataProjectionWidget
 from Orange.widgets.widget import AttributeList, Msg, Input, Output
@@ -95,12 +96,31 @@ class ScatterPlotVizRank(VizRankDialogAttrPair):
         return [a for _, a in attrs]
 
 
+class ParameterSetter(ScatterBaseParameterSetter):
+
+    def __init__(self, master):
+        super().__init__(master)
+
+    def update_setters(self):
+        super().update_setters()
+        self.initial_settings[self.LABELS_BOX].update({
+            self.AXIS_TITLE_LABEL: self.FONT_SETTING,
+            self.AXIS_TICKS_LABEL: self.FONT_SETTING
+        })
+
+    @property
+    def axis_items(self):
+        return [value["item"] for value in
+                self.master.plot_widget.plotItem.axes.values()]
+
+
 class OWScatterPlotGraph(OWScatterPlotBase):
     show_reg_line = Setting(False)
     orthonormal_regression = Setting(False)
 
     def __init__(self, scatter_widget, parent):
         super().__init__(scatter_widget, parent)
+        self.parameter_setter = ParameterSetter(self)
         self.reg_line_items = []
 
     def clear(self):
@@ -119,6 +139,8 @@ class OWScatterPlotGraph(OWScatterPlotBase):
 
     def update_axes(self):
         for axis, title in self.master.get_axes().items():
+            use_time = title is not None and title.is_time
+            self.plot_widget.plotItem.getAxis(axis).use_time(use_time)
             self.plot_widget.setLabel(axis=axis, text=title or "")
             if title is None:
                 self.plot_widget.hideAxis(axis)
@@ -207,7 +229,8 @@ class OWScatterPlot(OWDataProjectionWidget):
     description = "具有智能数据可视化增强功能的交互式散点图可视化工具。"
     icon = "icons/ScatterPlot.svg"
     priority = 140
-    keywords = []
+    keywords = ['sandiantu']
+    category = 'visualize'
 
     class Inputs(OWDataProjectionWidget.Inputs):
         features = Input("特征(Features)", AttributeList, replaces=['Features'])
@@ -230,8 +253,8 @@ class OWScatterPlot(OWDataProjectionWidget):
     class Warning(OWDataProjectionWidget.Warning):
         missing_coords = Msg(
             "Plot cannot be displayed because '{}' or '{}' "
-            "is missing for all data points")
-        no_continuous_vars = Msg("Data has no continuous variables")
+            "is missing for all data points.")
+        no_continuous_vars = Msg("Data has no numeric variables.")
 
     class Information(OWDataProjectionWidget.Information):
         sampled_sql = Msg("Large SQL table; showing a sample.")
@@ -266,12 +289,13 @@ class OWScatterPlot(OWDataProjectionWidget):
             callback=self.graph.update_regression_line,
             tooltip=
             "If checked, fit line to group (minimize distance from points);\n"
-            "otherwise fit y as a function of x (minimize vertical distances)")
+            "otherwise fit y as a function of x (minimize vertical distances)",
+            disabledBy=self.cb_reg_line)
 
     def _add_controls_axis(self):
         common_options = dict(
             labelWidth=50, orientation=Qt.Horizontal, sendSelectedValue=True,
-            valueType=str, contentsLength=14
+            contentsLength=14
         )
         self.attr_box = gui.vBox(self.controlArea, True)
         dmod = DomainModel
@@ -279,11 +303,13 @@ class OWScatterPlot(OWDataProjectionWidget):
         self.cb_attr_x = gui.comboBox(
             self.attr_box, self, "attr_x", label="x 轴:",
             callback=self.set_attr_from_combo,
-            model=self.xy_model, **common_options)
+            model=self.xy_model, **common_options,
+            searchable=True)
         self.cb_attr_y = gui.comboBox(
             self.attr_box, self, "attr_y", label="y 轴:",
             callback=self.set_attr_from_combo,
-            model=self.xy_model, **common_options)
+            model=self.xy_model, **common_options,
+            searchable=True)
         vizrank_box = gui.hBox(self.attr_box)
         self.vizrank, self.vizrank_button = ScatterPlotVizRank.add_vizrank(
             vizrank_box, self, "查找信息投影(Find Informative Projections)", self.set_attr)
@@ -298,6 +324,13 @@ class OWScatterPlot(OWDataProjectionWidget):
     def effective_variables(self):
         return [self.attr_x, self.attr_y] if self.attr_x and self.attr_y else []
 
+    @property
+    def effective_data(self):
+        eff_var = self.effective_variables
+        if eff_var and self.attr_x.name == self.attr_y.name:
+            eff_var = [self.attr_x]
+        return self.data.transform(Domain(eff_var))
+
     def _vizrank_color_change(self):
         self.vizrank.initialize()
         err_msg = ""
