@@ -3,100 +3,48 @@ import itertools
 import warnings
 from xml.sax.saxutils import escape
 from math import log10, floor, ceil
+from datetime import datetime, timezone
+from time import gmtime
 
 import numpy as np
 from AnyQt.QtCore import Qt, QRectF, QSize, QTimer, pyqtSignal as Signal, \
     QObject
-from AnyQt.QtGui import (
-    QStaticText, QColor, QPen, QBrush, QPainterPath, QTransform, QPainter
-)
-from AnyQt.QtWidgets import (
-    QApplication, QToolTip, QGraphicsTextItem, QGraphicsRectItem
-)
+from AnyQt.QtGui import QColor, QPen, QBrush, QPainterPath, QTransform, \
+    QPainter
+from AnyQt.QtWidgets import QApplication, QToolTip, QGraphicsTextItem, \
+    QGraphicsRectItem, QGraphicsItemGroup
 
 import pyqtgraph as pg
-import pyqtgraph.functions as fn
-from pyqtgraph.graphicsItems.ScatterPlotItem import Symbols, drawSymbol
-from pyqtgraph.graphicsItems.LegendItem import (
-    LegendItem as PgLegendItem, ItemSample
-)
+from pyqtgraph.graphicsItems.ScatterPlotItem import Symbols
+from pyqtgraph.graphicsItems.LegendItem import LegendItem as PgLegendItem
 from pyqtgraph.graphicsItems.TextItem import TextItem
 
+from Orange.preprocess.discretize import _time_binnings
 from Orange.widgets.utils import colorpalettes
 from Orange.util import OrangeDeprecationWarning
 from Orange.widgets import gui
 from Orange.widgets.settings import Setting
 from Orange.widgets.utils import classdensity
 from Orange.widgets.utils.plot import OWPalette
-from Orange.widgets.visualize.owscatterplotgraph_obsolete import (
-    OWScatterPlotGraph as OWScatterPlotGraphObs
-)
+from Orange.widgets.visualize.utils.customizableplot import Updater, \
+    CommonParameterSetter
 from Orange.widgets.visualize.utils.plotutils import (
-    HelpEventDelegate as EventDelegate,
-    InteractiveViewBox as ViewBox
+    HelpEventDelegate as EventDelegate, InteractiveViewBox as ViewBox,
+    PaletteItemSample, SymbolItemSample, AxisItem
 )
 
+with warnings.catch_warnings():
+    # This just loads an obsolete module; proper warning is issued below
+    warnings.simplefilter("ignore", DeprecationWarning)
+    from Orange.widgets.visualize.owscatterplotgraph_obsolete import (
+        OWScatterPlotGraph as OWScatterPlotGraphObs
+    )
 
 SELECTION_WIDTH = 5
 MAX_N_VALID_SIZE_ANIMATE = 1000
 
-
-class PaletteItemSample(ItemSample):
-    """A color strip to insert into legends for discretized continuous values"""
-
-    def __init__(self, palette, scale, label_formatter=None):
-        """
-        :param palette: palette used for showing continuous values
-        :type palette: BinnedContinuousPalette
-        :param scale: an instance of DiscretizedScale that defines the
-                      conversion of values into bins
-        :type scale: DiscretizedScale
-        """
-        super().__init__(None)
-        self.palette = palette
-        self.scale = scale
-        if label_formatter is None:
-            label_formatter = "{{:.{}f}}".format(scale.decimals).format
-        cuts = [label_formatter(scale.offset + i * scale.width)
-                for i in range(scale.bins + 1)]
-        self.labels = [QStaticText("{} - {}".format(fr, to))
-                       for fr, to in zip(cuts, cuts[1:])]
-        font = self.font()
-        font.setPixelSize(11)
-        for label in self.labels:
-            label.prepare(font=font)
-        self.text_width = max(label.size().width() for label in self.labels)
-
-    def boundingRect(self):
-        return QRectF(0, 0, 40 + self.text_width, 20 + self.scale.bins * 15)
-
-    def paint(self, p, *args):
-        p.setRenderHint(p.Antialiasing)
-        p.translate(5, 5)
-        font = p.font()
-        font.setPixelSize(11)
-        p.setFont(font)
-        colors = self.palette.qcolors
-        for i, color, label in zip(itertools.count(), colors, self.labels):
-            p.setPen(Qt.NoPen)
-            p.setBrush(QBrush(color))
-            p.drawRect(0, i * 15, 15, 15)
-            p.setPen(QPen(Qt.black))
-            p.drawStaticText(20, i * 15 + 1, label)
-
-
-class SymbolItemSample(ItemSample):
-    """Adjust position for symbols"""
-    def __init__(self, pen, brush, size, symbol):
-        super().__init__(None)
-        self.__pen = fn.mkPen(pen)
-        self.__brush = fn.mkBrush(brush)
-        self.__size = size
-        self.__symbol = symbol
-
-    def paint(self, p, *args):
-        p.translate(8, 12)
-        drawSymbol(p, self.__symbol, self.__size, self.__pen, self.__brush)
+# maximum number of colors (including Other)
+MAX_COLORS = 11
 
 
 class LegendItem(PgLegendItem):
@@ -125,7 +73,8 @@ class LegendItem(PgLegendItem):
         anchor, parentanchor = anchors
         self.anchor(*bound_anchor_pos(anchor, parentanchor))
 
-    def paint(self, painter, option, widget=None):
+    # pylint: disable=arguments-differ
+    def paint(self, painter, _option, _widget=None):
         painter.setPen(self.__pen)
         painter.setBrush(self.__brush)
         rect = self.contentsRect()
@@ -241,22 +190,81 @@ class OWScatterPlotGraph(OWScatterPlotGraphObs):
 
 
 class ScatterPlotItem(pg.ScatterPlotItem):
-    """PyQtGraph's ScatterPlotItem calls updateSpots at any change of sizes/colors/symbols,
-    which then rebuilds the stored pixmaps for each symbol. Because Orange calls
-    set* function in succession, we postpone updateSpots() to paint()."""
+    """
+    Modifies the behaviour of ScatterPlotItem as follows:
 
-    _update_spots_in_paint = False
+    - Add z-index. ScatterPlotItem paints points in order of appearance in
+      self.data. Plotting by z-index is achieved by sorting before calling
+      super().paint() and re-sorting afterwards. Re-sorting (instead of
+      storing the original data) is needed because the inherited paint
+      may modify the data.
+
+    - Prevent multiple calls to updateSpots. ScatterPlotItem calls updateSpots
+      at any change of sizes/colors/symbols, which then rebuilds the stored
+      pixmaps for each symbol. Orange calls set* functions in succession,
+      so we postpone updateSpots() to paint()."""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._update_spots_in_paint = False
+        self._z_mapping = None
+        self._inv_mapping = None
+
+    def setZ(self, z):
+        """
+        Set z values for all points.
+
+        Points with higher values are plotted on top of those with lower.
+
+        Args:
+            z (np.ndarray or None): a vector of z values
+        """
+        if z is None:
+            self._z_mapping = self._inv_mapping = None
+        else:
+            assert len(z) == len(self.data)
+            self._z_mapping = np.argsort(z)
+            self._inv_mapping = np.argsort(self._z_mapping)
+
+    def setCoordinates(self, x, y):
+        """
+        Change the coordinates of points while keeping other properties.
+
+        Asserts that the number of points stays the same.
+
+        Note. Pyqtgraph does not offer a method for this: setting coordinates
+        invalidates other data. We therefore retrieve the data to set it
+        together with the coordinates. Pyqtgraph also does not offer a
+        (documented) method for retrieving the data, yet using
+        data[prop]` looks reasonably safe.
+
+        The alternative, updating the whole scatterplot from the Orange Table,
+        is too slow.
+        """
+        assert len(self.data) == len(x) == len(y)
+        data = dict(x=x, y=y)
+        for prop in ('pen', 'brush', 'size', 'symbol', 'data'):
+            data[prop] = self.data[prop]
+        self.setData(**data)
 
     def updateSpots(self, dataSet=None):  # pylint: disable=unused-argument
         self._update_spots_in_paint = True
         self.update()
 
+    # pylint: disable=arguments-differ
     def paint(self, painter, option, widget=None):
-        if self._update_spots_in_paint:
-            self._update_spots_in_paint = False
-            super().updateSpots()
-        painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
-        super().paint(painter, option, widget)
+        try:
+            if self._z_mapping is not None:
+                assert len(self._z_mapping) == len(self.data)
+                self.data = self.data[self._z_mapping]
+            if self._update_spots_in_paint:
+                self._update_spots_in_paint = False
+                super().updateSpots()
+            painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
+            super().paint(painter, option, widget)
+        finally:
+            if self._inv_mapping is not None:
+                self.data = self.data[self._inv_mapping]
 
 
 def _define_symbols():
@@ -272,10 +280,26 @@ def _define_symbols():
     path.lineTo(0.5, 0.5)
     Symbols["?"] = path
 
+    path = QPainterPath()
+    plusCoords = [
+        (-0.5, -0.1), (-0.5, 0.1), (-0.1, 0.1), (-0.1, 0.5),
+        (0.1, 0.5), (0.1, 0.1), (0.5, 0.1), (0.5, -0.1),
+        (0.1, -0.1), (0.1, -0.5), (-0.1, -0.5), (-0.1, -0.1)
+    ]
+    path.moveTo(*plusCoords[0])
+    for x, y in plusCoords[1:]:
+        path.lineTo(x, y)
+    path.closeSubpath()
+    Symbols["+"] = path
+
     tr = QTransform()
     tr.rotate(180)
     Symbols['t'] = tr.map(Symbols['t'])
 
+    tr = QTransform()
+    tr.rotate(45)
+    Symbols['x'] = tr.map(Symbols["+"])
+
 
 _define_symbols()
 
@@ -286,6 +310,140 @@ def _make_pen(color, width):
     return p
 
 
+class AxisItem(AxisItem):
+    """
+    Axis that if needed displays ticks appropriate for time data.
+    """
+
+    _label_width = 80
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._use_time = False
+
+    def use_time(self, enable):
+        """Enables axes to display ticks for time data."""
+        self._use_time = enable
+        self.enableAutoSIPrefix(not enable)
+
+    def tickValues(self, minVal, maxVal, size):
+        """Find appropriate tick locations."""
+        if not self._use_time:
+            return super().tickValues(minVal, maxVal, size)
+
+        # if timezone is not set, then local is used which cause exceptions
+        minVal = max(minVal,
+                     datetime.min.replace(tzinfo=timezone.utc).timestamp() + 1)
+        maxVal = min(maxVal,
+                     datetime.max.replace(tzinfo=timezone.utc).timestamp() - 1)
+        mn, mx = gmtime(minVal), gmtime(maxVal)
+
+        try:
+            bins = _time_binnings(mn, mx, 6, 30)[-1]
+        except (IndexError, ValueError):
+            # cannot handle very large and very small time intervals
+            return super().tickValues(minVal, maxVal, size)
+
+        ticks = bins.thresholds
+
+        max_steps = max(int(size / self._label_width), 1)
+        if len(ticks) > max_steps:
+            # remove some of ticks so that they don't overlap
+            step = int(np.ceil(float(len(ticks)) / max_steps))
+            ticks = ticks[::step]
+
+        spacing = min(b - a for a, b in zip(ticks[:-1], ticks[1:]))
+        return [(spacing, ticks)]
+
+    def tickStrings(self, values, scale, spacing):
+        """Format tick values according to space between them."""
+        if not self._use_time:
+            return super().tickStrings(values, scale, spacing)
+
+        if spacing >= 3600 * 24 * 365:
+            fmt = "%Y"
+        elif spacing >= 3600 * 24 * 28:
+            fmt = "%Y %b"
+        elif spacing >= 3600 * 24:
+            fmt = "%Y %b %d"
+        elif spacing >= 3600:
+            fmt = "%d %Hh"
+        elif spacing >= 60:
+            fmt = "%H:%M"
+        elif spacing >= 1:
+            fmt = "%H:%M:%S"
+        else:
+            fmt = '%S.%f'
+
+        # if timezone is not set, then local timezone is used
+        # which cause exceptions for edge cases
+        return [datetime.fromtimestamp(x, tz=timezone.utc).strftime(fmt)
+                for x in values]
+
+
+class ScatterBaseParameterSetter(CommonParameterSetter):
+    CAT_LEGEND_LABEL = "Categorical legend"
+    NUM_LEGEND_LABEL = "Numerical legend"
+    NUM_LEGEND_SETTING = {
+        Updater.SIZE_LABEL: (range(4, 50), 11),
+        Updater.IS_ITALIC_LABEL: (None, False),
+    }
+
+    def __init__(self, master):
+        super().__init__()
+        self.master = master
+        self.cat_legend_settings = {}
+        self.num_legend_settings = {}
+
+    def update_setters(self):
+        self.initial_settings = {
+            self.LABELS_BOX: {
+                self.FONT_FAMILY_LABEL: self.FONT_FAMILY_SETTING,
+                self.TITLE_LABEL: self.FONT_SETTING,
+                self.LABEL_LABEL: self.FONT_SETTING,
+                self.CAT_LEGEND_LABEL: self.FONT_SETTING,
+                self.NUM_LEGEND_LABEL: self.NUM_LEGEND_SETTING,
+            },
+            self.ANNOT_BOX: {
+                self.TITLE_LABEL: {self.TITLE_LABEL: ("", "")},
+            }
+        }
+
+        def update_cat_legend(**settings):
+            self.cat_legend_settings.update(**settings)
+            Updater.update_legend_font(self.cat_legend_items, **settings)
+
+        def update_num_legend(**settings):
+            self.num_legend_settings.update(**settings)
+            Updater.update_num_legend_font(self.num_legend, **settings)
+
+        labels = self.LABELS_BOX
+        self._setters[labels][self.CAT_LEGEND_LABEL] = update_cat_legend
+        self._setters[labels][self.NUM_LEGEND_LABEL] = update_num_legend
+
+    @property
+    def title_item(self):
+        return self.master.plot_widget.getPlotItem().titleLabel
+
+    @property
+    def cat_legend_items(self):
+        items = self.master.color_legend.items
+        if items and items[0] and isinstance(items[0][0], PaletteItemSample):
+            items = []
+        return itertools.chain(self.master.shape_legend.items, items)
+
+    @property
+    def num_legend(self):
+        items = self.master.color_legend.items
+        if items and items[0] and isinstance(items[0][0], PaletteItemSample):
+            return self.master.color_legend
+        return None
+
+    @property
+    def labels(self):
+        return self.master.labels
+
+
 class OWScatterPlotBase(gui.OWComponent, QObject):
     """
     Provide a graph component for widgets that show any kind of point plot
@@ -397,7 +555,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
 
     COLOR_NOT_SUBSET = (128, 128, 128, 0)
     COLOR_SUBSET = (128, 128, 128, 255)
-    COLOR_DEFAULT = (128, 128, 128, 0)
+    COLOR_DEFAULT = (128, 128, 128, 255)
 
     MAX_VISIBLE_LABELS = 500
 
@@ -406,10 +564,12 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         gui.OWComponent.__init__(self, scatter_widget)
 
         self.subset_is_shown = False
+        self.jittering_suspended = False
 
         self.view_box = view_box(self)
+        _axis = {"left": AxisItem("left"), "bottom": AxisItem("bottom")}
         self.plot_widget = pg.PlotWidget(viewBox=self.view_box, parent=parent,
-                                         background="w")
+                                         background="w", axisItems=_axis)
         self.plot_widget.hideAxis("left")
         self.plot_widget.hideAxis("bottom")
         self.plot_widget.getPlotItem().buttonsHidden = True
@@ -422,7 +582,8 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         self.labels = []
 
         self.master = scatter_widget
-        self._create_drag_tooltip(self.plot_widget.scene())
+        tooltip = self._create_drag_tooltip()
+        self.view_box.setDragTooltip(tooltip)
 
         self.selection = None  # np.ndarray
 
@@ -453,21 +614,25 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
 
         self.timer = None
 
+        self.parameter_setter = ScatterBaseParameterSetter(self)
+
     def _create_legend(self, anchor):
         legend = LegendItem()
         legend.setParentItem(self.plot_widget.getViewBox())
         legend.restoreAnchor(anchor)
         return legend
 
-    def _create_drag_tooltip(self, scene):
+    def _create_drag_tooltip(self):
         tip_parts = [
-            (Qt.ShiftModifier, "Shift: 添加组"),
-            (Qt.ShiftModifier + Qt.ControlModifier,
-             "Shift-{}: 追加到组".
+            (Qt.ControlModifier,
+             "{}: 添加到组".
              format("Cmd" if sys.platform == "darwin" else "Ctrl")),
+            (Qt.ShiftModifier, "Shift: 添加组"),
             (Qt.AltModifier, "Alt: 删除")
         ]
-        all_parts = ", ".join(part for _, part in tip_parts)
+        all_parts = "<center>" + \
+                    ", ".join(part for _, part in tip_parts) + \
+                    "</center>"
         self.tiptexts = {
             int(modifier): all_parts.replace(part, "<b>{}</b>".format(part))
             for modifier, part in tip_parts
@@ -476,40 +641,50 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
 
         self.tip_textitem = text = QGraphicsTextItem()
         # Set to the longest text
-        text.setHtml(self.tiptexts[Qt.ShiftModifier + Qt.ControlModifier])
+        text.setHtml(self.tiptexts[Qt.ControlModifier])
         text.setPos(4, 2)
         r = text.boundingRect()
+        text.setTextWidth(r.width())
         rect = QGraphicsRectItem(0, 0, r.width() + 8, r.height() + 4)
         rect.setBrush(QColor(224, 224, 224, 212))
         rect.setPen(QPen(Qt.NoPen))
         self.update_tooltip()
 
-        scene.drag_tooltip = scene.createItemGroup([rect, text])
-        scene.drag_tooltip.hide()
+        tooltip_group = QGraphicsItemGroup()
+        tooltip_group.addToGroup(rect)
+        tooltip_group.addToGroup(text)
+        return tooltip_group
 
     def update_tooltip(self, modifiers=Qt.NoModifier):
-        modifiers &= Qt.ShiftModifier + Qt.ControlModifier + Qt.AltModifier
-        text = self.tiptexts.get(int(modifiers), self.tiptexts[0])
-        self.tip_textitem.setHtml(text + self._get_jittering_tooltip())
+        text = self.tiptexts[0]
+        for mod in [Qt.ControlModifier,
+                    Qt.ShiftModifier,
+                    Qt.AltModifier]:
+            if modifiers & mod:
+                text = self.tiptexts.get(int(mod))
+                break
+        self.tip_textitem.setHtml(text)
 
-    def _get_jittering_tooltip(self):
-        warn_jittered = ""
-        if self.jitter_size:
-            warn_jittered = \
-                '<br/><br/>' \
-                '<span style="background-color: red; color: white; ' \
-                'font-weight: 500;">' \
-                '&nbsp;Warning: Selection is applied to unjittered data&nbsp;' \
-                '</span>'
-        return warn_jittered
+    def suspend_jittering(self):
+        if self.jittering_suspended:
+            return
+        self.jittering_suspended = True
+        if self.jitter_size != 0:
+            self.update_jittering()
+
+    def unsuspend_jittering(self):
+        if not self.jittering_suspended:
+            return
+        self.jittering_suspended = False
+        if self.jitter_size != 0:
+            self.update_jittering()
 
     def update_jittering(self):
-        self.update_tooltip()
         x, y = self.get_coordinates()
-        if x is None or not len(x) or self.scatterplot_item is None:
+        if x is None or len(x) == 0 or self.scatterplot_item is None:
             return
-        self._update_plot_coordinates(self.scatterplot_item, x, y)
-        self._update_plot_coordinates(self.scatterplot_item_sel, x, y)
+        self.scatterplot_item.setCoordinates(x, y)
+        self.scatterplot_item_sel.setCoordinates(x, y)
         self.update_labels()
 
     # TODO: Rename to remove_plot_items
@@ -673,7 +848,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         Display coordinates to random positions within ellipses with
         radiuses of `self.jittter_size` percents of spans
         """
-        if self.jitter_size == 0:
+        if self.jitter_size == 0 or self.jittering_suspended:
             return x, y
         return self._jitter_data(x, y)
 
@@ -689,24 +864,6 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         return (x + magnitude * span_x * rs * np.cos(phis),
                 y + magnitude * span_y * rs * np.sin(phis))
 
-    def _update_plot_coordinates(self, plot, x, y):
-        """
-        Change the coordinates of points while keeping other properites
-
-        Note. Pyqtgraph does not offer a method for this: setting coordinates
-        invalidates other data. We therefore retrieve the data to set it
-        together with the coordinates. Pyqtgraph also does not offer a
-        (documented) method for retrieving the data, yet using
-        `plot.data[prop]` looks reasonably safe. The alternative, calling
-        update for every property would essentially reset the graph, which
-        can be time consuming.
-        """
-        data = dict(x=x, y=y)
-        for prop in ('pen', 'brush', 'size', 'symbol', 'data',
-                     'sourceRect', 'targetRect'):
-            data[prop] = plot.data[prop]
-        plot.setData(**data)
-
     def update_coordinates(self):
         """
         Trigger the update of coordinates while keeping other features intact.
@@ -718,8 +875,10 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         the complete update by calling `reset_graph` instead of this method.
         """
         x, y = self.get_coordinates()
-        if x is None or not len(x):
+        if x is None or len(x) == 0:
             return
+
+        self._reset_view(x, y)
         if self.scatterplot_item is None:
             if self.sample_indices is None:
                 indices = np.arange(self.n_valid)
@@ -732,12 +891,11 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             self.plot_widget.addItem(self.scatterplot_item_sel)
             self.plot_widget.addItem(self.scatterplot_item)
         else:
-            self._update_plot_coordinates(self.scatterplot_item, x, y)
-            self._update_plot_coordinates(self.scatterplot_item_sel, x, y)
+            self.scatterplot_item.setCoordinates(x, y)
+            self.scatterplot_item_sel.setCoordinates(x, y)
             self.update_labels()
 
         self.update_density()  # Todo: doesn't work: try MDS with density on
-        self._reset_view(x, y)
 
     # Sizes
     def get_sizes(self):
@@ -899,15 +1057,19 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         """
         color = self.plot_widget.palette().color(OWPalette.Data)
         pen = [_make_pen(color, 1.5)] * self.n_shown  # use a single QPen instance
+
+        # Prepare all brushes; we use the first two or the last
+        brushes = []
+        for c in (self.COLOR_SUBSET, self.COLOR_NOT_SUBSET, self.COLOR_DEFAULT):
+            color = QColor(*c)
+            if color.alpha():
+                color.setAlpha(self.alpha_value)
+            brushes.append(QBrush(color))
+
         if subset is not None:
-            brush = np.where(
-                subset,
-                *(QBrush(QColor(*col))
-                  for col in (self.COLOR_SUBSET, self.COLOR_NOT_SUBSET)))
+            brush = np.where(subset, *brushes[:2])
         else:
-            color = QColor(*self.COLOR_DEFAULT)
-            color.setAlpha(self.alpha_value)
-            brush = [QBrush(color)] * self.n_shown  # use a single QBrush instance
+            brush = brushes[-1:] * self.n_shown  # use a single QBrush instance
         return pen, brush
 
     def _get_continuous_colors(self, c_data, subset):
@@ -937,9 +1099,9 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         # Reuse pens and brushes with the same colors because PyQtGraph then
         # builds smaller pixmap atlas, which makes the drawing faster
 
-        def reuse(cache, fn, *args):
+        def reuse(cache, fun, *args):
             if args not in cache:
-                cache[args] = fn(args)
+                cache[args] = fun(args)
             return cache[args]
 
         def create_pen(col):
@@ -953,7 +1115,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
 
         if subset is not None:
             brush[:, 3] = 0
-            brush[subset, 3] = 255
+            brush[subset, 3] = self.alpha_value
 
         cached_brushes = {}
         brush = np.array([reuse(cached_brushes, create_brush, *col)
@@ -984,7 +1146,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         pens = np.array(
             [_make_pen(col.darker(self.DarkerValue), 1.5) for col in colors])
         pen = pens[c_data]
-        if subset is None and self.alpha_value < 255:
+        if self.alpha_value < 255:
             for col in colors:
                 col.setAlpha(self.alpha_value)
         brushes = np.array([QBrush(col) for col in colors])
@@ -997,7 +1159,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
 
     def update_colors(self):
         """
-        Trigger an update of point sizes
+        Trigger an update of point colors
 
         The method calls `self.get_colors`, which in turn calls the widget's
         `get_color_data` to get the indices in the pallette. `get_colors`
@@ -1009,6 +1171,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             pen_data, brush_data = self.get_colors()
             self.scatterplot_item.setPen(pen_data, update=False, mask=None)
             self.scatterplot_item.setBrush(brush_data, mask=None)
+        self.update_z_values()
         self.update_legends()
         self.update_density()
 
@@ -1026,17 +1189,25 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             self.plot_widget.removeItem(self.density_img)
             self.density_img = None
         if self.class_density and self.scatterplot_item is not None:
+            c_data = self.master.get_color_data()
+            if c_data is None:
+                return
+            visible_c_data = self._filter_visible(c_data)
+            mask = np.bitwise_and(np.isfinite(visible_c_data),
+                                  visible_c_data < MAX_COLORS - 1)
+            pens = self.scatterplot_item.data['pen']
             rgb_data = [
                 pen.color().getRgb()[:3] if pen is not None else (255, 255, 255)
-                for pen in self.scatterplot_item.data['pen']]
+                for known, pen in zip(mask, pens)
+                if known]
             if len(set(rgb_data)) <= 1:
                 return
             [min_x, max_x], [min_y, max_y] = self.view_box.viewRange()
             x_data, y_data = self.scatterplot_item.getData()
             self.density_img = classdensity.class_density_image(
                 min_x, max_x, min_y, max_y, self.resolution,
-                x_data, y_data, rgb_data)
-            self.plot_widget.addItem(self.density_img)
+                x_data[mask], y_data[mask], rgb_data)
+            self.plot_widget.addItem(self.density_img, ignoreBounds=True)
 
     def update_selection_colors(self):
         """
@@ -1053,6 +1224,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         pen, brush = self.get_colors_sel()
         self.scatterplot_item_sel.setPen(pen, update=False, mask=None)
         self.scatterplot_item_sel.setBrush(brush, mask=None)
+        self.update_z_values()
 
     def get_colors_sel(self):
         """
@@ -1073,14 +1245,14 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             if sels == 1:
                 pen = np.where(
                     self._filter_visible(self.selection),
-                    _make_pen(QColor(255, 190, 0, 255), SELECTION_WIDTH + 1),
+                    _make_pen(QColor(255, 190, 0, 255), SELECTION_WIDTH),
                     nopen)
             else:
                 palette = colorpalettes.LimitedDiscretePalette(
                     number_of_colors=sels + 1)
                 pen = np.choose(
                     self._filter_visible(self.selection),
-                    [nopen] + [_make_pen(palette[i], SELECTION_WIDTH + 1)
+                    [nopen] + [_make_pen(palette[i], SELECTION_WIDTH)
                                for i in range(sels)])
         return pen, [QBrush(QColor(255, 255, 255, 0))] * self.n_shown
 
@@ -1132,6 +1304,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             ti.setPos(xp, yp)
             self.plot_widget.addItem(ti)
             self.labels.append(ti)
+            ti.setFont(self.parameter_setter.label_font)
 
     def _signal_too_many_labels(self, too_many):
         if self._too_many_labels != too_many:
@@ -1217,6 +1390,52 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             self.scatterplot_item.setSymbol(shape_data)
         self.update_legends()
 
+    def update_z_values(self):
+        """
+        Set z-values for point in the plot
+
+        The order is as follows:
+        - selected points that are also in the subset on top,
+        - followed by selected points,
+        - followed by points from the subset,
+        - followed by the rest.
+        Within each of these four groups, points are ordered by their colors.
+
+        Points with less frequent colors are above those with more frequent.
+        The points for which the value for the color is missing are at the
+        bottom of their respective group.
+        """
+        if not self.scatterplot_item:
+            return
+
+        subset = self.master.get_subset_mask()
+        c_data = self.master.get_color_data()
+        if subset is None and self.selection is None and c_data is None:
+            self.scatterplot_item.setZ(None)
+            return
+
+        z = np.zeros(self.n_shown)
+
+        if subset is not None:
+            subset = self._filter_visible(subset)
+            z[subset] += 1000
+
+        if self.selection is not None:
+            z[self._filter_visible(self.selection) != 0] += 2000
+
+        if c_data is not None:
+            c_nan = np.isnan(c_data)
+            vis_data = self._filter_visible(c_data)
+            vis_nan = np.isnan(vis_data)
+            z[vis_nan] -= 999
+            if not self.master.is_continuous_color():
+                dist = np.bincount(c_data[~c_nan].astype(int))
+                vis_knowns = vis_data[~vis_nan].astype(int)
+                argdist = np.argsort(dist)
+                z[~vis_nan] -= argdist[vis_knowns]
+
+        self.scatterplot_item.setZ(z)
+
     def update_grid_visibility(self):
         """Show or hide the grid"""
         self.plot_widget.showGrid(x=self.show_grid, y=self.show_grid)
@@ -1252,6 +1471,10 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
             else:
                 self._update_color_legend(color_labels)
         self.update_legend_visibility()
+        Updater.update_legend_font(self.parameter_setter.cat_legend_items,
+                                   **self.parameter_setter.cat_legend_settings)
+        Updater.update_num_legend_font(self.parameter_setter.num_legend,
+                                       **self.parameter_setter.num_legend_settings)
 
     def _update_shape_legend(self, labels):
         self.shape_legend.clear()
@@ -1296,7 +1519,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         for color, label, symbol in zip(colors, labels, symbols):
             color = QColor(*color)
             pen = _make_pen(color.darker(self.DarkerValue), 1.5)
-            color.setAlpha(255 if self.subset_is_shown else self.alpha_value)
+            color.setAlpha(self.alpha_value)
             brush = QBrush(color)
             legend.addItem(
                 SymbolItemSample(pen=pen, brush=brush, size=10, symbol=symbol),
@@ -1350,12 +1573,12 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         if self.selection is None:
             self.selection = np.zeros(self.n_valid, dtype=np.uint8)
         keys = QApplication.keyboardModifiers()
-        if keys & Qt.AltModifier:
-            self.selection_remove(indices)
-        elif keys & Qt.ShiftModifier and keys & Qt.ControlModifier:
+        if keys & Qt.ControlModifier:
             self.selection_append(indices)
         elif keys & Qt.ShiftModifier:
             self.selection_new_group(indices)
+        elif keys & Qt.AltModifier:
+            self.selection_remove(indices)
         else:
             self.selection_select(indices)
 
@@ -1365,7 +1588,7 @@ class OWScatterPlotBase(gui.OWComponent, QObject):
         self._update_after_selection()
 
     def selection_append(self, indices):
-        self.selection[indices] = np.max(self.selection)
+        self.selection[indices] = max(np.max(self.selection), 1)
         self._update_after_selection()
 
     def selection_new_group(self, indices):
