@@ -10,8 +10,8 @@ import sklearn.metrics
 
 from AnyQt.QtWidgets import (
     QGraphicsScene, QGraphicsWidget, QGraphicsGridLayout,
-    QGraphicsSimpleTextItem, QGraphicsRectItem, QStyleOptionGraphicsItem,
-    QSizePolicy, QWidget, QWIDGETSIZE_MAX, QVBoxLayout
+    QGraphicsRectItem, QStyleOptionGraphicsItem, QSizePolicy, QWidget,
+    QVBoxLayout, QGraphicsSimpleTextItem, QWIDGETSIZE_MAX,
 )
 from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QFontMetrics, QPalette
 from AnyQt.QtCore import Qt, QEvent, QRectF, QSizeF, QSize, QPointF
@@ -20,6 +20,7 @@ from AnyQt.QtCore import pyqtSignal as Signal
 import pyqtgraph as pg
 
 import Orange.data
+from Orange.data.util import get_unique_names
 import Orange.distance
 import Orange.misc
 from Orange.data import Table, Domain
@@ -31,10 +32,10 @@ from Orange.widgets.utils import itemmodels
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_Chinese_NAME)
 from Orange.widgets.utils.graphicstextlist import TextListWidget
+from Orange.widgets.utils.graphicslayoutitem import SimpleLayoutItem
 from Orange.widgets.utils.sql import check_sql_input
-from Orange.widgets.unsupervised.owhierarchicalclustering import \
-    WrapperLayoutItem
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Msg, Input, Output
 
 
@@ -58,6 +59,7 @@ class ValidationError(InputValidationError):
 class OWSilhouettePlot(widget.OWWidget):
     name = "轮廓图(Silhouette Plot)"
     description = "视觉评估聚类质量和聚类成员的程度。"
+    category = 'visualize'
 
     icon = "icons/SilhouettePlot.svg"
     priority = 300
@@ -88,7 +90,6 @@ class OWSilhouettePlot(widget.OWWidget):
     #: A fixed size for an instance bar
     bar_size = settings.Setting(3)
     #: Add silhouette scores to output data
-    add_scores = settings.Setting(False)
     auto_commit = settings.Setting(True)
 
     pending_selection = settings.Setting(None, schema_only=True)
@@ -133,6 +134,9 @@ class OWSilhouettePlot(widget.OWWidget):
         self._silhouette = None  # type: Optional[np.ndarray]
         self._silplot = None     # type: Optional[SilhouettePlot]
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         controllayout = self.controlArea.layout()
         assert isinstance(controllayout, QVBoxLayout)
         self._distances_gui_box = distbox = gui.widgetBox(
@@ -147,7 +151,7 @@ class OWSilhouettePlot(widget.OWWidget):
         box = gui.vBox(self.controlArea, "聚类标签")
         self.cluster_var_cb = gui.comboBox(
             box, self, "cluster_var_idx", contentsLength=14, addSpace=4,
-            callback=self._invalidate_scores
+            searchable=True, callback=self._invalidate_scores
         )
         gui.checkBox(
             box, self, "group_by_cluster", "按聚类分组(Group by cluster)",
@@ -175,16 +179,12 @@ class OWSilhouettePlot(widget.OWWidget):
 
         gui.rubber(self.controlArea)
 
-        gui.separator(self.buttonsArea)
-        box = gui.vBox(self.buttonsArea, "输出")
-        # Thunk the call to commit to call conditional commit
-        gui.checkBox(box, self, "add_scores", "添加轮廓系数",
-                     callback=lambda: self.commit())
+        box = gui.vBox(self.buttonsArea, box=True)
         gui.auto_send(box, self, "auto_commit", box=False)
         # Ensure that the controlArea is not narrower than buttonsArea
         self.controlArea.layout().addWidget(self.buttonsArea)
 
-        self.scene = QGraphicsScene()
+        self.scene = QGraphicsScene(self)
         self.view = StickyGraphicsView(self.scene)
         self.view.setRenderHint(QPainter.Antialiasing, True)
         self.view.setAlignment(Qt.AlignTop | Qt.AlignLeft)
@@ -241,6 +241,9 @@ class OWSilhouettePlot(widget.OWWidget):
             self.data = data
 
     def handleNewSignals(self):
+        summary = len(self.data) if self.data else self.info.NoInput
+        details = format_summary_details(self.data) if self.data else ""
+        self.info.set_input_summary(summary, details)
         if not self._is_empty():
             self._update()
             self._replot()
@@ -498,29 +501,32 @@ class OWSilhouettePlot(widget.OWWidget):
             else:
                 scores = self._silhouette
 
-            silhouette_var = None
-            if self.add_scores:
-                var = self.cluster_var_model[self.cluster_var_idx]
-                silhouette_var = Orange.data.ContinuousVariable(
-                    "Silhouette ({})".format(escape(var.name)))
-                domain = Orange.data.Domain(
-                    self.data.domain.attributes,
-                    self.data.domain.class_vars,
-                    self.data.domain.metas + (silhouette_var, ))
-                data = self.data.transform(domain)
-            else:
-                domain = self.data.domain
-                data = self.data
+            var = self.cluster_var_model[self.cluster_var_idx]
+            
+            domain = self.data.domain
+            proposed = "Silhouette ({})".format(escape(var.name))
+            names = [var.name for var in itertools.chain(domain.attributes,
+                                                         domain.class_vars,
+                                                         domain.metas)]
+            unique = get_unique_names(names, proposed)
+            silhouette_var = Orange.data.ContinuousVariable(unique)
+            domain = Orange.data.Domain(
+                domain.attributes,
+                domain.class_vars,
+                domain.metas + (silhouette_var, ))
+            data = self.data.transform(domain)
 
             if np.count_nonzero(selectedmask):
                 selected = self.data.from_table(
                     domain, self.data, np.flatnonzero(selectedmask))
 
-            if self.add_scores:
-                if selected is not None:
-                    selected[:, silhouette_var] = np.c_[scores[selectedmask]]
-                data[:, silhouette_var] = np.c_[scores]
+            if selected is not None:
+                selected[:, silhouette_var] = np.c_[scores[selectedmask]]
+            data[:, silhouette_var] = np.c_[scores]
 
+        summary = len(selected) if selected else self.info.NoOutput
+        details = format_summary_details(selected) if selected else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(selected)
         self.Outputs.annotated_data.send(create_annotated_table(data, indices))
 
@@ -567,8 +573,8 @@ class SilhouettePlot(QGraphicsWidget):
         self.__hoveredItem = None
         self.__topScale = None     # type: Optional[pg.AxisItem]
         self.__bottomScale = None  # type: Optional[pg.AxisItem]
+        self.__layout.setColumnSpacing(0, 1.)
         self.setLayout(self.__layout)
-        self.layout().setColumnSpacing(0, 1.)
         self.setFocusPolicy(Qt.StrongFocus)
 
     def setScores(self, scores, labels, values, colors, rownames=None):
@@ -623,7 +629,8 @@ class SilhouettePlot(QGraphicsWidget):
         if names is not None:
             names = np.asarray(names, dtype=object)
 
-        layout = self.layout()
+        layout = self.__layout
+        assert layout is self.layout()
 
         font = self.font()
         font.setPixelSize(self.__barHeight)
@@ -631,7 +638,7 @@ class SilhouettePlot(QGraphicsWidget):
         for i, grp in enumerate(self.__groups):
             grp.rownames = names[grp.indices] if names is not None else None
             item = layout.itemAt(i + 1, 3)
-
+            assert isinstance(item, TextListWidget)
             if grp.rownames is not None:
                 metrics = QFontMetrics(self.font())
                 rownames = [metrics.elidedText(rowname, Qt.ElideRight, ROW_NAMES_WIDTH)
@@ -653,7 +660,7 @@ class SilhouettePlot(QGraphicsWidget):
             for baritem, tooltip in zip(baritems, tooltips):
                 baritem.setToolTip(tooltip)
 
-        self.layout().activate()
+        layout.activate()
 
     def setRowNamesVisible(self, visible):
         if self.__rowNamesVisible != visible:
@@ -717,7 +724,9 @@ class SilhouettePlot(QGraphicsWidget):
                          pen=axispen)
         ax.setRange(smin, smax)
         self.__topScale = ax
-        self.layout().addItem(ax, 0, 2)
+        layout = self.__layout
+        assert layout is self.layout()
+        layout.addItem(ax, 0, 2)
 
         for i, group in enumerate(self.__groups):
             silhouettegroup = BarPlotItem(parent=self)
@@ -727,15 +736,21 @@ class SilhouettePlot(QGraphicsWidget):
             silhouettegroup.setPlotData(group.scores)
             silhouettegroup.setPreferredBarSize(self.__barHeight)
             silhouettegroup.setData(0, group.indices)
-            self.layout().addItem(silhouettegroup, i + 1, 2)
+            layout.addItem(silhouettegroup, i + 1, 2)
 
             if group.label:
-                self.layout().addItem(Line(orientation=Qt.Vertical), i + 1, 1)
-                label = QGraphicsSimpleTextItem(self)
-                label.setText("{} ({})".format(escape(group.label),
-                                               len(group.scores)))
-                item = WrapperLayoutItem(label, Qt.Vertical, parent=self)
-                self.layout().addItem(item, i + 1, 0, Qt.AlignCenter)
+                layout.addItem(Line(orientation=Qt.Vertical), i + 1, 1)
+                label = QGraphicsSimpleTextItem(
+                    "{} ({})".format(group.label, len(group.scores)), self
+                )
+                label.setRotation(-90)
+                item = SimpleLayoutItem(
+                    label,
+                    anchor=(0., 1.0),
+                    anchorItem=(0., 0.),
+                )
+                item.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+                layout.addItem(item, i + 1, 0, Qt.AlignCenter)
 
             textlist = TextListWidget(self, font=font)
             sp = textlist.sizePolicy()
@@ -748,13 +763,13 @@ class SilhouettePlot(QGraphicsWidget):
             else:
                 textlist.setVisible(False)
 
-            self.layout().addItem(textlist, i + 1, 3)
+            layout.addItem(textlist, i + 1, 3)
 
         ax = pg.AxisItem(parent=self, orientation="bottom", maxTickLength=7,
                          pen=axispen)
         ax.setRange(smin, smax)
         self.__bottomScale = ax
-        self.layout().addItem(ax, len(self.__groups) + 1, 2)
+        layout.addItem(ax, len(self.__groups) + 1, 2)
 
     def topScaleItem(self):
         # type: () -> Optional[QGraphicsWidget]
