@@ -5,6 +5,8 @@ import numpy as np
 from AnyQt.QtCore import QSize, Signal
 from AnyQt.QtWidgets import QApplication
 
+from orangewidget.utils.visual_settings_dlg import VisualSettingsDialog
+
 from Orange.data import (
     Table, ContinuousVariable, Domain, Variable, StringVariable
 )
@@ -16,18 +18,20 @@ from Orange.widgets import gui, report
 from Orange.widgets.settings import (
     Setting, ContextSetting, DomainContextHandler, SettingProvider
 )
+from Orange.widgets.utils import colorpalettes
 from Orange.widgets.utils.annotated_data import (
     create_annotated_table, ANNOTATED_DATA_SIGNAL_NAME, 
     create_groups_table, ANNOTATED_DATA_SIGNAL_Chinese_NAME
 )
 from Orange.widgets.utils.plot import OWPlotGUI
 from Orange.widgets.utils.sql import check_sql_input
-from Orange.widgets.visualize.owscatterplotgraph import OWScatterPlotBase
+from Orange.widgets.utils.state_summary import format_summary_details
+from Orange.widgets.visualize.owscatterplotgraph import (
+    OWScatterPlotBase, MAX_COLORS
+)
 from Orange.widgets.visualize.utils.component import OWGraphWithAnchors
 from Orange.widgets.widget import OWWidget, Input, Output, Msg
 
-# maximum number of colors (including Other)
-MAX_COLORS = 11
 
 # maximum number of shapes (including Other)
 MAX_SHAPES = len(OWScatterPlotBase.CurveSymbols) - 1
@@ -222,7 +226,17 @@ class OWProjectionWidgetBase(OWWidget, openclass=True):
         This method must be overridden if the widget offers coloring that is
         not based on attribute values.
         """
-        return self.attr_color and self.attr_color.palette
+        attr = self.attr_color
+        if not attr:
+            return None
+        palette = attr.palette
+        if attr.is_discrete and len(attr.values) >= MAX_COLORS:
+            values = self.get_color_labels()
+            colors = [palette.palette[attr.to_val(value)]
+                      for value in values[:-1]] + [[192, 192, 192]]
+
+            palette = colorpalettes.DiscretePalette.from_colors(colors)
+        return palette
 
     def can_draw_density(self):
         """
@@ -237,6 +251,7 @@ class OWProjectionWidgetBase(OWWidget, openclass=True):
 
     def colors_changed(self):
         self.graph.update_colors()
+        self._update_opacity_warning()
         self.cb_class_density.setEnabled(self.can_draw_density())
 
     # Labels
@@ -365,9 +380,12 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
             "input data")
         subset_independent = Msg(
             "No subset data instances appear in input data")
+        transparent_subset = Msg(
+            "Increase opacity if subset is difficult to see")
 
     settingsHandler = DomainContextHandler()
     selection = Setting(None, schema_only=True)
+    visual_settings = Setting({}, schema_only=True)
     auto_commit = Setting(True)
 
     GRAPH_CLASS = OWScatterPlotBase
@@ -389,6 +407,7 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
         self.input_changed.connect(self.set_input_summary)
         self.output_changed.connect(self.set_output_summary)
         self.setup_gui()
+        VisualSettingsDialog(self, self.graph.parameter_setter.initial_settings)
 
     # GUI
     def setup_gui(self):
@@ -431,13 +450,10 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
     def set_data(self, data):
         data_existed = self.data is not None
         effective_data = self.effective_data if data_existed else None
-        same_domain = (data_existed and data is not None and
-                       data.domain.checksum() == self.data.domain.checksum())
         self.closeContext()
         self.data = data
         self.check_data()
-        if not same_domain:
-            self.init_attr_values()
+        self.init_attr_values()
         self.openContext(self.data)
         self._invalidated = not (
             data_existed and self.data is not None and
@@ -461,7 +477,6 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
     @check_sql_input
     def set_subset_data(self, subset):
         self.subset_data = subset
-        self.controls.graph.alpha_value.setEnabled(subset is None)
 
     def handleNewSignals(self):
         self._handle_subset_data()
@@ -470,6 +485,7 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
             self.setup_plot()
         else:
             self.graph.update_point_props()
+        self._update_opacity_warning()
         self.unconditional_commit()
 
     def _handle_subset_data(self):
@@ -485,20 +501,26 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
             elif self.subset_indices - ids:
                 self.Warning.subset_not_subset()
 
+    def _update_opacity_warning(self):
+        self.Warning.transparent_subset(
+            shown=self.subset_indices and self.graph.alpha_value < 128)
+
     def set_input_summary(self, data):
-        summary = str(len(data)) if data else self.info.NoInput
-        self.info.set_input_summary(summary)
+        summary = len(data) if data else self.info.NoInput
+        detail = format_summary_details(data) if data else ""
+        self.info.set_input_summary(summary, detail)
 
     def set_output_summary(self, data):
-        summary = str(len(data)) if data else self.info.NoOutput
-        self.info.set_output_summary(summary)
+        summary = len(data) if data else self.info.NoOutput
+        detail = format_summary_details(data) if data else ""
+        self.info.set_output_summary(summary, detail)
 
     def get_subset_mask(self):
         if not self.subset_indices:
             return None
         valid_data = self.data[self.valid_data]
         return np.fromiter((ex.id in self.subset_indices for ex in valid_data),
-                           dtype=np.bool, count=len(valid_data))
+                           dtype=bool, count=len(valid_data))
 
     # Plot
     def get_embedding(self):
@@ -540,6 +562,8 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
             self.__pending_selection = None
             self.graph.selection = selection
             self.graph.update_selection_colors()
+            if self.graph.label_only_selected:
+                self.graph.update_labels()
 
     def selection_changed(self):
         sel = None if self.data and isinstance(self.data, SqlTable) \
@@ -587,10 +611,16 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
 
     @staticmethod
     def _get_annotated_data(data, group_sel, graph_sel):
+        if data is None:
+            return None
         if graph_sel is not None and np.max(graph_sel) > 1:
             return create_groups_table(data, group_sel)
         else:
-            return create_annotated_table(data, np.nonzero(group_sel)[0])
+            if group_sel is None:
+                mask = np.full((len(data), ), False)
+            else:
+                mask = np.nonzero(group_sel)[0]
+            return create_annotated_table(data, mask)
 
     # Report
     def send_report(self):
@@ -611,6 +641,11 @@ class OWDataProjectionWidget(OWProjectionWidgetBase, openclass=True):
             ("Jittering", self.graph.jitter_size != 0 and
              "{} %".format(self.graph.jitter_size))))
 
+    # Customize plot
+    def set_visual_settings(self, key, value):
+        self.graph.parameter_setter.set_parameter(key, value)
+        self.visual_settings[key] = value
+
     @staticmethod
     def _get_caption_var_name(var):
         return var.name if isinstance(var, Variable) else var
@@ -647,8 +682,8 @@ class OWAnchorProjectionWidget(OWDataProjectionWidget, openclass=True):
         proj_error = Msg("An error occurred while projecting data.\n{}")
 
     def __init__(self):
-        super().__init__()
         self.projector = self.projection = None
+        super().__init__()
         self.graph.view_box.started.connect(self._manual_move_start)
         self.graph.view_box.moved.connect(self._manual_move)
         self.graph.view_box.finished.connect(self._manual_move_finish)
@@ -708,10 +743,18 @@ class OWAnchorProjectionWidget(OWDataProjectionWidget, openclass=True):
     def _get_projection_data(self):
         if self.data is None or self.projection is None:
             return None
+        proposed = [a.name for a in self.projection.domain.attributes]
+        names = get_unique_names(self.data.domain, proposed)
+
+        if proposed != names:
+            attributes = tuple([attr.copy(name=name) for name, attr in
+                                zip(names, self.projection.domain.attributes)])
+        else:
+            attributes = self.projection.domain.attributes
         return self.data.transform(
             Domain(self.data.domain.attributes,
                    self.data.domain.class_vars,
-                   self.data.domain.metas + self.projection.domain.attributes))
+                   self.data.domain.metas + attributes))
 
     def commit(self):
         super().commit()
@@ -720,7 +763,9 @@ class OWAnchorProjectionWidget(OWDataProjectionWidget, openclass=True):
     def send_components(self):
         components = None
         if self.data is not None and self.projection is not None:
-            meta_attrs = [StringVariable(name='component')]
+            proposed = [var.name for var in self.effective_variables]
+            comp_name = get_unique_names(proposed, 'component')
+            meta_attrs = [StringVariable(name=comp_name)]
             domain = Domain(self.effective_variables, metas=meta_attrs)
             components = Table(domain, self._send_components_x(),
                                metas=self._send_components_metas())
